/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_S390_ENUMS_H
#define LLVM_IR_INTRINSIC_S390_ENUMS_H
namespace llvm::Intrinsic {
enum S390Intrinsics : unsigned {
// Enum values for intrinsics.
    s390_bdepg = 12678,                                // llvm.s390.bdepg (IntrinsicsSystemZ.td:499)
    s390_bextg,                                // llvm.s390.bextg (IntrinsicsSystemZ.td:502)
    s390_efpc,                                 // llvm.s390.efpc (IntrinsicsSystemZ.td:492)
    s390_etnd,                                 // llvm.s390.etnd (IntrinsicsSystemZ.td:211)
    s390_lcbb,                                 // llvm.s390.lcbb (IntrinsicsSystemZ.td:228)
    s390_ntstg,                                // llvm.s390.ntstg (IntrinsicsSystemZ.td:214)
    s390_ppa_txassist,                         // llvm.s390.ppa.txassist (IntrinsicsSystemZ.td:217)
    s390_sfpc,                                 // llvm.s390.sfpc (IntrinsicsSystemZ.td:490)
    s390_tabort,                               // llvm.s390.tabort (IntrinsicsSystemZ.td:205)
    s390_tbegin,                               // llvm.s390.tbegin (IntrinsicsSystemZ.td:195)
    s390_tbegin_nofloat,                       // llvm.s390.tbegin.nofloat (IntrinsicsSystemZ.td:198)
    s390_tbeginc,                              // llvm.s390.tbeginc (IntrinsicsSystemZ.td:202)
    s390_tdc,                                  // llvm.s390.tdc (IntrinsicsSystemZ.td:495)
    s390_tend,                                 // llvm.s390.tend (IntrinsicsSystemZ.td:208)
    s390_vaccb,                                // llvm.s390.vaccb (IntrinsicsSystemZ.td:121)
    s390_vacccq,                               // llvm.s390.vacccq (IntrinsicsSystemZ.td:271)
    s390_vaccf,                                // llvm.s390.vaccf (IntrinsicsSystemZ.td:123)
    s390_vaccg,                                // llvm.s390.vaccg (IntrinsicsSystemZ.td:127)
    s390_vacch,                                // llvm.s390.vacch (IntrinsicsSystemZ.td:122)
    s390_vaccq,                                // llvm.s390.vaccq (IntrinsicsSystemZ.td:270)
    s390_vacq,                                 // llvm.s390.vacq (IntrinsicsSystemZ.td:269)
    s390_vaq,                                  // llvm.s390.vaq (IntrinsicsSystemZ.td:268)
    s390_vavgb,                                // llvm.s390.vavgb (IntrinsicsSystemZ.td:121)
    s390_vavgf,                                // llvm.s390.vavgf (IntrinsicsSystemZ.td:123)
    s390_vavgg,                                // llvm.s390.vavgg (IntrinsicsSystemZ.td:127)
    s390_vavgh,                                // llvm.s390.vavgh (IntrinsicsSystemZ.td:122)
    s390_vavglb,                               // llvm.s390.vavglb (IntrinsicsSystemZ.td:121)
    s390_vavglf,                               // llvm.s390.vavglf (IntrinsicsSystemZ.td:123)
    s390_vavglg,                               // llvm.s390.vavglg (IntrinsicsSystemZ.td:127)
    s390_vavglh,                               // llvm.s390.vavglh (IntrinsicsSystemZ.td:122)
    s390_vavglq,                               // llvm.s390.vavglq (IntrinsicsSystemZ.td:460)
    s390_vavgq,                                // llvm.s390.vavgq (IntrinsicsSystemZ.td:459)
    s390_vbperm,                               // llvm.s390.vbperm (IntrinsicsSystemZ.td:369)
    s390_vceqbs,                               // llvm.s390.vceqbs (IntrinsicsSystemZ.td:137)
    s390_vceqfs,                               // llvm.s390.vceqfs (IntrinsicsSystemZ.td:139)
    s390_vceqgs,                               // llvm.s390.vceqgs (IntrinsicsSystemZ.td:140)
    s390_vceqhs,                               // llvm.s390.vceqhs (IntrinsicsSystemZ.td:138)
    s390_vceqqs,                               // llvm.s390.vceqqs (IntrinsicsSystemZ.td:478)
    s390_vcfn,                                 // llvm.s390.vcfn (IntrinsicsSystemZ.td:440)
    s390_vchbs,                                // llvm.s390.vchbs (IntrinsicsSystemZ.td:137)
    s390_vchfs,                                // llvm.s390.vchfs (IntrinsicsSystemZ.td:139)
    s390_vchgs,                                // llvm.s390.vchgs (IntrinsicsSystemZ.td:140)
    s390_vchhs,                                // llvm.s390.vchhs (IntrinsicsSystemZ.td:138)
    s390_vchlbs,                               // llvm.s390.vchlbs (IntrinsicsSystemZ.td:137)
    s390_vchlfs,                               // llvm.s390.vchlfs (IntrinsicsSystemZ.td:139)
    s390_vchlgs,                               // llvm.s390.vchlgs (IntrinsicsSystemZ.td:140)
    s390_vchlhs,                               // llvm.s390.vchlhs (IntrinsicsSystemZ.td:138)
    s390_vchlqs,                               // llvm.s390.vchlqs (IntrinsicsSystemZ.td:480)
    s390_vchqs,                                // llvm.s390.vchqs (IntrinsicsSystemZ.td:479)
    s390_vcksm,                                // llvm.s390.vcksm (IntrinsicsSystemZ.td:276)
    s390_vclfnhs,                              // llvm.s390.vclfnhs (IntrinsicsSystemZ.td:428)
    s390_vclfnls,                              // llvm.s390.vclfnls (IntrinsicsSystemZ.td:432)
    s390_vcnf,                                 // llvm.s390.vcnf (IntrinsicsSystemZ.td:444)
    s390_vcrnfs,                               // llvm.s390.vcrnfs (IntrinsicsSystemZ.td:436)
    s390_verimb,                               // llvm.s390.verimb (IntrinsicsSystemZ.td:172)
    s390_verimf,                               // llvm.s390.verimf (IntrinsicsSystemZ.td:174)
    s390_verimg,                               // llvm.s390.verimg (IntrinsicsSystemZ.td:179)
    s390_verimh,                               // llvm.s390.verimh (IntrinsicsSystemZ.td:173)
    s390_veval,                                // llvm.s390.veval (IntrinsicsSystemZ.td:461)
    s390_vfaeb,                                // llvm.s390.vfaeb (IntrinsicsSystemZ.td:160)
    s390_vfaebs,                               // llvm.s390.vfaebs (IntrinsicsSystemZ.td:166)
    s390_vfaef,                                // llvm.s390.vfaef (IntrinsicsSystemZ.td:162)
    s390_vfaefs,                               // llvm.s390.vfaefs (IntrinsicsSystemZ.td:168)
    s390_vfaeh,                                // llvm.s390.vfaeh (IntrinsicsSystemZ.td:161)
    s390_vfaehs,                               // llvm.s390.vfaehs (IntrinsicsSystemZ.td:167)
    s390_vfaezb,                               // llvm.s390.vfaezb (IntrinsicsSystemZ.td:160)
    s390_vfaezbs,                              // llvm.s390.vfaezbs (IntrinsicsSystemZ.td:166)
    s390_vfaezf,                               // llvm.s390.vfaezf (IntrinsicsSystemZ.td:162)
    s390_vfaezfs,                              // llvm.s390.vfaezfs (IntrinsicsSystemZ.td:168)
    s390_vfaezh,                               // llvm.s390.vfaezh (IntrinsicsSystemZ.td:161)
    s390_vfaezhs,                              // llvm.s390.vfaezhs (IntrinsicsSystemZ.td:167)
    s390_vfcedbs,                              // llvm.s390.vfcedbs (IntrinsicsSystemZ.td:358)
    s390_vfcesbs,                              // llvm.s390.vfcesbs (IntrinsicsSystemZ.td:390)
    s390_vfchdbs,                              // llvm.s390.vfchdbs (IntrinsicsSystemZ.td:359)
    s390_vfchedbs,                             // llvm.s390.vfchedbs (IntrinsicsSystemZ.td:360)
    s390_vfchesbs,                             // llvm.s390.vfchesbs (IntrinsicsSystemZ.td:392)
    s390_vfchsbs,                              // llvm.s390.vfchsbs (IntrinsicsSystemZ.td:391)
    s390_vfeeb,                                // llvm.s390.vfeeb (IntrinsicsSystemZ.td:121)
    s390_vfeebs,                               // llvm.s390.vfeebs (IntrinsicsSystemZ.td:131)
    s390_vfeef,                                // llvm.s390.vfeef (IntrinsicsSystemZ.td:123)
    s390_vfeefs,                               // llvm.s390.vfeefs (IntrinsicsSystemZ.td:133)
    s390_vfeeh,                                // llvm.s390.vfeeh (IntrinsicsSystemZ.td:122)
    s390_vfeehs,                               // llvm.s390.vfeehs (IntrinsicsSystemZ.td:132)
    s390_vfeezb,                               // llvm.s390.vfeezb (IntrinsicsSystemZ.td:121)
    s390_vfeezbs,                              // llvm.s390.vfeezbs (IntrinsicsSystemZ.td:131)
    s390_vfeezf,                               // llvm.s390.vfeezf (IntrinsicsSystemZ.td:123)
    s390_vfeezfs,                              // llvm.s390.vfeezfs (IntrinsicsSystemZ.td:133)
    s390_vfeezh,                               // llvm.s390.vfeezh (IntrinsicsSystemZ.td:122)
    s390_vfeezhs,                              // llvm.s390.vfeezhs (IntrinsicsSystemZ.td:132)
    s390_vfeneb,                               // llvm.s390.vfeneb (IntrinsicsSystemZ.td:121)
    s390_vfenebs,                              // llvm.s390.vfenebs (IntrinsicsSystemZ.td:131)
    s390_vfenef,                               // llvm.s390.vfenef (IntrinsicsSystemZ.td:123)
    s390_vfenefs,                              // llvm.s390.vfenefs (IntrinsicsSystemZ.td:133)
    s390_vfeneh,                               // llvm.s390.vfeneh (IntrinsicsSystemZ.td:122)
    s390_vfenehs,                              // llvm.s390.vfenehs (IntrinsicsSystemZ.td:132)
    s390_vfenezb,                              // llvm.s390.vfenezb (IntrinsicsSystemZ.td:121)
    s390_vfenezbs,                             // llvm.s390.vfenezbs (IntrinsicsSystemZ.td:131)
    s390_vfenezf,                              // llvm.s390.vfenezf (IntrinsicsSystemZ.td:123)
    s390_vfenezfs,                             // llvm.s390.vfenezfs (IntrinsicsSystemZ.td:133)
    s390_vfenezh,                              // llvm.s390.vfenezh (IntrinsicsSystemZ.td:122)
    s390_vfenezhs,                             // llvm.s390.vfenezhs (IntrinsicsSystemZ.td:132)
    s390_vfidb,                                // llvm.s390.vfidb (IntrinsicsSystemZ.td:364)
    s390_vfisb,                                // llvm.s390.vfisb (IntrinsicsSystemZ.td:396)
    s390_vfmaxdb,                              // llvm.s390.vfmaxdb (IntrinsicsSystemZ.td:377)
    s390_vfmaxsb,                              // llvm.s390.vfmaxsb (IntrinsicsSystemZ.td:383)
    s390_vfmindb,                              // llvm.s390.vfmindb (IntrinsicsSystemZ.td:380)
    s390_vfminsb,                              // llvm.s390.vfminsb (IntrinsicsSystemZ.td:386)
    s390_vftcidb,                              // llvm.s390.vftcidb (IntrinsicsSystemZ.td:362)
    s390_vftcisb,                              // llvm.s390.vftcisb (IntrinsicsSystemZ.td:394)
    s390_vgemb,                                // llvm.s390.vgemb (IntrinsicsSystemZ.td:450)
    s390_vgemf,                                // llvm.s390.vgemf (IntrinsicsSystemZ.td:452)
    s390_vgemg,                                // llvm.s390.vgemg (IntrinsicsSystemZ.td:453)
    s390_vgemh,                                // llvm.s390.vgemh (IntrinsicsSystemZ.td:451)
    s390_vgemq,                                // llvm.s390.vgemq (IntrinsicsSystemZ.td:454)
    s390_vgfmab,                               // llvm.s390.vgfmab (IntrinsicsSystemZ.td:144)
    s390_vgfmaf,                               // llvm.s390.vgfmaf (IntrinsicsSystemZ.td:146)
    s390_vgfmag,                               // llvm.s390.vgfmag (IntrinsicsSystemZ.td:150)
    s390_vgfmah,                               // llvm.s390.vgfmah (IntrinsicsSystemZ.td:145)
    s390_vgfmb,                                // llvm.s390.vgfmb (IntrinsicsSystemZ.td:111)
    s390_vgfmf,                                // llvm.s390.vgfmf (IntrinsicsSystemZ.td:113)
    s390_vgfmg,                                // llvm.s390.vgfmg (IntrinsicsSystemZ.td:117)
    s390_vgfmh,                                // llvm.s390.vgfmh (IntrinsicsSystemZ.td:112)
    s390_vistrb,                               // llvm.s390.vistrb (IntrinsicsSystemZ.td:83)
    s390_vistrbs,                              // llvm.s390.vistrbs (IntrinsicsSystemZ.td:93)
    s390_vistrf,                               // llvm.s390.vistrf (IntrinsicsSystemZ.td:85)
    s390_vistrfs,                              // llvm.s390.vistrfs (IntrinsicsSystemZ.td:95)
    s390_vistrh,                               // llvm.s390.vistrh (IntrinsicsSystemZ.td:84)
    s390_vistrhs,                              // llvm.s390.vistrhs (IntrinsicsSystemZ.td:94)
    s390_vlbb,                                 // llvm.s390.vlbb (IntrinsicsSystemZ.td:232)
    s390_vll,                                  // llvm.s390.vll (IntrinsicsSystemZ.td:236)
    s390_vlrl,                                 // llvm.s390.vlrl (IntrinsicsSystemZ.td:401)
    s390_vmaeb,                                // llvm.s390.vmaeb (IntrinsicsSystemZ.td:144)
    s390_vmaef,                                // llvm.s390.vmaef (IntrinsicsSystemZ.td:146)
    s390_vmaeg,                                // llvm.s390.vmaeg (IntrinsicsSystemZ.td:466)
    s390_vmaeh,                                // llvm.s390.vmaeh (IntrinsicsSystemZ.td:145)
    s390_vmahb,                                // llvm.s390.vmahb (IntrinsicsSystemZ.td:154)
    s390_vmahf,                                // llvm.s390.vmahf (IntrinsicsSystemZ.td:156)
    s390_vmahg,                                // llvm.s390.vmahg (IntrinsicsSystemZ.td:462)
    s390_vmahh,                                // llvm.s390.vmahh (IntrinsicsSystemZ.td:155)
    s390_vmahq,                                // llvm.s390.vmahq (IntrinsicsSystemZ.td:463)
    s390_vmaleb,                               // llvm.s390.vmaleb (IntrinsicsSystemZ.td:144)
    s390_vmalef,                               // llvm.s390.vmalef (IntrinsicsSystemZ.td:146)
    s390_vmaleg,                               // llvm.s390.vmaleg (IntrinsicsSystemZ.td:467)
    s390_vmaleh,                               // llvm.s390.vmaleh (IntrinsicsSystemZ.td:145)
    s390_vmalhb,                               // llvm.s390.vmalhb (IntrinsicsSystemZ.td:154)
    s390_vmalhf,                               // llvm.s390.vmalhf (IntrinsicsSystemZ.td:156)
    s390_vmalhg,                               // llvm.s390.vmalhg (IntrinsicsSystemZ.td:464)
    s390_vmalhh,                               // llvm.s390.vmalhh (IntrinsicsSystemZ.td:155)
    s390_vmalhq,                               // llvm.s390.vmalhq (IntrinsicsSystemZ.td:465)
    s390_vmalob,                               // llvm.s390.vmalob (IntrinsicsSystemZ.td:144)
    s390_vmalof,                               // llvm.s390.vmalof (IntrinsicsSystemZ.td:146)
    s390_vmalog,                               // llvm.s390.vmalog (IntrinsicsSystemZ.td:469)
    s390_vmaloh,                               // llvm.s390.vmaloh (IntrinsicsSystemZ.td:145)
    s390_vmaob,                                // llvm.s390.vmaob (IntrinsicsSystemZ.td:144)
    s390_vmaof,                                // llvm.s390.vmaof (IntrinsicsSystemZ.td:146)
    s390_vmaog,                                // llvm.s390.vmaog (IntrinsicsSystemZ.td:468)
    s390_vmaoh,                                // llvm.s390.vmaoh (IntrinsicsSystemZ.td:145)
    s390_vmeb,                                 // llvm.s390.vmeb (IntrinsicsSystemZ.td:111)
    s390_vmef,                                 // llvm.s390.vmef (IntrinsicsSystemZ.td:113)
    s390_vmeg,                                 // llvm.s390.vmeg (IntrinsicsSystemZ.td:474)
    s390_vmeh,                                 // llvm.s390.vmeh (IntrinsicsSystemZ.td:112)
    s390_vmhb,                                 // llvm.s390.vmhb (IntrinsicsSystemZ.td:121)
    s390_vmhf,                                 // llvm.s390.vmhf (IntrinsicsSystemZ.td:123)
    s390_vmhg,                                 // llvm.s390.vmhg (IntrinsicsSystemZ.td:470)
    s390_vmhh,                                 // llvm.s390.vmhh (IntrinsicsSystemZ.td:122)
    s390_vmhq,                                 // llvm.s390.vmhq (IntrinsicsSystemZ.td:471)
    s390_vmleb,                                // llvm.s390.vmleb (IntrinsicsSystemZ.td:111)
    s390_vmlef,                                // llvm.s390.vmlef (IntrinsicsSystemZ.td:113)
    s390_vmleg,                                // llvm.s390.vmleg (IntrinsicsSystemZ.td:475)
    s390_vmleh,                                // llvm.s390.vmleh (IntrinsicsSystemZ.td:112)
    s390_vmlhb,                                // llvm.s390.vmlhb (IntrinsicsSystemZ.td:121)
    s390_vmlhf,                                // llvm.s390.vmlhf (IntrinsicsSystemZ.td:123)
    s390_vmlhg,                                // llvm.s390.vmlhg (IntrinsicsSystemZ.td:472)
    s390_vmlhh,                                // llvm.s390.vmlhh (IntrinsicsSystemZ.td:122)
    s390_vmlhq,                                // llvm.s390.vmlhq (IntrinsicsSystemZ.td:473)
    s390_vmlob,                                // llvm.s390.vmlob (IntrinsicsSystemZ.td:111)
    s390_vmlof,                                // llvm.s390.vmlof (IntrinsicsSystemZ.td:113)
    s390_vmlog,                                // llvm.s390.vmlog (IntrinsicsSystemZ.td:477)
    s390_vmloh,                                // llvm.s390.vmloh (IntrinsicsSystemZ.td:112)
    s390_vmob,                                 // llvm.s390.vmob (IntrinsicsSystemZ.td:111)
    s390_vmof,                                 // llvm.s390.vmof (IntrinsicsSystemZ.td:113)
    s390_vmog,                                 // llvm.s390.vmog (IntrinsicsSystemZ.td:476)
    s390_vmoh,                                 // llvm.s390.vmoh (IntrinsicsSystemZ.td:112)
    s390_vmslg,                                // llvm.s390.vmslg (IntrinsicsSystemZ.td:372)
    s390_vpdi,                                 // llvm.s390.vpdi (IntrinsicsSystemZ.td:240)
    s390_vperm,                                // llvm.s390.vperm (IntrinsicsSystemZ.td:245)
    s390_vpklsf,                               // llvm.s390.vpklsf (IntrinsicsSystemZ.td:100)
    s390_vpklsfs,                              // llvm.s390.vpklsfs (IntrinsicsSystemZ.td:106)
    s390_vpklsg,                               // llvm.s390.vpklsg (IntrinsicsSystemZ.td:101)
    s390_vpklsgs,                              // llvm.s390.vpklsgs (IntrinsicsSystemZ.td:107)
    s390_vpklsh,                               // llvm.s390.vpklsh (IntrinsicsSystemZ.td:99)
    s390_vpklshs,                              // llvm.s390.vpklshs (IntrinsicsSystemZ.td:105)
    s390_vpksf,                                // llvm.s390.vpksf (IntrinsicsSystemZ.td:100)
    s390_vpksfs,                               // llvm.s390.vpksfs (IntrinsicsSystemZ.td:106)
    s390_vpksg,                                // llvm.s390.vpksg (IntrinsicsSystemZ.td:101)
    s390_vpksgs,                               // llvm.s390.vpksgs (IntrinsicsSystemZ.td:107)
    s390_vpksh,                                // llvm.s390.vpksh (IntrinsicsSystemZ.td:99)
    s390_vpkshs,                               // llvm.s390.vpkshs (IntrinsicsSystemZ.td:105)
    s390_vsbcbiq,                              // llvm.s390.vsbcbiq (IntrinsicsSystemZ.td:314)
    s390_vsbiq,                                // llvm.s390.vsbiq (IntrinsicsSystemZ.td:312)
    s390_vscbib,                               // llvm.s390.vscbib (IntrinsicsSystemZ.td:121)
    s390_vscbif,                               // llvm.s390.vscbif (IntrinsicsSystemZ.td:123)
    s390_vscbig,                               // llvm.s390.vscbig (IntrinsicsSystemZ.td:127)
    s390_vscbih,                               // llvm.s390.vscbih (IntrinsicsSystemZ.td:122)
    s390_vscbiq,                               // llvm.s390.vscbiq (IntrinsicsSystemZ.td:313)
    s390_vsl,                                  // llvm.s390.vsl (IntrinsicsSystemZ.td:297)
    s390_vslb,                                 // llvm.s390.vslb (IntrinsicsSystemZ.td:298)
    s390_vsld,                                 // llvm.s390.vsld (IntrinsicsSystemZ.td:410)
    s390_vsldb,                                // llvm.s390.vsldb (IntrinsicsSystemZ.td:304)
    s390_vsq,                                  // llvm.s390.vsq (IntrinsicsSystemZ.td:311)
    s390_vsra,                                 // llvm.s390.vsra (IntrinsicsSystemZ.td:299)
    s390_vsrab,                                // llvm.s390.vsrab (IntrinsicsSystemZ.td:300)
    s390_vsrd,                                 // llvm.s390.vsrd (IntrinsicsSystemZ.td:415)
    s390_vsrl,                                 // llvm.s390.vsrl (IntrinsicsSystemZ.td:301)
    s390_vsrlb,                                // llvm.s390.vsrlb (IntrinsicsSystemZ.td:302)
    s390_vstl,                                 // llvm.s390.vstl (IntrinsicsSystemZ.td:256)
    s390_vstrcb,                               // llvm.s390.vstrcb (IntrinsicsSystemZ.td:172)
    s390_vstrcbs,                              // llvm.s390.vstrcbs (IntrinsicsSystemZ.td:183)
    s390_vstrcf,                               // llvm.s390.vstrcf (IntrinsicsSystemZ.td:174)
    s390_vstrcfs,                              // llvm.s390.vstrcfs (IntrinsicsSystemZ.td:185)
    s390_vstrch,                               // llvm.s390.vstrch (IntrinsicsSystemZ.td:173)
    s390_vstrchs,                              // llvm.s390.vstrchs (IntrinsicsSystemZ.td:184)
    s390_vstrczb,                              // llvm.s390.vstrczb (IntrinsicsSystemZ.td:172)
    s390_vstrczbs,                             // llvm.s390.vstrczbs (IntrinsicsSystemZ.td:183)
    s390_vstrczf,                              // llvm.s390.vstrczf (IntrinsicsSystemZ.td:174)
    s390_vstrczfs,                             // llvm.s390.vstrczfs (IntrinsicsSystemZ.td:185)
    s390_vstrczh,                              // llvm.s390.vstrczh (IntrinsicsSystemZ.td:173)
    s390_vstrczhs,                             // llvm.s390.vstrczhs (IntrinsicsSystemZ.td:184)
    s390_vstrl,                                // llvm.s390.vstrl (IntrinsicsSystemZ.td:405)
    s390_vstrsb,                               // llvm.s390.vstrsb (IntrinsicsSystemZ.td:420)
    s390_vstrsf,                               // llvm.s390.vstrsf (IntrinsicsSystemZ.td:422)
    s390_vstrsh,                               // llvm.s390.vstrsh (IntrinsicsSystemZ.td:421)
    s390_vstrszb,                              // llvm.s390.vstrszb (IntrinsicsSystemZ.td:423)
    s390_vstrszf,                              // llvm.s390.vstrszf (IntrinsicsSystemZ.td:425)
    s390_vstrszh,                              // llvm.s390.vstrszh (IntrinsicsSystemZ.td:424)
    s390_vsumb,                                // llvm.s390.vsumb (IntrinsicsSystemZ.td:316)
    s390_vsumgf,                               // llvm.s390.vsumgf (IntrinsicsSystemZ.td:321)
    s390_vsumgh,                               // llvm.s390.vsumgh (IntrinsicsSystemZ.td:319)
    s390_vsumh,                                // llvm.s390.vsumh (IntrinsicsSystemZ.td:317)
    s390_vsumqf,                               // llvm.s390.vsumqf (IntrinsicsSystemZ.td:324)
    s390_vsumqg,                               // llvm.s390.vsumqg (IntrinsicsSystemZ.td:326)
    s390_vtm,                                  // llvm.s390.vtm (IntrinsicsSystemZ.td:329)
    s390_vuphb,                                // llvm.s390.vuphb (IntrinsicsSystemZ.td:71)
    s390_vuphf,                                // llvm.s390.vuphf (IntrinsicsSystemZ.td:73)
    s390_vuphg,                                // llvm.s390.vuphg (IntrinsicsSystemZ.td:457)
    s390_vuphh,                                // llvm.s390.vuphh (IntrinsicsSystemZ.td:72)
    s390_vuplb,                                // llvm.s390.vuplb (IntrinsicsSystemZ.td:77)
    s390_vuplf,                                // llvm.s390.vuplf (IntrinsicsSystemZ.td:79)
    s390_vuplg,                                // llvm.s390.vuplg (IntrinsicsSystemZ.td:455)
    s390_vuplhb,                               // llvm.s390.vuplhb (IntrinsicsSystemZ.td:71)
    s390_vuplhf,                               // llvm.s390.vuplhf (IntrinsicsSystemZ.td:73)
    s390_vuplhg,                               // llvm.s390.vuplhg (IntrinsicsSystemZ.td:458)
    s390_vuplhh,                               // llvm.s390.vuplhh (IntrinsicsSystemZ.td:72)
    s390_vuplhw,                               // llvm.s390.vuplhw (IntrinsicsSystemZ.td:78)
    s390_vupllb,                               // llvm.s390.vupllb (IntrinsicsSystemZ.td:71)
    s390_vupllf,                               // llvm.s390.vupllf (IntrinsicsSystemZ.td:73)
    s390_vupllg,                               // llvm.s390.vupllg (IntrinsicsSystemZ.td:456)
    s390_vupllh,                               // llvm.s390.vupllh (IntrinsicsSystemZ.td:72)
}; // enum
} // namespace llvm::Intrinsic
#endif

