/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_HEXAGON_ENUMS_H
#define LLVM_IR_INTRINSIC_HEXAGON_ENUMS_H
namespace llvm::Intrinsic {
enum HEXAGONIntrinsics : unsigned {
// Enum values for intrinsics.
    hexagon_A2_abs = 4273,                            // llvm.hexagon.A2.abs (IntrinsicsHexagonDep.td:1063)
    hexagon_A2_absp,                           // llvm.hexagon.A2.absp (IntrinsicsHexagonDep.td:1066)
    hexagon_A2_abssat,                         // llvm.hexagon.A2.abssat (IntrinsicsHexagonDep.td:1069)
    hexagon_A2_add,                            // llvm.hexagon.A2.add (IntrinsicsHexagonDep.td:1072)
    hexagon_A2_addh_h16_hh,                    // llvm.hexagon.A2.addh.h16.hh (IntrinsicsHexagonDep.td:1075)
    hexagon_A2_addh_h16_hl,                    // llvm.hexagon.A2.addh.h16.hl (IntrinsicsHexagonDep.td:1078)
    hexagon_A2_addh_h16_lh,                    // llvm.hexagon.A2.addh.h16.lh (IntrinsicsHexagonDep.td:1081)
    hexagon_A2_addh_h16_ll,                    // llvm.hexagon.A2.addh.h16.ll (IntrinsicsHexagonDep.td:1084)
    hexagon_A2_addh_h16_sat_hh,                // llvm.hexagon.A2.addh.h16.sat.hh (IntrinsicsHexagonDep.td:1087)
    hexagon_A2_addh_h16_sat_hl,                // llvm.hexagon.A2.addh.h16.sat.hl (IntrinsicsHexagonDep.td:1090)
    hexagon_A2_addh_h16_sat_lh,                // llvm.hexagon.A2.addh.h16.sat.lh (IntrinsicsHexagonDep.td:1093)
    hexagon_A2_addh_h16_sat_ll,                // llvm.hexagon.A2.addh.h16.sat.ll (IntrinsicsHexagonDep.td:1096)
    hexagon_A2_addh_l16_hl,                    // llvm.hexagon.A2.addh.l16.hl (IntrinsicsHexagonDep.td:1099)
    hexagon_A2_addh_l16_ll,                    // llvm.hexagon.A2.addh.l16.ll (IntrinsicsHexagonDep.td:1102)
    hexagon_A2_addh_l16_sat_hl,                // llvm.hexagon.A2.addh.l16.sat.hl (IntrinsicsHexagonDep.td:1105)
    hexagon_A2_addh_l16_sat_ll,                // llvm.hexagon.A2.addh.l16.sat.ll (IntrinsicsHexagonDep.td:1108)
    hexagon_A2_addi,                           // llvm.hexagon.A2.addi (IntrinsicsHexagonDep.td:1111)
    hexagon_A2_addp,                           // llvm.hexagon.A2.addp (IntrinsicsHexagonDep.td:1114)
    hexagon_A2_addpsat,                        // llvm.hexagon.A2.addpsat (IntrinsicsHexagonDep.td:1117)
    hexagon_A2_addsat,                         // llvm.hexagon.A2.addsat (IntrinsicsHexagonDep.td:1120)
    hexagon_A2_addsp,                          // llvm.hexagon.A2.addsp (IntrinsicsHexagonDep.td:1123)
    hexagon_A2_and,                            // llvm.hexagon.A2.and (IntrinsicsHexagonDep.td:1126)
    hexagon_A2_andir,                          // llvm.hexagon.A2.andir (IntrinsicsHexagonDep.td:1129)
    hexagon_A2_andp,                           // llvm.hexagon.A2.andp (IntrinsicsHexagonDep.td:1132)
    hexagon_A2_aslh,                           // llvm.hexagon.A2.aslh (IntrinsicsHexagonDep.td:1135)
    hexagon_A2_asrh,                           // llvm.hexagon.A2.asrh (IntrinsicsHexagonDep.td:1138)
    hexagon_A2_combine_hh,                     // llvm.hexagon.A2.combine.hh (IntrinsicsHexagonDep.td:1141)
    hexagon_A2_combine_hl,                     // llvm.hexagon.A2.combine.hl (IntrinsicsHexagonDep.td:1144)
    hexagon_A2_combine_lh,                     // llvm.hexagon.A2.combine.lh (IntrinsicsHexagonDep.td:1147)
    hexagon_A2_combine_ll,                     // llvm.hexagon.A2.combine.ll (IntrinsicsHexagonDep.td:1150)
    hexagon_A2_combineii,                      // llvm.hexagon.A2.combineii (IntrinsicsHexagonDep.td:1153)
    hexagon_A2_combinew,                       // llvm.hexagon.A2.combinew (IntrinsicsHexagonDep.td:1156)
    hexagon_A2_max,                            // llvm.hexagon.A2.max (IntrinsicsHexagonDep.td:1159)
    hexagon_A2_maxp,                           // llvm.hexagon.A2.maxp (IntrinsicsHexagonDep.td:1162)
    hexagon_A2_maxu,                           // llvm.hexagon.A2.maxu (IntrinsicsHexagonDep.td:1165)
    hexagon_A2_maxup,                          // llvm.hexagon.A2.maxup (IntrinsicsHexagonDep.td:1168)
    hexagon_A2_min,                            // llvm.hexagon.A2.min (IntrinsicsHexagonDep.td:1171)
    hexagon_A2_minp,                           // llvm.hexagon.A2.minp (IntrinsicsHexagonDep.td:1174)
    hexagon_A2_minu,                           // llvm.hexagon.A2.minu (IntrinsicsHexagonDep.td:1177)
    hexagon_A2_minup,                          // llvm.hexagon.A2.minup (IntrinsicsHexagonDep.td:1180)
    hexagon_A2_neg,                            // llvm.hexagon.A2.neg (IntrinsicsHexagonDep.td:1183)
    hexagon_A2_negp,                           // llvm.hexagon.A2.negp (IntrinsicsHexagonDep.td:1186)
    hexagon_A2_negsat,                         // llvm.hexagon.A2.negsat (IntrinsicsHexagonDep.td:1189)
    hexagon_A2_not,                            // llvm.hexagon.A2.not (IntrinsicsHexagonDep.td:1192)
    hexagon_A2_notp,                           // llvm.hexagon.A2.notp (IntrinsicsHexagonDep.td:1195)
    hexagon_A2_or,                             // llvm.hexagon.A2.or (IntrinsicsHexagonDep.td:1198)
    hexagon_A2_orir,                           // llvm.hexagon.A2.orir (IntrinsicsHexagonDep.td:1201)
    hexagon_A2_orp,                            // llvm.hexagon.A2.orp (IntrinsicsHexagonDep.td:1204)
    hexagon_A2_roundsat,                       // llvm.hexagon.A2.roundsat (IntrinsicsHexagonDep.td:1207)
    hexagon_A2_sat,                            // llvm.hexagon.A2.sat (IntrinsicsHexagonDep.td:1210)
    hexagon_A2_satb,                           // llvm.hexagon.A2.satb (IntrinsicsHexagonDep.td:1213)
    hexagon_A2_sath,                           // llvm.hexagon.A2.sath (IntrinsicsHexagonDep.td:1216)
    hexagon_A2_satub,                          // llvm.hexagon.A2.satub (IntrinsicsHexagonDep.td:1219)
    hexagon_A2_satuh,                          // llvm.hexagon.A2.satuh (IntrinsicsHexagonDep.td:1222)
    hexagon_A2_sub,                            // llvm.hexagon.A2.sub (IntrinsicsHexagonDep.td:1225)
    hexagon_A2_subh_h16_hh,                    // llvm.hexagon.A2.subh.h16.hh (IntrinsicsHexagonDep.td:1228)
    hexagon_A2_subh_h16_hl,                    // llvm.hexagon.A2.subh.h16.hl (IntrinsicsHexagonDep.td:1231)
    hexagon_A2_subh_h16_lh,                    // llvm.hexagon.A2.subh.h16.lh (IntrinsicsHexagonDep.td:1234)
    hexagon_A2_subh_h16_ll,                    // llvm.hexagon.A2.subh.h16.ll (IntrinsicsHexagonDep.td:1237)
    hexagon_A2_subh_h16_sat_hh,                // llvm.hexagon.A2.subh.h16.sat.hh (IntrinsicsHexagonDep.td:1240)
    hexagon_A2_subh_h16_sat_hl,                // llvm.hexagon.A2.subh.h16.sat.hl (IntrinsicsHexagonDep.td:1243)
    hexagon_A2_subh_h16_sat_lh,                // llvm.hexagon.A2.subh.h16.sat.lh (IntrinsicsHexagonDep.td:1246)
    hexagon_A2_subh_h16_sat_ll,                // llvm.hexagon.A2.subh.h16.sat.ll (IntrinsicsHexagonDep.td:1249)
    hexagon_A2_subh_l16_hl,                    // llvm.hexagon.A2.subh.l16.hl (IntrinsicsHexagonDep.td:1252)
    hexagon_A2_subh_l16_ll,                    // llvm.hexagon.A2.subh.l16.ll (IntrinsicsHexagonDep.td:1255)
    hexagon_A2_subh_l16_sat_hl,                // llvm.hexagon.A2.subh.l16.sat.hl (IntrinsicsHexagonDep.td:1258)
    hexagon_A2_subh_l16_sat_ll,                // llvm.hexagon.A2.subh.l16.sat.ll (IntrinsicsHexagonDep.td:1261)
    hexagon_A2_subp,                           // llvm.hexagon.A2.subp (IntrinsicsHexagonDep.td:1264)
    hexagon_A2_subri,                          // llvm.hexagon.A2.subri (IntrinsicsHexagonDep.td:1267)
    hexagon_A2_subsat,                         // llvm.hexagon.A2.subsat (IntrinsicsHexagonDep.td:1270)
    hexagon_A2_svaddh,                         // llvm.hexagon.A2.svaddh (IntrinsicsHexagonDep.td:1273)
    hexagon_A2_svaddhs,                        // llvm.hexagon.A2.svaddhs (IntrinsicsHexagonDep.td:1276)
    hexagon_A2_svadduhs,                       // llvm.hexagon.A2.svadduhs (IntrinsicsHexagonDep.td:1279)
    hexagon_A2_svavgh,                         // llvm.hexagon.A2.svavgh (IntrinsicsHexagonDep.td:1282)
    hexagon_A2_svavghs,                        // llvm.hexagon.A2.svavghs (IntrinsicsHexagonDep.td:1285)
    hexagon_A2_svnavgh,                        // llvm.hexagon.A2.svnavgh (IntrinsicsHexagonDep.td:1288)
    hexagon_A2_svsubh,                         // llvm.hexagon.A2.svsubh (IntrinsicsHexagonDep.td:1291)
    hexagon_A2_svsubhs,                        // llvm.hexagon.A2.svsubhs (IntrinsicsHexagonDep.td:1294)
    hexagon_A2_svsubuhs,                       // llvm.hexagon.A2.svsubuhs (IntrinsicsHexagonDep.td:1297)
    hexagon_A2_swiz,                           // llvm.hexagon.A2.swiz (IntrinsicsHexagonDep.td:1300)
    hexagon_A2_sxtb,                           // llvm.hexagon.A2.sxtb (IntrinsicsHexagonDep.td:1303)
    hexagon_A2_sxth,                           // llvm.hexagon.A2.sxth (IntrinsicsHexagonDep.td:1306)
    hexagon_A2_sxtw,                           // llvm.hexagon.A2.sxtw (IntrinsicsHexagonDep.td:1309)
    hexagon_A2_tfr,                            // llvm.hexagon.A2.tfr (IntrinsicsHexagonDep.td:1312)
    hexagon_A2_tfrih,                          // llvm.hexagon.A2.tfrih (IntrinsicsHexagonDep.td:1315)
    hexagon_A2_tfril,                          // llvm.hexagon.A2.tfril (IntrinsicsHexagonDep.td:1318)
    hexagon_A2_tfrp,                           // llvm.hexagon.A2.tfrp (IntrinsicsHexagonDep.td:1321)
    hexagon_A2_tfrpi,                          // llvm.hexagon.A2.tfrpi (IntrinsicsHexagonDep.td:1324)
    hexagon_A2_tfrsi,                          // llvm.hexagon.A2.tfrsi (IntrinsicsHexagonDep.td:1327)
    hexagon_A2_vabsh,                          // llvm.hexagon.A2.vabsh (IntrinsicsHexagonDep.td:1330)
    hexagon_A2_vabshsat,                       // llvm.hexagon.A2.vabshsat (IntrinsicsHexagonDep.td:1333)
    hexagon_A2_vabsw,                          // llvm.hexagon.A2.vabsw (IntrinsicsHexagonDep.td:1336)
    hexagon_A2_vabswsat,                       // llvm.hexagon.A2.vabswsat (IntrinsicsHexagonDep.td:1339)
    hexagon_A2_vaddb_map,                      // llvm.hexagon.A2.vaddb.map (IntrinsicsHexagonDep.td:1342)
    hexagon_A2_vaddh,                          // llvm.hexagon.A2.vaddh (IntrinsicsHexagonDep.td:1345)
    hexagon_A2_vaddhs,                         // llvm.hexagon.A2.vaddhs (IntrinsicsHexagonDep.td:1348)
    hexagon_A2_vaddub,                         // llvm.hexagon.A2.vaddub (IntrinsicsHexagonDep.td:1351)
    hexagon_A2_vaddubs,                        // llvm.hexagon.A2.vaddubs (IntrinsicsHexagonDep.td:1354)
    hexagon_A2_vadduhs,                        // llvm.hexagon.A2.vadduhs (IntrinsicsHexagonDep.td:1357)
    hexagon_A2_vaddw,                          // llvm.hexagon.A2.vaddw (IntrinsicsHexagonDep.td:1360)
    hexagon_A2_vaddws,                         // llvm.hexagon.A2.vaddws (IntrinsicsHexagonDep.td:1363)
    hexagon_A2_vavgh,                          // llvm.hexagon.A2.vavgh (IntrinsicsHexagonDep.td:1366)
    hexagon_A2_vavghcr,                        // llvm.hexagon.A2.vavghcr (IntrinsicsHexagonDep.td:1369)
    hexagon_A2_vavghr,                         // llvm.hexagon.A2.vavghr (IntrinsicsHexagonDep.td:1372)
    hexagon_A2_vavgub,                         // llvm.hexagon.A2.vavgub (IntrinsicsHexagonDep.td:1375)
    hexagon_A2_vavgubr,                        // llvm.hexagon.A2.vavgubr (IntrinsicsHexagonDep.td:1378)
    hexagon_A2_vavguh,                         // llvm.hexagon.A2.vavguh (IntrinsicsHexagonDep.td:1381)
    hexagon_A2_vavguhr,                        // llvm.hexagon.A2.vavguhr (IntrinsicsHexagonDep.td:1384)
    hexagon_A2_vavguw,                         // llvm.hexagon.A2.vavguw (IntrinsicsHexagonDep.td:1387)
    hexagon_A2_vavguwr,                        // llvm.hexagon.A2.vavguwr (IntrinsicsHexagonDep.td:1390)
    hexagon_A2_vavgw,                          // llvm.hexagon.A2.vavgw (IntrinsicsHexagonDep.td:1393)
    hexagon_A2_vavgwcr,                        // llvm.hexagon.A2.vavgwcr (IntrinsicsHexagonDep.td:1396)
    hexagon_A2_vavgwr,                         // llvm.hexagon.A2.vavgwr (IntrinsicsHexagonDep.td:1399)
    hexagon_A2_vcmpbeq,                        // llvm.hexagon.A2.vcmpbeq (IntrinsicsHexagonDep.td:1402)
    hexagon_A2_vcmpbgtu,                       // llvm.hexagon.A2.vcmpbgtu (IntrinsicsHexagonDep.td:1405)
    hexagon_A2_vcmpheq,                        // llvm.hexagon.A2.vcmpheq (IntrinsicsHexagonDep.td:1408)
    hexagon_A2_vcmphgt,                        // llvm.hexagon.A2.vcmphgt (IntrinsicsHexagonDep.td:1411)
    hexagon_A2_vcmphgtu,                       // llvm.hexagon.A2.vcmphgtu (IntrinsicsHexagonDep.td:1414)
    hexagon_A2_vcmpweq,                        // llvm.hexagon.A2.vcmpweq (IntrinsicsHexagonDep.td:1417)
    hexagon_A2_vcmpwgt,                        // llvm.hexagon.A2.vcmpwgt (IntrinsicsHexagonDep.td:1420)
    hexagon_A2_vcmpwgtu,                       // llvm.hexagon.A2.vcmpwgtu (IntrinsicsHexagonDep.td:1423)
    hexagon_A2_vconj,                          // llvm.hexagon.A2.vconj (IntrinsicsHexagonDep.td:1426)
    hexagon_A2_vmaxb,                          // llvm.hexagon.A2.vmaxb (IntrinsicsHexagonDep.td:1429)
    hexagon_A2_vmaxh,                          // llvm.hexagon.A2.vmaxh (IntrinsicsHexagonDep.td:1432)
    hexagon_A2_vmaxub,                         // llvm.hexagon.A2.vmaxub (IntrinsicsHexagonDep.td:1435)
    hexagon_A2_vmaxuh,                         // llvm.hexagon.A2.vmaxuh (IntrinsicsHexagonDep.td:1438)
    hexagon_A2_vmaxuw,                         // llvm.hexagon.A2.vmaxuw (IntrinsicsHexagonDep.td:1441)
    hexagon_A2_vmaxw,                          // llvm.hexagon.A2.vmaxw (IntrinsicsHexagonDep.td:1444)
    hexagon_A2_vminb,                          // llvm.hexagon.A2.vminb (IntrinsicsHexagonDep.td:1447)
    hexagon_A2_vminh,                          // llvm.hexagon.A2.vminh (IntrinsicsHexagonDep.td:1450)
    hexagon_A2_vminub,                         // llvm.hexagon.A2.vminub (IntrinsicsHexagonDep.td:1453)
    hexagon_A2_vminuh,                         // llvm.hexagon.A2.vminuh (IntrinsicsHexagonDep.td:1456)
    hexagon_A2_vminuw,                         // llvm.hexagon.A2.vminuw (IntrinsicsHexagonDep.td:1459)
    hexagon_A2_vminw,                          // llvm.hexagon.A2.vminw (IntrinsicsHexagonDep.td:1462)
    hexagon_A2_vnavgh,                         // llvm.hexagon.A2.vnavgh (IntrinsicsHexagonDep.td:1465)
    hexagon_A2_vnavghcr,                       // llvm.hexagon.A2.vnavghcr (IntrinsicsHexagonDep.td:1468)
    hexagon_A2_vnavghr,                        // llvm.hexagon.A2.vnavghr (IntrinsicsHexagonDep.td:1471)
    hexagon_A2_vnavgw,                         // llvm.hexagon.A2.vnavgw (IntrinsicsHexagonDep.td:1474)
    hexagon_A2_vnavgwcr,                       // llvm.hexagon.A2.vnavgwcr (IntrinsicsHexagonDep.td:1477)
    hexagon_A2_vnavgwr,                        // llvm.hexagon.A2.vnavgwr (IntrinsicsHexagonDep.td:1480)
    hexagon_A2_vraddub,                        // llvm.hexagon.A2.vraddub (IntrinsicsHexagonDep.td:1483)
    hexagon_A2_vraddub_acc,                    // llvm.hexagon.A2.vraddub.acc (IntrinsicsHexagonDep.td:1486)
    hexagon_A2_vrsadub,                        // llvm.hexagon.A2.vrsadub (IntrinsicsHexagonDep.td:1489)
    hexagon_A2_vrsadub_acc,                    // llvm.hexagon.A2.vrsadub.acc (IntrinsicsHexagonDep.td:1492)
    hexagon_A2_vsubb_map,                      // llvm.hexagon.A2.vsubb.map (IntrinsicsHexagonDep.td:1495)
    hexagon_A2_vsubh,                          // llvm.hexagon.A2.vsubh (IntrinsicsHexagonDep.td:1498)
    hexagon_A2_vsubhs,                         // llvm.hexagon.A2.vsubhs (IntrinsicsHexagonDep.td:1501)
    hexagon_A2_vsubub,                         // llvm.hexagon.A2.vsubub (IntrinsicsHexagonDep.td:1504)
    hexagon_A2_vsububs,                        // llvm.hexagon.A2.vsububs (IntrinsicsHexagonDep.td:1507)
    hexagon_A2_vsubuhs,                        // llvm.hexagon.A2.vsubuhs (IntrinsicsHexagonDep.td:1510)
    hexagon_A2_vsubw,                          // llvm.hexagon.A2.vsubw (IntrinsicsHexagonDep.td:1513)
    hexagon_A2_vsubws,                         // llvm.hexagon.A2.vsubws (IntrinsicsHexagonDep.td:1516)
    hexagon_A2_xor,                            // llvm.hexagon.A2.xor (IntrinsicsHexagonDep.td:1519)
    hexagon_A2_xorp,                           // llvm.hexagon.A2.xorp (IntrinsicsHexagonDep.td:1522)
    hexagon_A2_zxtb,                           // llvm.hexagon.A2.zxtb (IntrinsicsHexagonDep.td:1525)
    hexagon_A2_zxth,                           // llvm.hexagon.A2.zxth (IntrinsicsHexagonDep.td:1528)
    hexagon_A4_andn,                           // llvm.hexagon.A4.andn (IntrinsicsHexagonDep.td:1531)
    hexagon_A4_andnp,                          // llvm.hexagon.A4.andnp (IntrinsicsHexagonDep.td:1534)
    hexagon_A4_bitsplit,                       // llvm.hexagon.A4.bitsplit (IntrinsicsHexagonDep.td:1537)
    hexagon_A4_bitspliti,                      // llvm.hexagon.A4.bitspliti (IntrinsicsHexagonDep.td:1540)
    hexagon_A4_boundscheck,                    // llvm.hexagon.A4.boundscheck (IntrinsicsHexagonDep.td:1543)
    hexagon_A4_cmpbeq,                         // llvm.hexagon.A4.cmpbeq (IntrinsicsHexagonDep.td:1546)
    hexagon_A4_cmpbeqi,                        // llvm.hexagon.A4.cmpbeqi (IntrinsicsHexagonDep.td:1549)
    hexagon_A4_cmpbgt,                         // llvm.hexagon.A4.cmpbgt (IntrinsicsHexagonDep.td:1552)
    hexagon_A4_cmpbgti,                        // llvm.hexagon.A4.cmpbgti (IntrinsicsHexagonDep.td:1555)
    hexagon_A4_cmpbgtu,                        // llvm.hexagon.A4.cmpbgtu (IntrinsicsHexagonDep.td:1558)
    hexagon_A4_cmpbgtui,                       // llvm.hexagon.A4.cmpbgtui (IntrinsicsHexagonDep.td:1561)
    hexagon_A4_cmpheq,                         // llvm.hexagon.A4.cmpheq (IntrinsicsHexagonDep.td:1564)
    hexagon_A4_cmpheqi,                        // llvm.hexagon.A4.cmpheqi (IntrinsicsHexagonDep.td:1567)
    hexagon_A4_cmphgt,                         // llvm.hexagon.A4.cmphgt (IntrinsicsHexagonDep.td:1570)
    hexagon_A4_cmphgti,                        // llvm.hexagon.A4.cmphgti (IntrinsicsHexagonDep.td:1573)
    hexagon_A4_cmphgtu,                        // llvm.hexagon.A4.cmphgtu (IntrinsicsHexagonDep.td:1576)
    hexagon_A4_cmphgtui,                       // llvm.hexagon.A4.cmphgtui (IntrinsicsHexagonDep.td:1579)
    hexagon_A4_combineir,                      // llvm.hexagon.A4.combineir (IntrinsicsHexagonDep.td:1582)
    hexagon_A4_combineri,                      // llvm.hexagon.A4.combineri (IntrinsicsHexagonDep.td:1585)
    hexagon_A4_cround_ri,                      // llvm.hexagon.A4.cround.ri (IntrinsicsHexagonDep.td:1588)
    hexagon_A4_cround_rr,                      // llvm.hexagon.A4.cround.rr (IntrinsicsHexagonDep.td:1591)
    hexagon_A4_modwrapu,                       // llvm.hexagon.A4.modwrapu (IntrinsicsHexagonDep.td:1594)
    hexagon_A4_orn,                            // llvm.hexagon.A4.orn (IntrinsicsHexagonDep.td:1597)
    hexagon_A4_ornp,                           // llvm.hexagon.A4.ornp (IntrinsicsHexagonDep.td:1600)
    hexagon_A4_rcmpeq,                         // llvm.hexagon.A4.rcmpeq (IntrinsicsHexagonDep.td:1603)
    hexagon_A4_rcmpeqi,                        // llvm.hexagon.A4.rcmpeqi (IntrinsicsHexagonDep.td:1606)
    hexagon_A4_rcmpneq,                        // llvm.hexagon.A4.rcmpneq (IntrinsicsHexagonDep.td:1609)
    hexagon_A4_rcmpneqi,                       // llvm.hexagon.A4.rcmpneqi (IntrinsicsHexagonDep.td:1612)
    hexagon_A4_round_ri,                       // llvm.hexagon.A4.round.ri (IntrinsicsHexagonDep.td:1615)
    hexagon_A4_round_ri_sat,                   // llvm.hexagon.A4.round.ri.sat (IntrinsicsHexagonDep.td:1618)
    hexagon_A4_round_rr,                       // llvm.hexagon.A4.round.rr (IntrinsicsHexagonDep.td:1621)
    hexagon_A4_round_rr_sat,                   // llvm.hexagon.A4.round.rr.sat (IntrinsicsHexagonDep.td:1624)
    hexagon_A4_tlbmatch,                       // llvm.hexagon.A4.tlbmatch (IntrinsicsHexagonDep.td:1627)
    hexagon_A4_vcmpbeq_any,                    // llvm.hexagon.A4.vcmpbeq.any (IntrinsicsHexagonDep.td:1630)
    hexagon_A4_vcmpbeqi,                       // llvm.hexagon.A4.vcmpbeqi (IntrinsicsHexagonDep.td:1633)
    hexagon_A4_vcmpbgt,                        // llvm.hexagon.A4.vcmpbgt (IntrinsicsHexagonDep.td:1636)
    hexagon_A4_vcmpbgti,                       // llvm.hexagon.A4.vcmpbgti (IntrinsicsHexagonDep.td:1639)
    hexagon_A4_vcmpbgtui,                      // llvm.hexagon.A4.vcmpbgtui (IntrinsicsHexagonDep.td:1642)
    hexagon_A4_vcmpheqi,                       // llvm.hexagon.A4.vcmpheqi (IntrinsicsHexagonDep.td:1645)
    hexagon_A4_vcmphgti,                       // llvm.hexagon.A4.vcmphgti (IntrinsicsHexagonDep.td:1648)
    hexagon_A4_vcmphgtui,                      // llvm.hexagon.A4.vcmphgtui (IntrinsicsHexagonDep.td:1651)
    hexagon_A4_vcmpweqi,                       // llvm.hexagon.A4.vcmpweqi (IntrinsicsHexagonDep.td:1654)
    hexagon_A4_vcmpwgti,                       // llvm.hexagon.A4.vcmpwgti (IntrinsicsHexagonDep.td:1657)
    hexagon_A4_vcmpwgtui,                      // llvm.hexagon.A4.vcmpwgtui (IntrinsicsHexagonDep.td:1660)
    hexagon_A4_vrmaxh,                         // llvm.hexagon.A4.vrmaxh (IntrinsicsHexagonDep.td:1663)
    hexagon_A4_vrmaxuh,                        // llvm.hexagon.A4.vrmaxuh (IntrinsicsHexagonDep.td:1666)
    hexagon_A4_vrmaxuw,                        // llvm.hexagon.A4.vrmaxuw (IntrinsicsHexagonDep.td:1669)
    hexagon_A4_vrmaxw,                         // llvm.hexagon.A4.vrmaxw (IntrinsicsHexagonDep.td:1672)
    hexagon_A4_vrminh,                         // llvm.hexagon.A4.vrminh (IntrinsicsHexagonDep.td:1675)
    hexagon_A4_vrminuh,                        // llvm.hexagon.A4.vrminuh (IntrinsicsHexagonDep.td:1678)
    hexagon_A4_vrminuw,                        // llvm.hexagon.A4.vrminuw (IntrinsicsHexagonDep.td:1681)
    hexagon_A4_vrminw,                         // llvm.hexagon.A4.vrminw (IntrinsicsHexagonDep.td:1684)
    hexagon_A5_vaddhubs,                       // llvm.hexagon.A5.vaddhubs (IntrinsicsHexagonDep.td:1687)
    hexagon_A6_vcmpbeq_notany,                 // llvm.hexagon.A6.vcmpbeq.notany (IntrinsicsHexagonDep.td:3655)
    hexagon_A7_clip,                           // llvm.hexagon.A7.clip (IntrinsicsHexagonDep.td:3674)
    hexagon_A7_croundd_ri,                     // llvm.hexagon.A7.croundd.ri (IntrinsicsHexagonDep.td:3677)
    hexagon_A7_croundd_rr,                     // llvm.hexagon.A7.croundd.rr (IntrinsicsHexagonDep.td:3680)
    hexagon_A7_vclip,                          // llvm.hexagon.A7.vclip (IntrinsicsHexagonDep.td:3683)
    hexagon_C2_all8,                           // llvm.hexagon.C2.all8 (IntrinsicsHexagonDep.td:1690)
    hexagon_C2_and,                            // llvm.hexagon.C2.and (IntrinsicsHexagonDep.td:1693)
    hexagon_C2_andn,                           // llvm.hexagon.C2.andn (IntrinsicsHexagonDep.td:1696)
    hexagon_C2_any8,                           // llvm.hexagon.C2.any8 (IntrinsicsHexagonDep.td:1699)
    hexagon_C2_bitsclr,                        // llvm.hexagon.C2.bitsclr (IntrinsicsHexagonDep.td:1702)
    hexagon_C2_bitsclri,                       // llvm.hexagon.C2.bitsclri (IntrinsicsHexagonDep.td:1705)
    hexagon_C2_bitsset,                        // llvm.hexagon.C2.bitsset (IntrinsicsHexagonDep.td:1708)
    hexagon_C2_cmpeq,                          // llvm.hexagon.C2.cmpeq (IntrinsicsHexagonDep.td:1711)
    hexagon_C2_cmpeqi,                         // llvm.hexagon.C2.cmpeqi (IntrinsicsHexagonDep.td:1714)
    hexagon_C2_cmpeqp,                         // llvm.hexagon.C2.cmpeqp (IntrinsicsHexagonDep.td:1717)
    hexagon_C2_cmpgei,                         // llvm.hexagon.C2.cmpgei (IntrinsicsHexagonDep.td:1720)
    hexagon_C2_cmpgeui,                        // llvm.hexagon.C2.cmpgeui (IntrinsicsHexagonDep.td:1723)
    hexagon_C2_cmpgt,                          // llvm.hexagon.C2.cmpgt (IntrinsicsHexagonDep.td:1726)
    hexagon_C2_cmpgti,                         // llvm.hexagon.C2.cmpgti (IntrinsicsHexagonDep.td:1729)
    hexagon_C2_cmpgtp,                         // llvm.hexagon.C2.cmpgtp (IntrinsicsHexagonDep.td:1732)
    hexagon_C2_cmpgtu,                         // llvm.hexagon.C2.cmpgtu (IntrinsicsHexagonDep.td:1735)
    hexagon_C2_cmpgtui,                        // llvm.hexagon.C2.cmpgtui (IntrinsicsHexagonDep.td:1738)
    hexagon_C2_cmpgtup,                        // llvm.hexagon.C2.cmpgtup (IntrinsicsHexagonDep.td:1741)
    hexagon_C2_cmplt,                          // llvm.hexagon.C2.cmplt (IntrinsicsHexagonDep.td:1744)
    hexagon_C2_cmpltu,                         // llvm.hexagon.C2.cmpltu (IntrinsicsHexagonDep.td:1747)
    hexagon_C2_mask,                           // llvm.hexagon.C2.mask (IntrinsicsHexagonDep.td:1750)
    hexagon_C2_mux,                            // llvm.hexagon.C2.mux (IntrinsicsHexagonDep.td:1753)
    hexagon_C2_muxii,                          // llvm.hexagon.C2.muxii (IntrinsicsHexagonDep.td:1756)
    hexagon_C2_muxir,                          // llvm.hexagon.C2.muxir (IntrinsicsHexagonDep.td:1759)
    hexagon_C2_muxri,                          // llvm.hexagon.C2.muxri (IntrinsicsHexagonDep.td:1762)
    hexagon_C2_not,                            // llvm.hexagon.C2.not (IntrinsicsHexagonDep.td:1765)
    hexagon_C2_or,                             // llvm.hexagon.C2.or (IntrinsicsHexagonDep.td:1768)
    hexagon_C2_orn,                            // llvm.hexagon.C2.orn (IntrinsicsHexagonDep.td:1771)
    hexagon_C2_pxfer_map,                      // llvm.hexagon.C2.pxfer.map (IntrinsicsHexagonDep.td:1774)
    hexagon_C2_tfrpr,                          // llvm.hexagon.C2.tfrpr (IntrinsicsHexagonDep.td:1777)
    hexagon_C2_tfrrp,                          // llvm.hexagon.C2.tfrrp (IntrinsicsHexagonDep.td:1780)
    hexagon_C2_vitpack,                        // llvm.hexagon.C2.vitpack (IntrinsicsHexagonDep.td:1783)
    hexagon_C2_vmux,                           // llvm.hexagon.C2.vmux (IntrinsicsHexagonDep.td:1786)
    hexagon_C2_xor,                            // llvm.hexagon.C2.xor (IntrinsicsHexagonDep.td:1789)
    hexagon_C4_and_and,                        // llvm.hexagon.C4.and.and (IntrinsicsHexagonDep.td:1792)
    hexagon_C4_and_andn,                       // llvm.hexagon.C4.and.andn (IntrinsicsHexagonDep.td:1795)
    hexagon_C4_and_or,                         // llvm.hexagon.C4.and.or (IntrinsicsHexagonDep.td:1798)
    hexagon_C4_and_orn,                        // llvm.hexagon.C4.and.orn (IntrinsicsHexagonDep.td:1801)
    hexagon_C4_cmplte,                         // llvm.hexagon.C4.cmplte (IntrinsicsHexagonDep.td:1804)
    hexagon_C4_cmpltei,                        // llvm.hexagon.C4.cmpltei (IntrinsicsHexagonDep.td:1807)
    hexagon_C4_cmplteu,                        // llvm.hexagon.C4.cmplteu (IntrinsicsHexagonDep.td:1810)
    hexagon_C4_cmplteui,                       // llvm.hexagon.C4.cmplteui (IntrinsicsHexagonDep.td:1813)
    hexagon_C4_cmpneq,                         // llvm.hexagon.C4.cmpneq (IntrinsicsHexagonDep.td:1816)
    hexagon_C4_cmpneqi,                        // llvm.hexagon.C4.cmpneqi (IntrinsicsHexagonDep.td:1819)
    hexagon_C4_fastcorner9,                    // llvm.hexagon.C4.fastcorner9 (IntrinsicsHexagonDep.td:1822)
    hexagon_C4_fastcorner9_not,                // llvm.hexagon.C4.fastcorner9.not (IntrinsicsHexagonDep.td:1825)
    hexagon_C4_nbitsclr,                       // llvm.hexagon.C4.nbitsclr (IntrinsicsHexagonDep.td:1828)
    hexagon_C4_nbitsclri,                      // llvm.hexagon.C4.nbitsclri (IntrinsicsHexagonDep.td:1831)
    hexagon_C4_nbitsset,                       // llvm.hexagon.C4.nbitsset (IntrinsicsHexagonDep.td:1834)
    hexagon_C4_or_and,                         // llvm.hexagon.C4.or.and (IntrinsicsHexagonDep.td:1837)
    hexagon_C4_or_andn,                        // llvm.hexagon.C4.or.andn (IntrinsicsHexagonDep.td:1840)
    hexagon_C4_or_or,                          // llvm.hexagon.C4.or.or (IntrinsicsHexagonDep.td:1843)
    hexagon_C4_or_orn,                         // llvm.hexagon.C4.or.orn (IntrinsicsHexagonDep.td:1846)
    hexagon_F2_conv_d2df,                      // llvm.hexagon.F2.conv.d2df (IntrinsicsHexagonDep.td:1849)
    hexagon_F2_conv_d2sf,                      // llvm.hexagon.F2.conv.d2sf (IntrinsicsHexagonDep.td:1852)
    hexagon_F2_conv_df2d,                      // llvm.hexagon.F2.conv.df2d (IntrinsicsHexagonDep.td:1855)
    hexagon_F2_conv_df2d_chop,                 // llvm.hexagon.F2.conv.df2d.chop (IntrinsicsHexagonDep.td:1858)
    hexagon_F2_conv_df2sf,                     // llvm.hexagon.F2.conv.df2sf (IntrinsicsHexagonDep.td:1861)
    hexagon_F2_conv_df2ud,                     // llvm.hexagon.F2.conv.df2ud (IntrinsicsHexagonDep.td:1864)
    hexagon_F2_conv_df2ud_chop,                // llvm.hexagon.F2.conv.df2ud.chop (IntrinsicsHexagonDep.td:1867)
    hexagon_F2_conv_df2uw,                     // llvm.hexagon.F2.conv.df2uw (IntrinsicsHexagonDep.td:1870)
    hexagon_F2_conv_df2uw_chop,                // llvm.hexagon.F2.conv.df2uw.chop (IntrinsicsHexagonDep.td:1873)
    hexagon_F2_conv_df2w,                      // llvm.hexagon.F2.conv.df2w (IntrinsicsHexagonDep.td:1876)
    hexagon_F2_conv_df2w_chop,                 // llvm.hexagon.F2.conv.df2w.chop (IntrinsicsHexagonDep.td:1879)
    hexagon_F2_conv_sf2d,                      // llvm.hexagon.F2.conv.sf2d (IntrinsicsHexagonDep.td:1882)
    hexagon_F2_conv_sf2d_chop,                 // llvm.hexagon.F2.conv.sf2d.chop (IntrinsicsHexagonDep.td:1885)
    hexagon_F2_conv_sf2df,                     // llvm.hexagon.F2.conv.sf2df (IntrinsicsHexagonDep.td:1888)
    hexagon_F2_conv_sf2ud,                     // llvm.hexagon.F2.conv.sf2ud (IntrinsicsHexagonDep.td:1891)
    hexagon_F2_conv_sf2ud_chop,                // llvm.hexagon.F2.conv.sf2ud.chop (IntrinsicsHexagonDep.td:1894)
    hexagon_F2_conv_sf2uw,                     // llvm.hexagon.F2.conv.sf2uw (IntrinsicsHexagonDep.td:1897)
    hexagon_F2_conv_sf2uw_chop,                // llvm.hexagon.F2.conv.sf2uw.chop (IntrinsicsHexagonDep.td:1900)
    hexagon_F2_conv_sf2w,                      // llvm.hexagon.F2.conv.sf2w (IntrinsicsHexagonDep.td:1903)
    hexagon_F2_conv_sf2w_chop,                 // llvm.hexagon.F2.conv.sf2w.chop (IntrinsicsHexagonDep.td:1906)
    hexagon_F2_conv_ud2df,                     // llvm.hexagon.F2.conv.ud2df (IntrinsicsHexagonDep.td:1909)
    hexagon_F2_conv_ud2sf,                     // llvm.hexagon.F2.conv.ud2sf (IntrinsicsHexagonDep.td:1912)
    hexagon_F2_conv_uw2df,                     // llvm.hexagon.F2.conv.uw2df (IntrinsicsHexagonDep.td:1915)
    hexagon_F2_conv_uw2sf,                     // llvm.hexagon.F2.conv.uw2sf (IntrinsicsHexagonDep.td:1918)
    hexagon_F2_conv_w2df,                      // llvm.hexagon.F2.conv.w2df (IntrinsicsHexagonDep.td:1921)
    hexagon_F2_conv_w2sf,                      // llvm.hexagon.F2.conv.w2sf (IntrinsicsHexagonDep.td:1924)
    hexagon_F2_dfadd,                          // llvm.hexagon.F2.dfadd (IntrinsicsHexagonDep.td:3660)
    hexagon_F2_dfclass,                        // llvm.hexagon.F2.dfclass (IntrinsicsHexagonDep.td:1927)
    hexagon_F2_dfcmpeq,                        // llvm.hexagon.F2.dfcmpeq (IntrinsicsHexagonDep.td:1930)
    hexagon_F2_dfcmpge,                        // llvm.hexagon.F2.dfcmpge (IntrinsicsHexagonDep.td:1933)
    hexagon_F2_dfcmpgt,                        // llvm.hexagon.F2.dfcmpgt (IntrinsicsHexagonDep.td:1936)
    hexagon_F2_dfcmpuo,                        // llvm.hexagon.F2.dfcmpuo (IntrinsicsHexagonDep.td:1939)
    hexagon_F2_dfimm_n,                        // llvm.hexagon.F2.dfimm.n (IntrinsicsHexagonDep.td:1942)
    hexagon_F2_dfimm_p,                        // llvm.hexagon.F2.dfimm.p (IntrinsicsHexagonDep.td:1945)
    hexagon_F2_dfmax,                          // llvm.hexagon.F2.dfmax (IntrinsicsHexagonDep.td:3686)
    hexagon_F2_dfmin,                          // llvm.hexagon.F2.dfmin (IntrinsicsHexagonDep.td:3689)
    hexagon_F2_dfmpyfix,                       // llvm.hexagon.F2.dfmpyfix (IntrinsicsHexagonDep.td:3692)
    hexagon_F2_dfmpyhh,                        // llvm.hexagon.F2.dfmpyhh (IntrinsicsHexagonDep.td:3695)
    hexagon_F2_dfmpylh,                        // llvm.hexagon.F2.dfmpylh (IntrinsicsHexagonDep.td:3698)
    hexagon_F2_dfmpyll,                        // llvm.hexagon.F2.dfmpyll (IntrinsicsHexagonDep.td:3701)
    hexagon_F2_dfsub,                          // llvm.hexagon.F2.dfsub (IntrinsicsHexagonDep.td:3663)
    hexagon_F2_sfadd,                          // llvm.hexagon.F2.sfadd (IntrinsicsHexagonDep.td:1948)
    hexagon_F2_sfclass,                        // llvm.hexagon.F2.sfclass (IntrinsicsHexagonDep.td:1951)
    hexagon_F2_sfcmpeq,                        // llvm.hexagon.F2.sfcmpeq (IntrinsicsHexagonDep.td:1954)
    hexagon_F2_sfcmpge,                        // llvm.hexagon.F2.sfcmpge (IntrinsicsHexagonDep.td:1957)
    hexagon_F2_sfcmpgt,                        // llvm.hexagon.F2.sfcmpgt (IntrinsicsHexagonDep.td:1960)
    hexagon_F2_sfcmpuo,                        // llvm.hexagon.F2.sfcmpuo (IntrinsicsHexagonDep.td:1963)
    hexagon_F2_sffixupd,                       // llvm.hexagon.F2.sffixupd (IntrinsicsHexagonDep.td:1966)
    hexagon_F2_sffixupn,                       // llvm.hexagon.F2.sffixupn (IntrinsicsHexagonDep.td:1969)
    hexagon_F2_sffixupr,                       // llvm.hexagon.F2.sffixupr (IntrinsicsHexagonDep.td:1972)
    hexagon_F2_sffma,                          // llvm.hexagon.F2.sffma (IntrinsicsHexagonDep.td:1975)
    hexagon_F2_sffma_lib,                      // llvm.hexagon.F2.sffma.lib (IntrinsicsHexagonDep.td:1978)
    hexagon_F2_sffma_sc,                       // llvm.hexagon.F2.sffma.sc (IntrinsicsHexagonDep.td:1981)
    hexagon_F2_sffms,                          // llvm.hexagon.F2.sffms (IntrinsicsHexagonDep.td:1984)
    hexagon_F2_sffms_lib,                      // llvm.hexagon.F2.sffms.lib (IntrinsicsHexagonDep.td:1987)
    hexagon_F2_sfimm_n,                        // llvm.hexagon.F2.sfimm.n (IntrinsicsHexagonDep.td:1990)
    hexagon_F2_sfimm_p,                        // llvm.hexagon.F2.sfimm.p (IntrinsicsHexagonDep.td:1993)
    hexagon_F2_sfmax,                          // llvm.hexagon.F2.sfmax (IntrinsicsHexagonDep.td:1996)
    hexagon_F2_sfmin,                          // llvm.hexagon.F2.sfmin (IntrinsicsHexagonDep.td:1999)
    hexagon_F2_sfmpy,                          // llvm.hexagon.F2.sfmpy (IntrinsicsHexagonDep.td:2002)
    hexagon_F2_sfsub,                          // llvm.hexagon.F2.sfsub (IntrinsicsHexagonDep.td:2005)
    hexagon_L2_loadrb_pbr,                     // llvm.hexagon.L2.loadrb.pbr (IntrinsicsHexagon.td:201)
    hexagon_L2_loadrb_pci,                     // llvm.hexagon.L2.loadrb.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadrb_pcr,                     // llvm.hexagon.L2.loadrb.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadrd_pbr,                     // llvm.hexagon.L2.loadrd.pbr (IntrinsicsHexagon.td:205)
    hexagon_L2_loadrd_pci,                     // llvm.hexagon.L2.loadrd.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadrd_pcr,                     // llvm.hexagon.L2.loadrd.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadrh_pbr,                     // llvm.hexagon.L2.loadrh.pbr (IntrinsicsHexagon.td:203)
    hexagon_L2_loadrh_pci,                     // llvm.hexagon.L2.loadrh.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadrh_pcr,                     // llvm.hexagon.L2.loadrh.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadri_pbr,                     // llvm.hexagon.L2.loadri.pbr (IntrinsicsHexagon.td:204)
    hexagon_L2_loadri_pci,                     // llvm.hexagon.L2.loadri.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadri_pcr,                     // llvm.hexagon.L2.loadri.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadrub_pbr,                    // llvm.hexagon.L2.loadrub.pbr (IntrinsicsHexagon.td:200)
    hexagon_L2_loadrub_pci,                    // llvm.hexagon.L2.loadrub.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadrub_pcr,                    // llvm.hexagon.L2.loadrub.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadruh_pbr,                    // llvm.hexagon.L2.loadruh.pbr (IntrinsicsHexagon.td:202)
    hexagon_L2_loadruh_pci,                    // llvm.hexagon.L2.loadruh.pci (IntrinsicsHexagon.td:160)
    hexagon_L2_loadruh_pcr,                    // llvm.hexagon.L2.loadruh.pcr (IntrinsicsHexagon.td:164)
    hexagon_L2_loadw_locked,                   // llvm.hexagon.L2.loadw.locked (IntrinsicsHexagon.td:132)
    hexagon_L4_loadd_locked,                   // llvm.hexagon.L4.loadd.locked (IntrinsicsHexagon.td:136)
    hexagon_M2_acci,                           // llvm.hexagon.M2.acci (IntrinsicsHexagonDep.td:2008)
    hexagon_M2_accii,                          // llvm.hexagon.M2.accii (IntrinsicsHexagonDep.td:2011)
    hexagon_M2_cmaci_s0,                       // llvm.hexagon.M2.cmaci.s0 (IntrinsicsHexagonDep.td:2014)
    hexagon_M2_cmacr_s0,                       // llvm.hexagon.M2.cmacr.s0 (IntrinsicsHexagonDep.td:2017)
    hexagon_M2_cmacs_s0,                       // llvm.hexagon.M2.cmacs.s0 (IntrinsicsHexagonDep.td:2020)
    hexagon_M2_cmacs_s1,                       // llvm.hexagon.M2.cmacs.s1 (IntrinsicsHexagonDep.td:2023)
    hexagon_M2_cmacsc_s0,                      // llvm.hexagon.M2.cmacsc.s0 (IntrinsicsHexagonDep.td:2026)
    hexagon_M2_cmacsc_s1,                      // llvm.hexagon.M2.cmacsc.s1 (IntrinsicsHexagonDep.td:2029)
    hexagon_M2_cmpyi_s0,                       // llvm.hexagon.M2.cmpyi.s0 (IntrinsicsHexagonDep.td:2032)
    hexagon_M2_cmpyr_s0,                       // llvm.hexagon.M2.cmpyr.s0 (IntrinsicsHexagonDep.td:2035)
    hexagon_M2_cmpyrs_s0,                      // llvm.hexagon.M2.cmpyrs.s0 (IntrinsicsHexagonDep.td:2038)
    hexagon_M2_cmpyrs_s1,                      // llvm.hexagon.M2.cmpyrs.s1 (IntrinsicsHexagonDep.td:2041)
    hexagon_M2_cmpyrsc_s0,                     // llvm.hexagon.M2.cmpyrsc.s0 (IntrinsicsHexagonDep.td:2044)
    hexagon_M2_cmpyrsc_s1,                     // llvm.hexagon.M2.cmpyrsc.s1 (IntrinsicsHexagonDep.td:2047)
    hexagon_M2_cmpys_s0,                       // llvm.hexagon.M2.cmpys.s0 (IntrinsicsHexagonDep.td:2050)
    hexagon_M2_cmpys_s1,                       // llvm.hexagon.M2.cmpys.s1 (IntrinsicsHexagonDep.td:2053)
    hexagon_M2_cmpysc_s0,                      // llvm.hexagon.M2.cmpysc.s0 (IntrinsicsHexagonDep.td:2056)
    hexagon_M2_cmpysc_s1,                      // llvm.hexagon.M2.cmpysc.s1 (IntrinsicsHexagonDep.td:2059)
    hexagon_M2_cnacs_s0,                       // llvm.hexagon.M2.cnacs.s0 (IntrinsicsHexagonDep.td:2062)
    hexagon_M2_cnacs_s1,                       // llvm.hexagon.M2.cnacs.s1 (IntrinsicsHexagonDep.td:2065)
    hexagon_M2_cnacsc_s0,                      // llvm.hexagon.M2.cnacsc.s0 (IntrinsicsHexagonDep.td:2068)
    hexagon_M2_cnacsc_s1,                      // llvm.hexagon.M2.cnacsc.s1 (IntrinsicsHexagonDep.td:2071)
    hexagon_M2_dpmpyss_acc_s0,                 // llvm.hexagon.M2.dpmpyss.acc.s0 (IntrinsicsHexagonDep.td:2074)
    hexagon_M2_dpmpyss_nac_s0,                 // llvm.hexagon.M2.dpmpyss.nac.s0 (IntrinsicsHexagonDep.td:2077)
    hexagon_M2_dpmpyss_rnd_s0,                 // llvm.hexagon.M2.dpmpyss.rnd.s0 (IntrinsicsHexagonDep.td:2080)
    hexagon_M2_dpmpyss_s0,                     // llvm.hexagon.M2.dpmpyss.s0 (IntrinsicsHexagonDep.td:2083)
    hexagon_M2_dpmpyuu_acc_s0,                 // llvm.hexagon.M2.dpmpyuu.acc.s0 (IntrinsicsHexagonDep.td:2086)
    hexagon_M2_dpmpyuu_nac_s0,                 // llvm.hexagon.M2.dpmpyuu.nac.s0 (IntrinsicsHexagonDep.td:2089)
    hexagon_M2_dpmpyuu_s0,                     // llvm.hexagon.M2.dpmpyuu.s0 (IntrinsicsHexagonDep.td:2092)
    hexagon_M2_hmmpyh_rs1,                     // llvm.hexagon.M2.hmmpyh.rs1 (IntrinsicsHexagonDep.td:2095)
    hexagon_M2_hmmpyh_s1,                      // llvm.hexagon.M2.hmmpyh.s1 (IntrinsicsHexagonDep.td:2098)
    hexagon_M2_hmmpyl_rs1,                     // llvm.hexagon.M2.hmmpyl.rs1 (IntrinsicsHexagonDep.td:2101)
    hexagon_M2_hmmpyl_s1,                      // llvm.hexagon.M2.hmmpyl.s1 (IntrinsicsHexagonDep.td:2104)
    hexagon_M2_maci,                           // llvm.hexagon.M2.maci (IntrinsicsHexagonDep.td:2107)
    hexagon_M2_macsin,                         // llvm.hexagon.M2.macsin (IntrinsicsHexagonDep.td:2110)
    hexagon_M2_macsip,                         // llvm.hexagon.M2.macsip (IntrinsicsHexagonDep.td:2113)
    hexagon_M2_mmachs_rs0,                     // llvm.hexagon.M2.mmachs.rs0 (IntrinsicsHexagonDep.td:2116)
    hexagon_M2_mmachs_rs1,                     // llvm.hexagon.M2.mmachs.rs1 (IntrinsicsHexagonDep.td:2119)
    hexagon_M2_mmachs_s0,                      // llvm.hexagon.M2.mmachs.s0 (IntrinsicsHexagonDep.td:2122)
    hexagon_M2_mmachs_s1,                      // llvm.hexagon.M2.mmachs.s1 (IntrinsicsHexagonDep.td:2125)
    hexagon_M2_mmacls_rs0,                     // llvm.hexagon.M2.mmacls.rs0 (IntrinsicsHexagonDep.td:2128)
    hexagon_M2_mmacls_rs1,                     // llvm.hexagon.M2.mmacls.rs1 (IntrinsicsHexagonDep.td:2131)
    hexagon_M2_mmacls_s0,                      // llvm.hexagon.M2.mmacls.s0 (IntrinsicsHexagonDep.td:2134)
    hexagon_M2_mmacls_s1,                      // llvm.hexagon.M2.mmacls.s1 (IntrinsicsHexagonDep.td:2137)
    hexagon_M2_mmacuhs_rs0,                    // llvm.hexagon.M2.mmacuhs.rs0 (IntrinsicsHexagonDep.td:2140)
    hexagon_M2_mmacuhs_rs1,                    // llvm.hexagon.M2.mmacuhs.rs1 (IntrinsicsHexagonDep.td:2143)
    hexagon_M2_mmacuhs_s0,                     // llvm.hexagon.M2.mmacuhs.s0 (IntrinsicsHexagonDep.td:2146)
    hexagon_M2_mmacuhs_s1,                     // llvm.hexagon.M2.mmacuhs.s1 (IntrinsicsHexagonDep.td:2149)
    hexagon_M2_mmaculs_rs0,                    // llvm.hexagon.M2.mmaculs.rs0 (IntrinsicsHexagonDep.td:2152)
    hexagon_M2_mmaculs_rs1,                    // llvm.hexagon.M2.mmaculs.rs1 (IntrinsicsHexagonDep.td:2155)
    hexagon_M2_mmaculs_s0,                     // llvm.hexagon.M2.mmaculs.s0 (IntrinsicsHexagonDep.td:2158)
    hexagon_M2_mmaculs_s1,                     // llvm.hexagon.M2.mmaculs.s1 (IntrinsicsHexagonDep.td:2161)
    hexagon_M2_mmpyh_rs0,                      // llvm.hexagon.M2.mmpyh.rs0 (IntrinsicsHexagonDep.td:2164)
    hexagon_M2_mmpyh_rs1,                      // llvm.hexagon.M2.mmpyh.rs1 (IntrinsicsHexagonDep.td:2167)
    hexagon_M2_mmpyh_s0,                       // llvm.hexagon.M2.mmpyh.s0 (IntrinsicsHexagonDep.td:2170)
    hexagon_M2_mmpyh_s1,                       // llvm.hexagon.M2.mmpyh.s1 (IntrinsicsHexagonDep.td:2173)
    hexagon_M2_mmpyl_rs0,                      // llvm.hexagon.M2.mmpyl.rs0 (IntrinsicsHexagonDep.td:2176)
    hexagon_M2_mmpyl_rs1,                      // llvm.hexagon.M2.mmpyl.rs1 (IntrinsicsHexagonDep.td:2179)
    hexagon_M2_mmpyl_s0,                       // llvm.hexagon.M2.mmpyl.s0 (IntrinsicsHexagonDep.td:2182)
    hexagon_M2_mmpyl_s1,                       // llvm.hexagon.M2.mmpyl.s1 (IntrinsicsHexagonDep.td:2185)
    hexagon_M2_mmpyuh_rs0,                     // llvm.hexagon.M2.mmpyuh.rs0 (IntrinsicsHexagonDep.td:2188)
    hexagon_M2_mmpyuh_rs1,                     // llvm.hexagon.M2.mmpyuh.rs1 (IntrinsicsHexagonDep.td:2191)
    hexagon_M2_mmpyuh_s0,                      // llvm.hexagon.M2.mmpyuh.s0 (IntrinsicsHexagonDep.td:2194)
    hexagon_M2_mmpyuh_s1,                      // llvm.hexagon.M2.mmpyuh.s1 (IntrinsicsHexagonDep.td:2197)
    hexagon_M2_mmpyul_rs0,                     // llvm.hexagon.M2.mmpyul.rs0 (IntrinsicsHexagonDep.td:2200)
    hexagon_M2_mmpyul_rs1,                     // llvm.hexagon.M2.mmpyul.rs1 (IntrinsicsHexagonDep.td:2203)
    hexagon_M2_mmpyul_s0,                      // llvm.hexagon.M2.mmpyul.s0 (IntrinsicsHexagonDep.td:2206)
    hexagon_M2_mmpyul_s1,                      // llvm.hexagon.M2.mmpyul.s1 (IntrinsicsHexagonDep.td:2209)
    hexagon_M2_mnaci,                          // llvm.hexagon.M2.mnaci (IntrinsicsHexagonDep.td:3666)
    hexagon_M2_mpy_acc_hh_s0,                  // llvm.hexagon.M2.mpy.acc.hh.s0 (IntrinsicsHexagonDep.td:2212)
    hexagon_M2_mpy_acc_hh_s1,                  // llvm.hexagon.M2.mpy.acc.hh.s1 (IntrinsicsHexagonDep.td:2215)
    hexagon_M2_mpy_acc_hl_s0,                  // llvm.hexagon.M2.mpy.acc.hl.s0 (IntrinsicsHexagonDep.td:2218)
    hexagon_M2_mpy_acc_hl_s1,                  // llvm.hexagon.M2.mpy.acc.hl.s1 (IntrinsicsHexagonDep.td:2221)
    hexagon_M2_mpy_acc_lh_s0,                  // llvm.hexagon.M2.mpy.acc.lh.s0 (IntrinsicsHexagonDep.td:2224)
    hexagon_M2_mpy_acc_lh_s1,                  // llvm.hexagon.M2.mpy.acc.lh.s1 (IntrinsicsHexagonDep.td:2227)
    hexagon_M2_mpy_acc_ll_s0,                  // llvm.hexagon.M2.mpy.acc.ll.s0 (IntrinsicsHexagonDep.td:2230)
    hexagon_M2_mpy_acc_ll_s1,                  // llvm.hexagon.M2.mpy.acc.ll.s1 (IntrinsicsHexagonDep.td:2233)
    hexagon_M2_mpy_acc_sat_hh_s0,              // llvm.hexagon.M2.mpy.acc.sat.hh.s0 (IntrinsicsHexagonDep.td:2236)
    hexagon_M2_mpy_acc_sat_hh_s1,              // llvm.hexagon.M2.mpy.acc.sat.hh.s1 (IntrinsicsHexagonDep.td:2239)
    hexagon_M2_mpy_acc_sat_hl_s0,              // llvm.hexagon.M2.mpy.acc.sat.hl.s0 (IntrinsicsHexagonDep.td:2242)
    hexagon_M2_mpy_acc_sat_hl_s1,              // llvm.hexagon.M2.mpy.acc.sat.hl.s1 (IntrinsicsHexagonDep.td:2245)
    hexagon_M2_mpy_acc_sat_lh_s0,              // llvm.hexagon.M2.mpy.acc.sat.lh.s0 (IntrinsicsHexagonDep.td:2248)
    hexagon_M2_mpy_acc_sat_lh_s1,              // llvm.hexagon.M2.mpy.acc.sat.lh.s1 (IntrinsicsHexagonDep.td:2251)
    hexagon_M2_mpy_acc_sat_ll_s0,              // llvm.hexagon.M2.mpy.acc.sat.ll.s0 (IntrinsicsHexagonDep.td:2254)
    hexagon_M2_mpy_acc_sat_ll_s1,              // llvm.hexagon.M2.mpy.acc.sat.ll.s1 (IntrinsicsHexagonDep.td:2257)
    hexagon_M2_mpy_hh_s0,                      // llvm.hexagon.M2.mpy.hh.s0 (IntrinsicsHexagonDep.td:2260)
    hexagon_M2_mpy_hh_s1,                      // llvm.hexagon.M2.mpy.hh.s1 (IntrinsicsHexagonDep.td:2263)
    hexagon_M2_mpy_hl_s0,                      // llvm.hexagon.M2.mpy.hl.s0 (IntrinsicsHexagonDep.td:2266)
    hexagon_M2_mpy_hl_s1,                      // llvm.hexagon.M2.mpy.hl.s1 (IntrinsicsHexagonDep.td:2269)
    hexagon_M2_mpy_lh_s0,                      // llvm.hexagon.M2.mpy.lh.s0 (IntrinsicsHexagonDep.td:2272)
    hexagon_M2_mpy_lh_s1,                      // llvm.hexagon.M2.mpy.lh.s1 (IntrinsicsHexagonDep.td:2275)
    hexagon_M2_mpy_ll_s0,                      // llvm.hexagon.M2.mpy.ll.s0 (IntrinsicsHexagonDep.td:2278)
    hexagon_M2_mpy_ll_s1,                      // llvm.hexagon.M2.mpy.ll.s1 (IntrinsicsHexagonDep.td:2281)
    hexagon_M2_mpy_nac_hh_s0,                  // llvm.hexagon.M2.mpy.nac.hh.s0 (IntrinsicsHexagonDep.td:2284)
    hexagon_M2_mpy_nac_hh_s1,                  // llvm.hexagon.M2.mpy.nac.hh.s1 (IntrinsicsHexagonDep.td:2287)
    hexagon_M2_mpy_nac_hl_s0,                  // llvm.hexagon.M2.mpy.nac.hl.s0 (IntrinsicsHexagonDep.td:2290)
    hexagon_M2_mpy_nac_hl_s1,                  // llvm.hexagon.M2.mpy.nac.hl.s1 (IntrinsicsHexagonDep.td:2293)
    hexagon_M2_mpy_nac_lh_s0,                  // llvm.hexagon.M2.mpy.nac.lh.s0 (IntrinsicsHexagonDep.td:2296)
    hexagon_M2_mpy_nac_lh_s1,                  // llvm.hexagon.M2.mpy.nac.lh.s1 (IntrinsicsHexagonDep.td:2299)
    hexagon_M2_mpy_nac_ll_s0,                  // llvm.hexagon.M2.mpy.nac.ll.s0 (IntrinsicsHexagonDep.td:2302)
    hexagon_M2_mpy_nac_ll_s1,                  // llvm.hexagon.M2.mpy.nac.ll.s1 (IntrinsicsHexagonDep.td:2305)
    hexagon_M2_mpy_nac_sat_hh_s0,              // llvm.hexagon.M2.mpy.nac.sat.hh.s0 (IntrinsicsHexagonDep.td:2308)
    hexagon_M2_mpy_nac_sat_hh_s1,              // llvm.hexagon.M2.mpy.nac.sat.hh.s1 (IntrinsicsHexagonDep.td:2311)
    hexagon_M2_mpy_nac_sat_hl_s0,              // llvm.hexagon.M2.mpy.nac.sat.hl.s0 (IntrinsicsHexagonDep.td:2314)
    hexagon_M2_mpy_nac_sat_hl_s1,              // llvm.hexagon.M2.mpy.nac.sat.hl.s1 (IntrinsicsHexagonDep.td:2317)
    hexagon_M2_mpy_nac_sat_lh_s0,              // llvm.hexagon.M2.mpy.nac.sat.lh.s0 (IntrinsicsHexagonDep.td:2320)
    hexagon_M2_mpy_nac_sat_lh_s1,              // llvm.hexagon.M2.mpy.nac.sat.lh.s1 (IntrinsicsHexagonDep.td:2323)
    hexagon_M2_mpy_nac_sat_ll_s0,              // llvm.hexagon.M2.mpy.nac.sat.ll.s0 (IntrinsicsHexagonDep.td:2326)
    hexagon_M2_mpy_nac_sat_ll_s1,              // llvm.hexagon.M2.mpy.nac.sat.ll.s1 (IntrinsicsHexagonDep.td:2329)
    hexagon_M2_mpy_rnd_hh_s0,                  // llvm.hexagon.M2.mpy.rnd.hh.s0 (IntrinsicsHexagonDep.td:2332)
    hexagon_M2_mpy_rnd_hh_s1,                  // llvm.hexagon.M2.mpy.rnd.hh.s1 (IntrinsicsHexagonDep.td:2335)
    hexagon_M2_mpy_rnd_hl_s0,                  // llvm.hexagon.M2.mpy.rnd.hl.s0 (IntrinsicsHexagonDep.td:2338)
    hexagon_M2_mpy_rnd_hl_s1,                  // llvm.hexagon.M2.mpy.rnd.hl.s1 (IntrinsicsHexagonDep.td:2341)
    hexagon_M2_mpy_rnd_lh_s0,                  // llvm.hexagon.M2.mpy.rnd.lh.s0 (IntrinsicsHexagonDep.td:2344)
    hexagon_M2_mpy_rnd_lh_s1,                  // llvm.hexagon.M2.mpy.rnd.lh.s1 (IntrinsicsHexagonDep.td:2347)
    hexagon_M2_mpy_rnd_ll_s0,                  // llvm.hexagon.M2.mpy.rnd.ll.s0 (IntrinsicsHexagonDep.td:2350)
    hexagon_M2_mpy_rnd_ll_s1,                  // llvm.hexagon.M2.mpy.rnd.ll.s1 (IntrinsicsHexagonDep.td:2353)
    hexagon_M2_mpy_sat_hh_s0,                  // llvm.hexagon.M2.mpy.sat.hh.s0 (IntrinsicsHexagonDep.td:2356)
    hexagon_M2_mpy_sat_hh_s1,                  // llvm.hexagon.M2.mpy.sat.hh.s1 (IntrinsicsHexagonDep.td:2359)
    hexagon_M2_mpy_sat_hl_s0,                  // llvm.hexagon.M2.mpy.sat.hl.s0 (IntrinsicsHexagonDep.td:2362)
    hexagon_M2_mpy_sat_hl_s1,                  // llvm.hexagon.M2.mpy.sat.hl.s1 (IntrinsicsHexagonDep.td:2365)
    hexagon_M2_mpy_sat_lh_s0,                  // llvm.hexagon.M2.mpy.sat.lh.s0 (IntrinsicsHexagonDep.td:2368)
    hexagon_M2_mpy_sat_lh_s1,                  // llvm.hexagon.M2.mpy.sat.lh.s1 (IntrinsicsHexagonDep.td:2371)
    hexagon_M2_mpy_sat_ll_s0,                  // llvm.hexagon.M2.mpy.sat.ll.s0 (IntrinsicsHexagonDep.td:2374)
    hexagon_M2_mpy_sat_ll_s1,                  // llvm.hexagon.M2.mpy.sat.ll.s1 (IntrinsicsHexagonDep.td:2377)
    hexagon_M2_mpy_sat_rnd_hh_s0,              // llvm.hexagon.M2.mpy.sat.rnd.hh.s0 (IntrinsicsHexagonDep.td:2380)
    hexagon_M2_mpy_sat_rnd_hh_s1,              // llvm.hexagon.M2.mpy.sat.rnd.hh.s1 (IntrinsicsHexagonDep.td:2383)
    hexagon_M2_mpy_sat_rnd_hl_s0,              // llvm.hexagon.M2.mpy.sat.rnd.hl.s0 (IntrinsicsHexagonDep.td:2386)
    hexagon_M2_mpy_sat_rnd_hl_s1,              // llvm.hexagon.M2.mpy.sat.rnd.hl.s1 (IntrinsicsHexagonDep.td:2389)
    hexagon_M2_mpy_sat_rnd_lh_s0,              // llvm.hexagon.M2.mpy.sat.rnd.lh.s0 (IntrinsicsHexagonDep.td:2392)
    hexagon_M2_mpy_sat_rnd_lh_s1,              // llvm.hexagon.M2.mpy.sat.rnd.lh.s1 (IntrinsicsHexagonDep.td:2395)
    hexagon_M2_mpy_sat_rnd_ll_s0,              // llvm.hexagon.M2.mpy.sat.rnd.ll.s0 (IntrinsicsHexagonDep.td:2398)
    hexagon_M2_mpy_sat_rnd_ll_s1,              // llvm.hexagon.M2.mpy.sat.rnd.ll.s1 (IntrinsicsHexagonDep.td:2401)
    hexagon_M2_mpy_up,                         // llvm.hexagon.M2.mpy.up (IntrinsicsHexagonDep.td:2404)
    hexagon_M2_mpy_up_s1,                      // llvm.hexagon.M2.mpy.up.s1 (IntrinsicsHexagonDep.td:2407)
    hexagon_M2_mpy_up_s1_sat,                  // llvm.hexagon.M2.mpy.up.s1.sat (IntrinsicsHexagonDep.td:2410)
    hexagon_M2_mpyd_acc_hh_s0,                 // llvm.hexagon.M2.mpyd.acc.hh.s0 (IntrinsicsHexagonDep.td:2413)
    hexagon_M2_mpyd_acc_hh_s1,                 // llvm.hexagon.M2.mpyd.acc.hh.s1 (IntrinsicsHexagonDep.td:2416)
    hexagon_M2_mpyd_acc_hl_s0,                 // llvm.hexagon.M2.mpyd.acc.hl.s0 (IntrinsicsHexagonDep.td:2419)
    hexagon_M2_mpyd_acc_hl_s1,                 // llvm.hexagon.M2.mpyd.acc.hl.s1 (IntrinsicsHexagonDep.td:2422)
    hexagon_M2_mpyd_acc_lh_s0,                 // llvm.hexagon.M2.mpyd.acc.lh.s0 (IntrinsicsHexagonDep.td:2425)
    hexagon_M2_mpyd_acc_lh_s1,                 // llvm.hexagon.M2.mpyd.acc.lh.s1 (IntrinsicsHexagonDep.td:2428)
    hexagon_M2_mpyd_acc_ll_s0,                 // llvm.hexagon.M2.mpyd.acc.ll.s0 (IntrinsicsHexagonDep.td:2431)
    hexagon_M2_mpyd_acc_ll_s1,                 // llvm.hexagon.M2.mpyd.acc.ll.s1 (IntrinsicsHexagonDep.td:2434)
    hexagon_M2_mpyd_hh_s0,                     // llvm.hexagon.M2.mpyd.hh.s0 (IntrinsicsHexagonDep.td:2437)
    hexagon_M2_mpyd_hh_s1,                     // llvm.hexagon.M2.mpyd.hh.s1 (IntrinsicsHexagonDep.td:2440)
    hexagon_M2_mpyd_hl_s0,                     // llvm.hexagon.M2.mpyd.hl.s0 (IntrinsicsHexagonDep.td:2443)
    hexagon_M2_mpyd_hl_s1,                     // llvm.hexagon.M2.mpyd.hl.s1 (IntrinsicsHexagonDep.td:2446)
    hexagon_M2_mpyd_lh_s0,                     // llvm.hexagon.M2.mpyd.lh.s0 (IntrinsicsHexagonDep.td:2449)
    hexagon_M2_mpyd_lh_s1,                     // llvm.hexagon.M2.mpyd.lh.s1 (IntrinsicsHexagonDep.td:2452)
    hexagon_M2_mpyd_ll_s0,                     // llvm.hexagon.M2.mpyd.ll.s0 (IntrinsicsHexagonDep.td:2455)
    hexagon_M2_mpyd_ll_s1,                     // llvm.hexagon.M2.mpyd.ll.s1 (IntrinsicsHexagonDep.td:2458)
    hexagon_M2_mpyd_nac_hh_s0,                 // llvm.hexagon.M2.mpyd.nac.hh.s0 (IntrinsicsHexagonDep.td:2461)
    hexagon_M2_mpyd_nac_hh_s1,                 // llvm.hexagon.M2.mpyd.nac.hh.s1 (IntrinsicsHexagonDep.td:2464)
    hexagon_M2_mpyd_nac_hl_s0,                 // llvm.hexagon.M2.mpyd.nac.hl.s0 (IntrinsicsHexagonDep.td:2467)
    hexagon_M2_mpyd_nac_hl_s1,                 // llvm.hexagon.M2.mpyd.nac.hl.s1 (IntrinsicsHexagonDep.td:2470)
    hexagon_M2_mpyd_nac_lh_s0,                 // llvm.hexagon.M2.mpyd.nac.lh.s0 (IntrinsicsHexagonDep.td:2473)
    hexagon_M2_mpyd_nac_lh_s1,                 // llvm.hexagon.M2.mpyd.nac.lh.s1 (IntrinsicsHexagonDep.td:2476)
    hexagon_M2_mpyd_nac_ll_s0,                 // llvm.hexagon.M2.mpyd.nac.ll.s0 (IntrinsicsHexagonDep.td:2479)
    hexagon_M2_mpyd_nac_ll_s1,                 // llvm.hexagon.M2.mpyd.nac.ll.s1 (IntrinsicsHexagonDep.td:2482)
    hexagon_M2_mpyd_rnd_hh_s0,                 // llvm.hexagon.M2.mpyd.rnd.hh.s0 (IntrinsicsHexagonDep.td:2485)
    hexagon_M2_mpyd_rnd_hh_s1,                 // llvm.hexagon.M2.mpyd.rnd.hh.s1 (IntrinsicsHexagonDep.td:2488)
    hexagon_M2_mpyd_rnd_hl_s0,                 // llvm.hexagon.M2.mpyd.rnd.hl.s0 (IntrinsicsHexagonDep.td:2491)
    hexagon_M2_mpyd_rnd_hl_s1,                 // llvm.hexagon.M2.mpyd.rnd.hl.s1 (IntrinsicsHexagonDep.td:2494)
    hexagon_M2_mpyd_rnd_lh_s0,                 // llvm.hexagon.M2.mpyd.rnd.lh.s0 (IntrinsicsHexagonDep.td:2497)
    hexagon_M2_mpyd_rnd_lh_s1,                 // llvm.hexagon.M2.mpyd.rnd.lh.s1 (IntrinsicsHexagonDep.td:2500)
    hexagon_M2_mpyd_rnd_ll_s0,                 // llvm.hexagon.M2.mpyd.rnd.ll.s0 (IntrinsicsHexagonDep.td:2503)
    hexagon_M2_mpyd_rnd_ll_s1,                 // llvm.hexagon.M2.mpyd.rnd.ll.s1 (IntrinsicsHexagonDep.td:2506)
    hexagon_M2_mpyi,                           // llvm.hexagon.M2.mpyi (IntrinsicsHexagonDep.td:2509)
    hexagon_M2_mpysmi,                         // llvm.hexagon.M2.mpysmi (IntrinsicsHexagonDep.td:2512)
    hexagon_M2_mpysu_up,                       // llvm.hexagon.M2.mpysu.up (IntrinsicsHexagonDep.td:2515)
    hexagon_M2_mpyu_acc_hh_s0,                 // llvm.hexagon.M2.mpyu.acc.hh.s0 (IntrinsicsHexagonDep.td:2518)
    hexagon_M2_mpyu_acc_hh_s1,                 // llvm.hexagon.M2.mpyu.acc.hh.s1 (IntrinsicsHexagonDep.td:2521)
    hexagon_M2_mpyu_acc_hl_s0,                 // llvm.hexagon.M2.mpyu.acc.hl.s0 (IntrinsicsHexagonDep.td:2524)
    hexagon_M2_mpyu_acc_hl_s1,                 // llvm.hexagon.M2.mpyu.acc.hl.s1 (IntrinsicsHexagonDep.td:2527)
    hexagon_M2_mpyu_acc_lh_s0,                 // llvm.hexagon.M2.mpyu.acc.lh.s0 (IntrinsicsHexagonDep.td:2530)
    hexagon_M2_mpyu_acc_lh_s1,                 // llvm.hexagon.M2.mpyu.acc.lh.s1 (IntrinsicsHexagonDep.td:2533)
    hexagon_M2_mpyu_acc_ll_s0,                 // llvm.hexagon.M2.mpyu.acc.ll.s0 (IntrinsicsHexagonDep.td:2536)
    hexagon_M2_mpyu_acc_ll_s1,                 // llvm.hexagon.M2.mpyu.acc.ll.s1 (IntrinsicsHexagonDep.td:2539)
    hexagon_M2_mpyu_hh_s0,                     // llvm.hexagon.M2.mpyu.hh.s0 (IntrinsicsHexagonDep.td:2542)
    hexagon_M2_mpyu_hh_s1,                     // llvm.hexagon.M2.mpyu.hh.s1 (IntrinsicsHexagonDep.td:2545)
    hexagon_M2_mpyu_hl_s0,                     // llvm.hexagon.M2.mpyu.hl.s0 (IntrinsicsHexagonDep.td:2548)
    hexagon_M2_mpyu_hl_s1,                     // llvm.hexagon.M2.mpyu.hl.s1 (IntrinsicsHexagonDep.td:2551)
    hexagon_M2_mpyu_lh_s0,                     // llvm.hexagon.M2.mpyu.lh.s0 (IntrinsicsHexagonDep.td:2554)
    hexagon_M2_mpyu_lh_s1,                     // llvm.hexagon.M2.mpyu.lh.s1 (IntrinsicsHexagonDep.td:2557)
    hexagon_M2_mpyu_ll_s0,                     // llvm.hexagon.M2.mpyu.ll.s0 (IntrinsicsHexagonDep.td:2560)
    hexagon_M2_mpyu_ll_s1,                     // llvm.hexagon.M2.mpyu.ll.s1 (IntrinsicsHexagonDep.td:2563)
    hexagon_M2_mpyu_nac_hh_s0,                 // llvm.hexagon.M2.mpyu.nac.hh.s0 (IntrinsicsHexagonDep.td:2566)
    hexagon_M2_mpyu_nac_hh_s1,                 // llvm.hexagon.M2.mpyu.nac.hh.s1 (IntrinsicsHexagonDep.td:2569)
    hexagon_M2_mpyu_nac_hl_s0,                 // llvm.hexagon.M2.mpyu.nac.hl.s0 (IntrinsicsHexagonDep.td:2572)
    hexagon_M2_mpyu_nac_hl_s1,                 // llvm.hexagon.M2.mpyu.nac.hl.s1 (IntrinsicsHexagonDep.td:2575)
    hexagon_M2_mpyu_nac_lh_s0,                 // llvm.hexagon.M2.mpyu.nac.lh.s0 (IntrinsicsHexagonDep.td:2578)
    hexagon_M2_mpyu_nac_lh_s1,                 // llvm.hexagon.M2.mpyu.nac.lh.s1 (IntrinsicsHexagonDep.td:2581)
    hexagon_M2_mpyu_nac_ll_s0,                 // llvm.hexagon.M2.mpyu.nac.ll.s0 (IntrinsicsHexagonDep.td:2584)
    hexagon_M2_mpyu_nac_ll_s1,                 // llvm.hexagon.M2.mpyu.nac.ll.s1 (IntrinsicsHexagonDep.td:2587)
    hexagon_M2_mpyu_up,                        // llvm.hexagon.M2.mpyu.up (IntrinsicsHexagonDep.td:2590)
    hexagon_M2_mpyud_acc_hh_s0,                // llvm.hexagon.M2.mpyud.acc.hh.s0 (IntrinsicsHexagonDep.td:2593)
    hexagon_M2_mpyud_acc_hh_s1,                // llvm.hexagon.M2.mpyud.acc.hh.s1 (IntrinsicsHexagonDep.td:2596)
    hexagon_M2_mpyud_acc_hl_s0,                // llvm.hexagon.M2.mpyud.acc.hl.s0 (IntrinsicsHexagonDep.td:2599)
    hexagon_M2_mpyud_acc_hl_s1,                // llvm.hexagon.M2.mpyud.acc.hl.s1 (IntrinsicsHexagonDep.td:2602)
    hexagon_M2_mpyud_acc_lh_s0,                // llvm.hexagon.M2.mpyud.acc.lh.s0 (IntrinsicsHexagonDep.td:2605)
    hexagon_M2_mpyud_acc_lh_s1,                // llvm.hexagon.M2.mpyud.acc.lh.s1 (IntrinsicsHexagonDep.td:2608)
    hexagon_M2_mpyud_acc_ll_s0,                // llvm.hexagon.M2.mpyud.acc.ll.s0 (IntrinsicsHexagonDep.td:2611)
    hexagon_M2_mpyud_acc_ll_s1,                // llvm.hexagon.M2.mpyud.acc.ll.s1 (IntrinsicsHexagonDep.td:2614)
    hexagon_M2_mpyud_hh_s0,                    // llvm.hexagon.M2.mpyud.hh.s0 (IntrinsicsHexagonDep.td:2617)
    hexagon_M2_mpyud_hh_s1,                    // llvm.hexagon.M2.mpyud.hh.s1 (IntrinsicsHexagonDep.td:2620)
    hexagon_M2_mpyud_hl_s0,                    // llvm.hexagon.M2.mpyud.hl.s0 (IntrinsicsHexagonDep.td:2623)
    hexagon_M2_mpyud_hl_s1,                    // llvm.hexagon.M2.mpyud.hl.s1 (IntrinsicsHexagonDep.td:2626)
    hexagon_M2_mpyud_lh_s0,                    // llvm.hexagon.M2.mpyud.lh.s0 (IntrinsicsHexagonDep.td:2629)
    hexagon_M2_mpyud_lh_s1,                    // llvm.hexagon.M2.mpyud.lh.s1 (IntrinsicsHexagonDep.td:2632)
    hexagon_M2_mpyud_ll_s0,                    // llvm.hexagon.M2.mpyud.ll.s0 (IntrinsicsHexagonDep.td:2635)
    hexagon_M2_mpyud_ll_s1,                    // llvm.hexagon.M2.mpyud.ll.s1 (IntrinsicsHexagonDep.td:2638)
    hexagon_M2_mpyud_nac_hh_s0,                // llvm.hexagon.M2.mpyud.nac.hh.s0 (IntrinsicsHexagonDep.td:2641)
    hexagon_M2_mpyud_nac_hh_s1,                // llvm.hexagon.M2.mpyud.nac.hh.s1 (IntrinsicsHexagonDep.td:2644)
    hexagon_M2_mpyud_nac_hl_s0,                // llvm.hexagon.M2.mpyud.nac.hl.s0 (IntrinsicsHexagonDep.td:2647)
    hexagon_M2_mpyud_nac_hl_s1,                // llvm.hexagon.M2.mpyud.nac.hl.s1 (IntrinsicsHexagonDep.td:2650)
    hexagon_M2_mpyud_nac_lh_s0,                // llvm.hexagon.M2.mpyud.nac.lh.s0 (IntrinsicsHexagonDep.td:2653)
    hexagon_M2_mpyud_nac_lh_s1,                // llvm.hexagon.M2.mpyud.nac.lh.s1 (IntrinsicsHexagonDep.td:2656)
    hexagon_M2_mpyud_nac_ll_s0,                // llvm.hexagon.M2.mpyud.nac.ll.s0 (IntrinsicsHexagonDep.td:2659)
    hexagon_M2_mpyud_nac_ll_s1,                // llvm.hexagon.M2.mpyud.nac.ll.s1 (IntrinsicsHexagonDep.td:2662)
    hexagon_M2_mpyui,                          // llvm.hexagon.M2.mpyui (IntrinsicsHexagonDep.td:2665)
    hexagon_M2_nacci,                          // llvm.hexagon.M2.nacci (IntrinsicsHexagonDep.td:2668)
    hexagon_M2_naccii,                         // llvm.hexagon.M2.naccii (IntrinsicsHexagonDep.td:2671)
    hexagon_M2_subacc,                         // llvm.hexagon.M2.subacc (IntrinsicsHexagonDep.td:2674)
    hexagon_M2_vabsdiffh,                      // llvm.hexagon.M2.vabsdiffh (IntrinsicsHexagonDep.td:2677)
    hexagon_M2_vabsdiffw,                      // llvm.hexagon.M2.vabsdiffw (IntrinsicsHexagonDep.td:2680)
    hexagon_M2_vcmac_s0_sat_i,                 // llvm.hexagon.M2.vcmac.s0.sat.i (IntrinsicsHexagonDep.td:2683)
    hexagon_M2_vcmac_s0_sat_r,                 // llvm.hexagon.M2.vcmac.s0.sat.r (IntrinsicsHexagonDep.td:2686)
    hexagon_M2_vcmpy_s0_sat_i,                 // llvm.hexagon.M2.vcmpy.s0.sat.i (IntrinsicsHexagonDep.td:2689)
    hexagon_M2_vcmpy_s0_sat_r,                 // llvm.hexagon.M2.vcmpy.s0.sat.r (IntrinsicsHexagonDep.td:2692)
    hexagon_M2_vcmpy_s1_sat_i,                 // llvm.hexagon.M2.vcmpy.s1.sat.i (IntrinsicsHexagonDep.td:2695)
    hexagon_M2_vcmpy_s1_sat_r,                 // llvm.hexagon.M2.vcmpy.s1.sat.r (IntrinsicsHexagonDep.td:2698)
    hexagon_M2_vdmacs_s0,                      // llvm.hexagon.M2.vdmacs.s0 (IntrinsicsHexagonDep.td:2701)
    hexagon_M2_vdmacs_s1,                      // llvm.hexagon.M2.vdmacs.s1 (IntrinsicsHexagonDep.td:2704)
    hexagon_M2_vdmpyrs_s0,                     // llvm.hexagon.M2.vdmpyrs.s0 (IntrinsicsHexagonDep.td:2707)
    hexagon_M2_vdmpyrs_s1,                     // llvm.hexagon.M2.vdmpyrs.s1 (IntrinsicsHexagonDep.td:2710)
    hexagon_M2_vdmpys_s0,                      // llvm.hexagon.M2.vdmpys.s0 (IntrinsicsHexagonDep.td:2713)
    hexagon_M2_vdmpys_s1,                      // llvm.hexagon.M2.vdmpys.s1 (IntrinsicsHexagonDep.td:2716)
    hexagon_M2_vmac2,                          // llvm.hexagon.M2.vmac2 (IntrinsicsHexagonDep.td:2719)
    hexagon_M2_vmac2es,                        // llvm.hexagon.M2.vmac2es (IntrinsicsHexagonDep.td:2722)
    hexagon_M2_vmac2es_s0,                     // llvm.hexagon.M2.vmac2es.s0 (IntrinsicsHexagonDep.td:2725)
    hexagon_M2_vmac2es_s1,                     // llvm.hexagon.M2.vmac2es.s1 (IntrinsicsHexagonDep.td:2728)
    hexagon_M2_vmac2s_s0,                      // llvm.hexagon.M2.vmac2s.s0 (IntrinsicsHexagonDep.td:2731)
    hexagon_M2_vmac2s_s1,                      // llvm.hexagon.M2.vmac2s.s1 (IntrinsicsHexagonDep.td:2734)
    hexagon_M2_vmac2su_s0,                     // llvm.hexagon.M2.vmac2su.s0 (IntrinsicsHexagonDep.td:2737)
    hexagon_M2_vmac2su_s1,                     // llvm.hexagon.M2.vmac2su.s1 (IntrinsicsHexagonDep.td:2740)
    hexagon_M2_vmpy2es_s0,                     // llvm.hexagon.M2.vmpy2es.s0 (IntrinsicsHexagonDep.td:2743)
    hexagon_M2_vmpy2es_s1,                     // llvm.hexagon.M2.vmpy2es.s1 (IntrinsicsHexagonDep.td:2746)
    hexagon_M2_vmpy2s_s0,                      // llvm.hexagon.M2.vmpy2s.s0 (IntrinsicsHexagonDep.td:2749)
    hexagon_M2_vmpy2s_s0pack,                  // llvm.hexagon.M2.vmpy2s.s0pack (IntrinsicsHexagonDep.td:2752)
    hexagon_M2_vmpy2s_s1,                      // llvm.hexagon.M2.vmpy2s.s1 (IntrinsicsHexagonDep.td:2755)
    hexagon_M2_vmpy2s_s1pack,                  // llvm.hexagon.M2.vmpy2s.s1pack (IntrinsicsHexagonDep.td:2758)
    hexagon_M2_vmpy2su_s0,                     // llvm.hexagon.M2.vmpy2su.s0 (IntrinsicsHexagonDep.td:2761)
    hexagon_M2_vmpy2su_s1,                     // llvm.hexagon.M2.vmpy2su.s1 (IntrinsicsHexagonDep.td:2764)
    hexagon_M2_vraddh,                         // llvm.hexagon.M2.vraddh (IntrinsicsHexagonDep.td:2767)
    hexagon_M2_vradduh,                        // llvm.hexagon.M2.vradduh (IntrinsicsHexagonDep.td:2770)
    hexagon_M2_vrcmaci_s0,                     // llvm.hexagon.M2.vrcmaci.s0 (IntrinsicsHexagonDep.td:2773)
    hexagon_M2_vrcmaci_s0c,                    // llvm.hexagon.M2.vrcmaci.s0c (IntrinsicsHexagonDep.td:2776)
    hexagon_M2_vrcmacr_s0,                     // llvm.hexagon.M2.vrcmacr.s0 (IntrinsicsHexagonDep.td:2779)
    hexagon_M2_vrcmacr_s0c,                    // llvm.hexagon.M2.vrcmacr.s0c (IntrinsicsHexagonDep.td:2782)
    hexagon_M2_vrcmpyi_s0,                     // llvm.hexagon.M2.vrcmpyi.s0 (IntrinsicsHexagonDep.td:2785)
    hexagon_M2_vrcmpyi_s0c,                    // llvm.hexagon.M2.vrcmpyi.s0c (IntrinsicsHexagonDep.td:2788)
    hexagon_M2_vrcmpyr_s0,                     // llvm.hexagon.M2.vrcmpyr.s0 (IntrinsicsHexagonDep.td:2791)
    hexagon_M2_vrcmpyr_s0c,                    // llvm.hexagon.M2.vrcmpyr.s0c (IntrinsicsHexagonDep.td:2794)
    hexagon_M2_vrcmpys_acc_s1,                 // llvm.hexagon.M2.vrcmpys.acc.s1 (IntrinsicsHexagonDep.td:2797)
    hexagon_M2_vrcmpys_s1,                     // llvm.hexagon.M2.vrcmpys.s1 (IntrinsicsHexagonDep.td:2800)
    hexagon_M2_vrcmpys_s1rp,                   // llvm.hexagon.M2.vrcmpys.s1rp (IntrinsicsHexagonDep.td:2803)
    hexagon_M2_vrmac_s0,                       // llvm.hexagon.M2.vrmac.s0 (IntrinsicsHexagonDep.td:2806)
    hexagon_M2_vrmpy_s0,                       // llvm.hexagon.M2.vrmpy.s0 (IntrinsicsHexagonDep.td:2809)
    hexagon_M2_xor_xacc,                       // llvm.hexagon.M2.xor.xacc (IntrinsicsHexagonDep.td:2812)
    hexagon_M4_and_and,                        // llvm.hexagon.M4.and.and (IntrinsicsHexagonDep.td:2815)
    hexagon_M4_and_andn,                       // llvm.hexagon.M4.and.andn (IntrinsicsHexagonDep.td:2818)
    hexagon_M4_and_or,                         // llvm.hexagon.M4.and.or (IntrinsicsHexagonDep.td:2821)
    hexagon_M4_and_xor,                        // llvm.hexagon.M4.and.xor (IntrinsicsHexagonDep.td:2824)
    hexagon_M4_cmpyi_wh,                       // llvm.hexagon.M4.cmpyi.wh (IntrinsicsHexagonDep.td:2827)
    hexagon_M4_cmpyi_whc,                      // llvm.hexagon.M4.cmpyi.whc (IntrinsicsHexagonDep.td:2830)
    hexagon_M4_cmpyr_wh,                       // llvm.hexagon.M4.cmpyr.wh (IntrinsicsHexagonDep.td:2833)
    hexagon_M4_cmpyr_whc,                      // llvm.hexagon.M4.cmpyr.whc (IntrinsicsHexagonDep.td:2836)
    hexagon_M4_mac_up_s1_sat,                  // llvm.hexagon.M4.mac.up.s1.sat (IntrinsicsHexagonDep.td:2839)
    hexagon_M4_mpyri_addi,                     // llvm.hexagon.M4.mpyri.addi (IntrinsicsHexagonDep.td:2842)
    hexagon_M4_mpyri_addr,                     // llvm.hexagon.M4.mpyri.addr (IntrinsicsHexagonDep.td:2845)
    hexagon_M4_mpyri_addr_u2,                  // llvm.hexagon.M4.mpyri.addr.u2 (IntrinsicsHexagonDep.td:2848)
    hexagon_M4_mpyrr_addi,                     // llvm.hexagon.M4.mpyrr.addi (IntrinsicsHexagonDep.td:2851)
    hexagon_M4_mpyrr_addr,                     // llvm.hexagon.M4.mpyrr.addr (IntrinsicsHexagonDep.td:2854)
    hexagon_M4_nac_up_s1_sat,                  // llvm.hexagon.M4.nac.up.s1.sat (IntrinsicsHexagonDep.td:2857)
    hexagon_M4_or_and,                         // llvm.hexagon.M4.or.and (IntrinsicsHexagonDep.td:2860)
    hexagon_M4_or_andn,                        // llvm.hexagon.M4.or.andn (IntrinsicsHexagonDep.td:2863)
    hexagon_M4_or_or,                          // llvm.hexagon.M4.or.or (IntrinsicsHexagonDep.td:2866)
    hexagon_M4_or_xor,                         // llvm.hexagon.M4.or.xor (IntrinsicsHexagonDep.td:2869)
    hexagon_M4_pmpyw,                          // llvm.hexagon.M4.pmpyw (IntrinsicsHexagonDep.td:2872)
    hexagon_M4_pmpyw_acc,                      // llvm.hexagon.M4.pmpyw.acc (IntrinsicsHexagonDep.td:2875)
    hexagon_M4_vpmpyh,                         // llvm.hexagon.M4.vpmpyh (IntrinsicsHexagonDep.td:2878)
    hexagon_M4_vpmpyh_acc,                     // llvm.hexagon.M4.vpmpyh.acc (IntrinsicsHexagonDep.td:2881)
    hexagon_M4_vrmpyeh_acc_s0,                 // llvm.hexagon.M4.vrmpyeh.acc.s0 (IntrinsicsHexagonDep.td:2884)
    hexagon_M4_vrmpyeh_acc_s1,                 // llvm.hexagon.M4.vrmpyeh.acc.s1 (IntrinsicsHexagonDep.td:2887)
    hexagon_M4_vrmpyeh_s0,                     // llvm.hexagon.M4.vrmpyeh.s0 (IntrinsicsHexagonDep.td:2890)
    hexagon_M4_vrmpyeh_s1,                     // llvm.hexagon.M4.vrmpyeh.s1 (IntrinsicsHexagonDep.td:2893)
    hexagon_M4_vrmpyoh_acc_s0,                 // llvm.hexagon.M4.vrmpyoh.acc.s0 (IntrinsicsHexagonDep.td:2896)
    hexagon_M4_vrmpyoh_acc_s1,                 // llvm.hexagon.M4.vrmpyoh.acc.s1 (IntrinsicsHexagonDep.td:2899)
    hexagon_M4_vrmpyoh_s0,                     // llvm.hexagon.M4.vrmpyoh.s0 (IntrinsicsHexagonDep.td:2902)
    hexagon_M4_vrmpyoh_s1,                     // llvm.hexagon.M4.vrmpyoh.s1 (IntrinsicsHexagonDep.td:2905)
    hexagon_M4_xor_and,                        // llvm.hexagon.M4.xor.and (IntrinsicsHexagonDep.td:2908)
    hexagon_M4_xor_andn,                       // llvm.hexagon.M4.xor.andn (IntrinsicsHexagonDep.td:2911)
    hexagon_M4_xor_or,                         // llvm.hexagon.M4.xor.or (IntrinsicsHexagonDep.td:2914)
    hexagon_M4_xor_xacc,                       // llvm.hexagon.M4.xor.xacc (IntrinsicsHexagonDep.td:2917)
    hexagon_M5_vdmacbsu,                       // llvm.hexagon.M5.vdmacbsu (IntrinsicsHexagonDep.td:2920)
    hexagon_M5_vdmpybsu,                       // llvm.hexagon.M5.vdmpybsu (IntrinsicsHexagonDep.td:2923)
    hexagon_M5_vmacbsu,                        // llvm.hexagon.M5.vmacbsu (IntrinsicsHexagonDep.td:2926)
    hexagon_M5_vmacbuu,                        // llvm.hexagon.M5.vmacbuu (IntrinsicsHexagonDep.td:2929)
    hexagon_M5_vmpybsu,                        // llvm.hexagon.M5.vmpybsu (IntrinsicsHexagonDep.td:2932)
    hexagon_M5_vmpybuu,                        // llvm.hexagon.M5.vmpybuu (IntrinsicsHexagonDep.td:2935)
    hexagon_M5_vrmacbsu,                       // llvm.hexagon.M5.vrmacbsu (IntrinsicsHexagonDep.td:2938)
    hexagon_M5_vrmacbuu,                       // llvm.hexagon.M5.vrmacbuu (IntrinsicsHexagonDep.td:2941)
    hexagon_M5_vrmpybsu,                       // llvm.hexagon.M5.vrmpybsu (IntrinsicsHexagonDep.td:2944)
    hexagon_M5_vrmpybuu,                       // llvm.hexagon.M5.vrmpybuu (IntrinsicsHexagonDep.td:2947)
    hexagon_M6_vabsdiffb,                      // llvm.hexagon.M6.vabsdiffb (IntrinsicsHexagonDep.td:3638)
    hexagon_M6_vabsdiffub,                     // llvm.hexagon.M6.vabsdiffub (IntrinsicsHexagonDep.td:3641)
    hexagon_M7_dcmpyiw,                        // llvm.hexagon.M7.dcmpyiw (IntrinsicsHexagonDep.td:3704)
    hexagon_M7_dcmpyiw_acc,                    // llvm.hexagon.M7.dcmpyiw.acc (IntrinsicsHexagonDep.td:3707)
    hexagon_M7_dcmpyiwc,                       // llvm.hexagon.M7.dcmpyiwc (IntrinsicsHexagonDep.td:3710)
    hexagon_M7_dcmpyiwc_acc,                   // llvm.hexagon.M7.dcmpyiwc.acc (IntrinsicsHexagonDep.td:3713)
    hexagon_M7_dcmpyrw,                        // llvm.hexagon.M7.dcmpyrw (IntrinsicsHexagonDep.td:3716)
    hexagon_M7_dcmpyrw_acc,                    // llvm.hexagon.M7.dcmpyrw.acc (IntrinsicsHexagonDep.td:3719)
    hexagon_M7_dcmpyrwc,                       // llvm.hexagon.M7.dcmpyrwc (IntrinsicsHexagonDep.td:3722)
    hexagon_M7_dcmpyrwc_acc,                   // llvm.hexagon.M7.dcmpyrwc.acc (IntrinsicsHexagonDep.td:3725)
    hexagon_M7_vdmpy,                          // llvm.hexagon.M7.vdmpy (IntrinsicsHexagonDep.td:3728)
    hexagon_M7_vdmpy_acc,                      // llvm.hexagon.M7.vdmpy.acc (IntrinsicsHexagonDep.td:3731)
    hexagon_M7_wcmpyiw,                        // llvm.hexagon.M7.wcmpyiw (IntrinsicsHexagonDep.td:3734)
    hexagon_M7_wcmpyiw_rnd,                    // llvm.hexagon.M7.wcmpyiw.rnd (IntrinsicsHexagonDep.td:3737)
    hexagon_M7_wcmpyiwc,                       // llvm.hexagon.M7.wcmpyiwc (IntrinsicsHexagonDep.td:3740)
    hexagon_M7_wcmpyiwc_rnd,                   // llvm.hexagon.M7.wcmpyiwc.rnd (IntrinsicsHexagonDep.td:3743)
    hexagon_M7_wcmpyrw,                        // llvm.hexagon.M7.wcmpyrw (IntrinsicsHexagonDep.td:3746)
    hexagon_M7_wcmpyrw_rnd,                    // llvm.hexagon.M7.wcmpyrw.rnd (IntrinsicsHexagonDep.td:3749)
    hexagon_M7_wcmpyrwc,                       // llvm.hexagon.M7.wcmpyrwc (IntrinsicsHexagonDep.td:3752)
    hexagon_M7_wcmpyrwc_rnd,                   // llvm.hexagon.M7.wcmpyrwc.rnd (IntrinsicsHexagonDep.td:3755)
    hexagon_S2_addasl_rrri,                    // llvm.hexagon.S2.addasl.rrri (IntrinsicsHexagonDep.td:2950)
    hexagon_S2_asl_i_p,                        // llvm.hexagon.S2.asl.i.p (IntrinsicsHexagonDep.td:2953)
    hexagon_S2_asl_i_p_acc,                    // llvm.hexagon.S2.asl.i.p.acc (IntrinsicsHexagonDep.td:2956)
    hexagon_S2_asl_i_p_and,                    // llvm.hexagon.S2.asl.i.p.and (IntrinsicsHexagonDep.td:2959)
    hexagon_S2_asl_i_p_nac,                    // llvm.hexagon.S2.asl.i.p.nac (IntrinsicsHexagonDep.td:2962)
    hexagon_S2_asl_i_p_or,                     // llvm.hexagon.S2.asl.i.p.or (IntrinsicsHexagonDep.td:2965)
    hexagon_S2_asl_i_p_xacc,                   // llvm.hexagon.S2.asl.i.p.xacc (IntrinsicsHexagonDep.td:2968)
    hexagon_S2_asl_i_r,                        // llvm.hexagon.S2.asl.i.r (IntrinsicsHexagonDep.td:2971)
    hexagon_S2_asl_i_r_acc,                    // llvm.hexagon.S2.asl.i.r.acc (IntrinsicsHexagonDep.td:2974)
    hexagon_S2_asl_i_r_and,                    // llvm.hexagon.S2.asl.i.r.and (IntrinsicsHexagonDep.td:2977)
    hexagon_S2_asl_i_r_nac,                    // llvm.hexagon.S2.asl.i.r.nac (IntrinsicsHexagonDep.td:2980)
    hexagon_S2_asl_i_r_or,                     // llvm.hexagon.S2.asl.i.r.or (IntrinsicsHexagonDep.td:2983)
    hexagon_S2_asl_i_r_sat,                    // llvm.hexagon.S2.asl.i.r.sat (IntrinsicsHexagonDep.td:2986)
    hexagon_S2_asl_i_r_xacc,                   // llvm.hexagon.S2.asl.i.r.xacc (IntrinsicsHexagonDep.td:2989)
    hexagon_S2_asl_i_vh,                       // llvm.hexagon.S2.asl.i.vh (IntrinsicsHexagonDep.td:2992)
    hexagon_S2_asl_i_vw,                       // llvm.hexagon.S2.asl.i.vw (IntrinsicsHexagonDep.td:2995)
    hexagon_S2_asl_r_p,                        // llvm.hexagon.S2.asl.r.p (IntrinsicsHexagonDep.td:2998)
    hexagon_S2_asl_r_p_acc,                    // llvm.hexagon.S2.asl.r.p.acc (IntrinsicsHexagonDep.td:3001)
    hexagon_S2_asl_r_p_and,                    // llvm.hexagon.S2.asl.r.p.and (IntrinsicsHexagonDep.td:3004)
    hexagon_S2_asl_r_p_nac,                    // llvm.hexagon.S2.asl.r.p.nac (IntrinsicsHexagonDep.td:3007)
    hexagon_S2_asl_r_p_or,                     // llvm.hexagon.S2.asl.r.p.or (IntrinsicsHexagonDep.td:3010)
    hexagon_S2_asl_r_p_xor,                    // llvm.hexagon.S2.asl.r.p.xor (IntrinsicsHexagonDep.td:3013)
    hexagon_S2_asl_r_r,                        // llvm.hexagon.S2.asl.r.r (IntrinsicsHexagonDep.td:3016)
    hexagon_S2_asl_r_r_acc,                    // llvm.hexagon.S2.asl.r.r.acc (IntrinsicsHexagonDep.td:3019)
    hexagon_S2_asl_r_r_and,                    // llvm.hexagon.S2.asl.r.r.and (IntrinsicsHexagonDep.td:3022)
    hexagon_S2_asl_r_r_nac,                    // llvm.hexagon.S2.asl.r.r.nac (IntrinsicsHexagonDep.td:3025)
    hexagon_S2_asl_r_r_or,                     // llvm.hexagon.S2.asl.r.r.or (IntrinsicsHexagonDep.td:3028)
    hexagon_S2_asl_r_r_sat,                    // llvm.hexagon.S2.asl.r.r.sat (IntrinsicsHexagonDep.td:3031)
    hexagon_S2_asl_r_vh,                       // llvm.hexagon.S2.asl.r.vh (IntrinsicsHexagonDep.td:3034)
    hexagon_S2_asl_r_vw,                       // llvm.hexagon.S2.asl.r.vw (IntrinsicsHexagonDep.td:3037)
    hexagon_S2_asr_i_p,                        // llvm.hexagon.S2.asr.i.p (IntrinsicsHexagonDep.td:3040)
    hexagon_S2_asr_i_p_acc,                    // llvm.hexagon.S2.asr.i.p.acc (IntrinsicsHexagonDep.td:3043)
    hexagon_S2_asr_i_p_and,                    // llvm.hexagon.S2.asr.i.p.and (IntrinsicsHexagonDep.td:3046)
    hexagon_S2_asr_i_p_nac,                    // llvm.hexagon.S2.asr.i.p.nac (IntrinsicsHexagonDep.td:3049)
    hexagon_S2_asr_i_p_or,                     // llvm.hexagon.S2.asr.i.p.or (IntrinsicsHexagonDep.td:3052)
    hexagon_S2_asr_i_p_rnd,                    // llvm.hexagon.S2.asr.i.p.rnd (IntrinsicsHexagonDep.td:3055)
    hexagon_S2_asr_i_p_rnd_goodsyntax,         // llvm.hexagon.S2.asr.i.p.rnd.goodsyntax (IntrinsicsHexagonDep.td:3058)
    hexagon_S2_asr_i_r,                        // llvm.hexagon.S2.asr.i.r (IntrinsicsHexagonDep.td:3061)
    hexagon_S2_asr_i_r_acc,                    // llvm.hexagon.S2.asr.i.r.acc (IntrinsicsHexagonDep.td:3064)
    hexagon_S2_asr_i_r_and,                    // llvm.hexagon.S2.asr.i.r.and (IntrinsicsHexagonDep.td:3067)
    hexagon_S2_asr_i_r_nac,                    // llvm.hexagon.S2.asr.i.r.nac (IntrinsicsHexagonDep.td:3070)
    hexagon_S2_asr_i_r_or,                     // llvm.hexagon.S2.asr.i.r.or (IntrinsicsHexagonDep.td:3073)
    hexagon_S2_asr_i_r_rnd,                    // llvm.hexagon.S2.asr.i.r.rnd (IntrinsicsHexagonDep.td:3076)
    hexagon_S2_asr_i_r_rnd_goodsyntax,         // llvm.hexagon.S2.asr.i.r.rnd.goodsyntax (IntrinsicsHexagonDep.td:3079)
    hexagon_S2_asr_i_svw_trun,                 // llvm.hexagon.S2.asr.i.svw.trun (IntrinsicsHexagonDep.td:3082)
    hexagon_S2_asr_i_vh,                       // llvm.hexagon.S2.asr.i.vh (IntrinsicsHexagonDep.td:3085)
    hexagon_S2_asr_i_vw,                       // llvm.hexagon.S2.asr.i.vw (IntrinsicsHexagonDep.td:3088)
    hexagon_S2_asr_r_p,                        // llvm.hexagon.S2.asr.r.p (IntrinsicsHexagonDep.td:3091)
    hexagon_S2_asr_r_p_acc,                    // llvm.hexagon.S2.asr.r.p.acc (IntrinsicsHexagonDep.td:3094)
    hexagon_S2_asr_r_p_and,                    // llvm.hexagon.S2.asr.r.p.and (IntrinsicsHexagonDep.td:3097)
    hexagon_S2_asr_r_p_nac,                    // llvm.hexagon.S2.asr.r.p.nac (IntrinsicsHexagonDep.td:3100)
    hexagon_S2_asr_r_p_or,                     // llvm.hexagon.S2.asr.r.p.or (IntrinsicsHexagonDep.td:3103)
    hexagon_S2_asr_r_p_xor,                    // llvm.hexagon.S2.asr.r.p.xor (IntrinsicsHexagonDep.td:3106)
    hexagon_S2_asr_r_r,                        // llvm.hexagon.S2.asr.r.r (IntrinsicsHexagonDep.td:3109)
    hexagon_S2_asr_r_r_acc,                    // llvm.hexagon.S2.asr.r.r.acc (IntrinsicsHexagonDep.td:3112)
    hexagon_S2_asr_r_r_and,                    // llvm.hexagon.S2.asr.r.r.and (IntrinsicsHexagonDep.td:3115)
    hexagon_S2_asr_r_r_nac,                    // llvm.hexagon.S2.asr.r.r.nac (IntrinsicsHexagonDep.td:3118)
    hexagon_S2_asr_r_r_or,                     // llvm.hexagon.S2.asr.r.r.or (IntrinsicsHexagonDep.td:3121)
    hexagon_S2_asr_r_r_sat,                    // llvm.hexagon.S2.asr.r.r.sat (IntrinsicsHexagonDep.td:3124)
    hexagon_S2_asr_r_svw_trun,                 // llvm.hexagon.S2.asr.r.svw.trun (IntrinsicsHexagonDep.td:3127)
    hexagon_S2_asr_r_vh,                       // llvm.hexagon.S2.asr.r.vh (IntrinsicsHexagonDep.td:3130)
    hexagon_S2_asr_r_vw,                       // llvm.hexagon.S2.asr.r.vw (IntrinsicsHexagonDep.td:3133)
    hexagon_S2_brev,                           // llvm.hexagon.S2.brev (IntrinsicsHexagonDep.td:3136)
    hexagon_S2_brevp,                          // llvm.hexagon.S2.brevp (IntrinsicsHexagonDep.td:3139)
    hexagon_S2_cl0,                            // llvm.hexagon.S2.cl0 (IntrinsicsHexagonDep.td:3142)
    hexagon_S2_cl0p,                           // llvm.hexagon.S2.cl0p (IntrinsicsHexagonDep.td:3145)
    hexagon_S2_cl1,                            // llvm.hexagon.S2.cl1 (IntrinsicsHexagonDep.td:3148)
    hexagon_S2_cl1p,                           // llvm.hexagon.S2.cl1p (IntrinsicsHexagonDep.td:3151)
    hexagon_S2_clb,                            // llvm.hexagon.S2.clb (IntrinsicsHexagonDep.td:3154)
    hexagon_S2_clbnorm,                        // llvm.hexagon.S2.clbnorm (IntrinsicsHexagonDep.td:3157)
    hexagon_S2_clbp,                           // llvm.hexagon.S2.clbp (IntrinsicsHexagonDep.td:3160)
    hexagon_S2_clrbit_i,                       // llvm.hexagon.S2.clrbit.i (IntrinsicsHexagonDep.td:3163)
    hexagon_S2_clrbit_r,                       // llvm.hexagon.S2.clrbit.r (IntrinsicsHexagonDep.td:3166)
    hexagon_S2_ct0,                            // llvm.hexagon.S2.ct0 (IntrinsicsHexagonDep.td:3169)
    hexagon_S2_ct0p,                           // llvm.hexagon.S2.ct0p (IntrinsicsHexagonDep.td:3172)
    hexagon_S2_ct1,                            // llvm.hexagon.S2.ct1 (IntrinsicsHexagonDep.td:3175)
    hexagon_S2_ct1p,                           // llvm.hexagon.S2.ct1p (IntrinsicsHexagonDep.td:3178)
    hexagon_S2_deinterleave,                   // llvm.hexagon.S2.deinterleave (IntrinsicsHexagonDep.td:3181)
    hexagon_S2_extractu,                       // llvm.hexagon.S2.extractu (IntrinsicsHexagonDep.td:3184)
    hexagon_S2_extractu_rp,                    // llvm.hexagon.S2.extractu.rp (IntrinsicsHexagonDep.td:3187)
    hexagon_S2_extractup,                      // llvm.hexagon.S2.extractup (IntrinsicsHexagonDep.td:3190)
    hexagon_S2_extractup_rp,                   // llvm.hexagon.S2.extractup.rp (IntrinsicsHexagonDep.td:3193)
    hexagon_S2_insert,                         // llvm.hexagon.S2.insert (IntrinsicsHexagonDep.td:3196)
    hexagon_S2_insert_rp,                      // llvm.hexagon.S2.insert.rp (IntrinsicsHexagonDep.td:3199)
    hexagon_S2_insertp,                        // llvm.hexagon.S2.insertp (IntrinsicsHexagonDep.td:3202)
    hexagon_S2_insertp_rp,                     // llvm.hexagon.S2.insertp.rp (IntrinsicsHexagonDep.td:3205)
    hexagon_S2_interleave,                     // llvm.hexagon.S2.interleave (IntrinsicsHexagonDep.td:3208)
    hexagon_S2_lfsp,                           // llvm.hexagon.S2.lfsp (IntrinsicsHexagonDep.td:3211)
    hexagon_S2_lsl_r_p,                        // llvm.hexagon.S2.lsl.r.p (IntrinsicsHexagonDep.td:3214)
    hexagon_S2_lsl_r_p_acc,                    // llvm.hexagon.S2.lsl.r.p.acc (IntrinsicsHexagonDep.td:3217)
    hexagon_S2_lsl_r_p_and,                    // llvm.hexagon.S2.lsl.r.p.and (IntrinsicsHexagonDep.td:3220)
    hexagon_S2_lsl_r_p_nac,                    // llvm.hexagon.S2.lsl.r.p.nac (IntrinsicsHexagonDep.td:3223)
    hexagon_S2_lsl_r_p_or,                     // llvm.hexagon.S2.lsl.r.p.or (IntrinsicsHexagonDep.td:3226)
    hexagon_S2_lsl_r_p_xor,                    // llvm.hexagon.S2.lsl.r.p.xor (IntrinsicsHexagonDep.td:3229)
    hexagon_S2_lsl_r_r,                        // llvm.hexagon.S2.lsl.r.r (IntrinsicsHexagonDep.td:3232)
    hexagon_S2_lsl_r_r_acc,                    // llvm.hexagon.S2.lsl.r.r.acc (IntrinsicsHexagonDep.td:3235)
    hexagon_S2_lsl_r_r_and,                    // llvm.hexagon.S2.lsl.r.r.and (IntrinsicsHexagonDep.td:3238)
    hexagon_S2_lsl_r_r_nac,                    // llvm.hexagon.S2.lsl.r.r.nac (IntrinsicsHexagonDep.td:3241)
    hexagon_S2_lsl_r_r_or,                     // llvm.hexagon.S2.lsl.r.r.or (IntrinsicsHexagonDep.td:3244)
    hexagon_S2_lsl_r_vh,                       // llvm.hexagon.S2.lsl.r.vh (IntrinsicsHexagonDep.td:3247)
    hexagon_S2_lsl_r_vw,                       // llvm.hexagon.S2.lsl.r.vw (IntrinsicsHexagonDep.td:3250)
    hexagon_S2_lsr_i_p,                        // llvm.hexagon.S2.lsr.i.p (IntrinsicsHexagonDep.td:3253)
    hexagon_S2_lsr_i_p_acc,                    // llvm.hexagon.S2.lsr.i.p.acc (IntrinsicsHexagonDep.td:3256)
    hexagon_S2_lsr_i_p_and,                    // llvm.hexagon.S2.lsr.i.p.and (IntrinsicsHexagonDep.td:3259)
    hexagon_S2_lsr_i_p_nac,                    // llvm.hexagon.S2.lsr.i.p.nac (IntrinsicsHexagonDep.td:3262)
    hexagon_S2_lsr_i_p_or,                     // llvm.hexagon.S2.lsr.i.p.or (IntrinsicsHexagonDep.td:3265)
    hexagon_S2_lsr_i_p_xacc,                   // llvm.hexagon.S2.lsr.i.p.xacc (IntrinsicsHexagonDep.td:3268)
    hexagon_S2_lsr_i_r,                        // llvm.hexagon.S2.lsr.i.r (IntrinsicsHexagonDep.td:3271)
    hexagon_S2_lsr_i_r_acc,                    // llvm.hexagon.S2.lsr.i.r.acc (IntrinsicsHexagonDep.td:3274)
    hexagon_S2_lsr_i_r_and,                    // llvm.hexagon.S2.lsr.i.r.and (IntrinsicsHexagonDep.td:3277)
    hexagon_S2_lsr_i_r_nac,                    // llvm.hexagon.S2.lsr.i.r.nac (IntrinsicsHexagonDep.td:3280)
    hexagon_S2_lsr_i_r_or,                     // llvm.hexagon.S2.lsr.i.r.or (IntrinsicsHexagonDep.td:3283)
    hexagon_S2_lsr_i_r_xacc,                   // llvm.hexagon.S2.lsr.i.r.xacc (IntrinsicsHexagonDep.td:3286)
    hexagon_S2_lsr_i_vh,                       // llvm.hexagon.S2.lsr.i.vh (IntrinsicsHexagonDep.td:3289)
    hexagon_S2_lsr_i_vw,                       // llvm.hexagon.S2.lsr.i.vw (IntrinsicsHexagonDep.td:3292)
    hexagon_S2_lsr_r_p,                        // llvm.hexagon.S2.lsr.r.p (IntrinsicsHexagonDep.td:3295)
    hexagon_S2_lsr_r_p_acc,                    // llvm.hexagon.S2.lsr.r.p.acc (IntrinsicsHexagonDep.td:3298)
    hexagon_S2_lsr_r_p_and,                    // llvm.hexagon.S2.lsr.r.p.and (IntrinsicsHexagonDep.td:3301)
    hexagon_S2_lsr_r_p_nac,                    // llvm.hexagon.S2.lsr.r.p.nac (IntrinsicsHexagonDep.td:3304)
    hexagon_S2_lsr_r_p_or,                     // llvm.hexagon.S2.lsr.r.p.or (IntrinsicsHexagonDep.td:3307)
    hexagon_S2_lsr_r_p_xor,                    // llvm.hexagon.S2.lsr.r.p.xor (IntrinsicsHexagonDep.td:3310)
    hexagon_S2_lsr_r_r,                        // llvm.hexagon.S2.lsr.r.r (IntrinsicsHexagonDep.td:3313)
    hexagon_S2_lsr_r_r_acc,                    // llvm.hexagon.S2.lsr.r.r.acc (IntrinsicsHexagonDep.td:3316)
    hexagon_S2_lsr_r_r_and,                    // llvm.hexagon.S2.lsr.r.r.and (IntrinsicsHexagonDep.td:3319)
    hexagon_S2_lsr_r_r_nac,                    // llvm.hexagon.S2.lsr.r.r.nac (IntrinsicsHexagonDep.td:3322)
    hexagon_S2_lsr_r_r_or,                     // llvm.hexagon.S2.lsr.r.r.or (IntrinsicsHexagonDep.td:3325)
    hexagon_S2_lsr_r_vh,                       // llvm.hexagon.S2.lsr.r.vh (IntrinsicsHexagonDep.td:3328)
    hexagon_S2_lsr_r_vw,                       // llvm.hexagon.S2.lsr.r.vw (IntrinsicsHexagonDep.td:3331)
    hexagon_S2_mask,                           // llvm.hexagon.S2.mask (IntrinsicsHexagonDep.td:3669)
    hexagon_S2_packhl,                         // llvm.hexagon.S2.packhl (IntrinsicsHexagonDep.td:3334)
    hexagon_S2_parityp,                        // llvm.hexagon.S2.parityp (IntrinsicsHexagonDep.td:3337)
    hexagon_S2_setbit_i,                       // llvm.hexagon.S2.setbit.i (IntrinsicsHexagonDep.td:3340)
    hexagon_S2_setbit_r,                       // llvm.hexagon.S2.setbit.r (IntrinsicsHexagonDep.td:3343)
    hexagon_S2_shuffeb,                        // llvm.hexagon.S2.shuffeb (IntrinsicsHexagonDep.td:3346)
    hexagon_S2_shuffeh,                        // llvm.hexagon.S2.shuffeh (IntrinsicsHexagonDep.td:3349)
    hexagon_S2_shuffob,                        // llvm.hexagon.S2.shuffob (IntrinsicsHexagonDep.td:3352)
    hexagon_S2_shuffoh,                        // llvm.hexagon.S2.shuffoh (IntrinsicsHexagonDep.td:3355)
    hexagon_S2_storerb_pbr,                    // llvm.hexagon.S2.storerb.pbr (IntrinsicsHexagon.td:207)
    hexagon_S2_storerb_pci,                    // llvm.hexagon.S2.storerb.pci (IntrinsicsHexagon.td:177)
    hexagon_S2_storerb_pcr,                    // llvm.hexagon.S2.storerb.pcr (IntrinsicsHexagon.td:181)
    hexagon_S2_storerd_pbr,                    // llvm.hexagon.S2.storerd.pbr (IntrinsicsHexagon.td:211)
    hexagon_S2_storerd_pci,                    // llvm.hexagon.S2.storerd.pci (IntrinsicsHexagon.td:177)
    hexagon_S2_storerd_pcr,                    // llvm.hexagon.S2.storerd.pcr (IntrinsicsHexagon.td:181)
    hexagon_S2_storerf_pbr,                    // llvm.hexagon.S2.storerf.pbr (IntrinsicsHexagon.td:209)
    hexagon_S2_storerf_pci,                    // llvm.hexagon.S2.storerf.pci (IntrinsicsHexagon.td:177)
    hexagon_S2_storerf_pcr,                    // llvm.hexagon.S2.storerf.pcr (IntrinsicsHexagon.td:181)
    hexagon_S2_storerh_pbr,                    // llvm.hexagon.S2.storerh.pbr (IntrinsicsHexagon.td:208)
    hexagon_S2_storerh_pci,                    // llvm.hexagon.S2.storerh.pci (IntrinsicsHexagon.td:177)
    hexagon_S2_storerh_pcr,                    // llvm.hexagon.S2.storerh.pcr (IntrinsicsHexagon.td:181)
    hexagon_S2_storeri_pbr,                    // llvm.hexagon.S2.storeri.pbr (IntrinsicsHexagon.td:210)
    hexagon_S2_storeri_pci,                    // llvm.hexagon.S2.storeri.pci (IntrinsicsHexagon.td:177)
    hexagon_S2_storeri_pcr,                    // llvm.hexagon.S2.storeri.pcr (IntrinsicsHexagon.td:181)
    hexagon_S2_storew_locked,                  // llvm.hexagon.S2.storew.locked (IntrinsicsHexagon.td:141)
    hexagon_S2_svsathb,                        // llvm.hexagon.S2.svsathb (IntrinsicsHexagonDep.td:3358)
    hexagon_S2_svsathub,                       // llvm.hexagon.S2.svsathub (IntrinsicsHexagonDep.td:3361)
    hexagon_S2_tableidxb_goodsyntax,           // llvm.hexagon.S2.tableidxb.goodsyntax (IntrinsicsHexagonDep.td:3364)
    hexagon_S2_tableidxd_goodsyntax,           // llvm.hexagon.S2.tableidxd.goodsyntax (IntrinsicsHexagonDep.td:3367)
    hexagon_S2_tableidxh_goodsyntax,           // llvm.hexagon.S2.tableidxh.goodsyntax (IntrinsicsHexagonDep.td:3370)
    hexagon_S2_tableidxw_goodsyntax,           // llvm.hexagon.S2.tableidxw.goodsyntax (IntrinsicsHexagonDep.td:3373)
    hexagon_S2_togglebit_i,                    // llvm.hexagon.S2.togglebit.i (IntrinsicsHexagonDep.td:3376)
    hexagon_S2_togglebit_r,                    // llvm.hexagon.S2.togglebit.r (IntrinsicsHexagonDep.td:3379)
    hexagon_S2_tstbit_i,                       // llvm.hexagon.S2.tstbit.i (IntrinsicsHexagonDep.td:3382)
    hexagon_S2_tstbit_r,                       // llvm.hexagon.S2.tstbit.r (IntrinsicsHexagonDep.td:3385)
    hexagon_S2_valignib,                       // llvm.hexagon.S2.valignib (IntrinsicsHexagonDep.td:3388)
    hexagon_S2_valignrb,                       // llvm.hexagon.S2.valignrb (IntrinsicsHexagonDep.td:3391)
    hexagon_S2_vcnegh,                         // llvm.hexagon.S2.vcnegh (IntrinsicsHexagonDep.td:3394)
    hexagon_S2_vcrotate,                       // llvm.hexagon.S2.vcrotate (IntrinsicsHexagonDep.td:3397)
    hexagon_S2_vrcnegh,                        // llvm.hexagon.S2.vrcnegh (IntrinsicsHexagonDep.td:3400)
    hexagon_S2_vrndpackwh,                     // llvm.hexagon.S2.vrndpackwh (IntrinsicsHexagonDep.td:3403)
    hexagon_S2_vrndpackwhs,                    // llvm.hexagon.S2.vrndpackwhs (IntrinsicsHexagonDep.td:3406)
    hexagon_S2_vsathb,                         // llvm.hexagon.S2.vsathb (IntrinsicsHexagonDep.td:3409)
    hexagon_S2_vsathb_nopack,                  // llvm.hexagon.S2.vsathb.nopack (IntrinsicsHexagonDep.td:3412)
    hexagon_S2_vsathub,                        // llvm.hexagon.S2.vsathub (IntrinsicsHexagonDep.td:3415)
    hexagon_S2_vsathub_nopack,                 // llvm.hexagon.S2.vsathub.nopack (IntrinsicsHexagonDep.td:3418)
    hexagon_S2_vsatwh,                         // llvm.hexagon.S2.vsatwh (IntrinsicsHexagonDep.td:3421)
    hexagon_S2_vsatwh_nopack,                  // llvm.hexagon.S2.vsatwh.nopack (IntrinsicsHexagonDep.td:3424)
    hexagon_S2_vsatwuh,                        // llvm.hexagon.S2.vsatwuh (IntrinsicsHexagonDep.td:3427)
    hexagon_S2_vsatwuh_nopack,                 // llvm.hexagon.S2.vsatwuh.nopack (IntrinsicsHexagonDep.td:3430)
    hexagon_S2_vsplatrb,                       // llvm.hexagon.S2.vsplatrb (IntrinsicsHexagonDep.td:3433)
    hexagon_S2_vsplatrh,                       // llvm.hexagon.S2.vsplatrh (IntrinsicsHexagonDep.td:3436)
    hexagon_S2_vspliceib,                      // llvm.hexagon.S2.vspliceib (IntrinsicsHexagonDep.td:3439)
    hexagon_S2_vsplicerb,                      // llvm.hexagon.S2.vsplicerb (IntrinsicsHexagonDep.td:3442)
    hexagon_S2_vsxtbh,                         // llvm.hexagon.S2.vsxtbh (IntrinsicsHexagonDep.td:3445)
    hexagon_S2_vsxthw,                         // llvm.hexagon.S2.vsxthw (IntrinsicsHexagonDep.td:3448)
    hexagon_S2_vtrunehb,                       // llvm.hexagon.S2.vtrunehb (IntrinsicsHexagonDep.td:3451)
    hexagon_S2_vtrunewh,                       // llvm.hexagon.S2.vtrunewh (IntrinsicsHexagonDep.td:3454)
    hexagon_S2_vtrunohb,                       // llvm.hexagon.S2.vtrunohb (IntrinsicsHexagonDep.td:3457)
    hexagon_S2_vtrunowh,                       // llvm.hexagon.S2.vtrunowh (IntrinsicsHexagonDep.td:3460)
    hexagon_S2_vzxtbh,                         // llvm.hexagon.S2.vzxtbh (IntrinsicsHexagonDep.td:3463)
    hexagon_S2_vzxthw,                         // llvm.hexagon.S2.vzxthw (IntrinsicsHexagonDep.td:3466)
    hexagon_S4_addaddi,                        // llvm.hexagon.S4.addaddi (IntrinsicsHexagonDep.td:3469)
    hexagon_S4_addi_asl_ri,                    // llvm.hexagon.S4.addi.asl.ri (IntrinsicsHexagonDep.td:3472)
    hexagon_S4_addi_lsr_ri,                    // llvm.hexagon.S4.addi.lsr.ri (IntrinsicsHexagonDep.td:3475)
    hexagon_S4_andi_asl_ri,                    // llvm.hexagon.S4.andi.asl.ri (IntrinsicsHexagonDep.td:3478)
    hexagon_S4_andi_lsr_ri,                    // llvm.hexagon.S4.andi.lsr.ri (IntrinsicsHexagonDep.td:3481)
    hexagon_S4_clbaddi,                        // llvm.hexagon.S4.clbaddi (IntrinsicsHexagonDep.td:3484)
    hexagon_S4_clbpaddi,                       // llvm.hexagon.S4.clbpaddi (IntrinsicsHexagonDep.td:3487)
    hexagon_S4_clbpnorm,                       // llvm.hexagon.S4.clbpnorm (IntrinsicsHexagonDep.td:3490)
    hexagon_S4_extract,                        // llvm.hexagon.S4.extract (IntrinsicsHexagonDep.td:3493)
    hexagon_S4_extract_rp,                     // llvm.hexagon.S4.extract.rp (IntrinsicsHexagonDep.td:3496)
    hexagon_S4_extractp,                       // llvm.hexagon.S4.extractp (IntrinsicsHexagonDep.td:3499)
    hexagon_S4_extractp_rp,                    // llvm.hexagon.S4.extractp.rp (IntrinsicsHexagonDep.td:3502)
    hexagon_S4_lsli,                           // llvm.hexagon.S4.lsli (IntrinsicsHexagonDep.td:3505)
    hexagon_S4_ntstbit_i,                      // llvm.hexagon.S4.ntstbit.i (IntrinsicsHexagonDep.td:3508)
    hexagon_S4_ntstbit_r,                      // llvm.hexagon.S4.ntstbit.r (IntrinsicsHexagonDep.td:3511)
    hexagon_S4_or_andi,                        // llvm.hexagon.S4.or.andi (IntrinsicsHexagonDep.td:3514)
    hexagon_S4_or_andix,                       // llvm.hexagon.S4.or.andix (IntrinsicsHexagonDep.td:3517)
    hexagon_S4_or_ori,                         // llvm.hexagon.S4.or.ori (IntrinsicsHexagonDep.td:3520)
    hexagon_S4_ori_asl_ri,                     // llvm.hexagon.S4.ori.asl.ri (IntrinsicsHexagonDep.td:3523)
    hexagon_S4_ori_lsr_ri,                     // llvm.hexagon.S4.ori.lsr.ri (IntrinsicsHexagonDep.td:3526)
    hexagon_S4_parity,                         // llvm.hexagon.S4.parity (IntrinsicsHexagonDep.td:3529)
    hexagon_S4_stored_locked,                  // llvm.hexagon.S4.stored.locked (IntrinsicsHexagon.td:145)
    hexagon_S4_subaddi,                        // llvm.hexagon.S4.subaddi (IntrinsicsHexagonDep.td:3532)
    hexagon_S4_subi_asl_ri,                    // llvm.hexagon.S4.subi.asl.ri (IntrinsicsHexagonDep.td:3535)
    hexagon_S4_subi_lsr_ri,                    // llvm.hexagon.S4.subi.lsr.ri (IntrinsicsHexagonDep.td:3538)
    hexagon_S4_vrcrotate,                      // llvm.hexagon.S4.vrcrotate (IntrinsicsHexagonDep.td:3541)
    hexagon_S4_vrcrotate_acc,                  // llvm.hexagon.S4.vrcrotate.acc (IntrinsicsHexagonDep.td:3544)
    hexagon_S4_vxaddsubh,                      // llvm.hexagon.S4.vxaddsubh (IntrinsicsHexagonDep.td:3547)
    hexagon_S4_vxaddsubhr,                     // llvm.hexagon.S4.vxaddsubhr (IntrinsicsHexagonDep.td:3550)
    hexagon_S4_vxaddsubw,                      // llvm.hexagon.S4.vxaddsubw (IntrinsicsHexagonDep.td:3553)
    hexagon_S4_vxsubaddh,                      // llvm.hexagon.S4.vxsubaddh (IntrinsicsHexagonDep.td:3556)
    hexagon_S4_vxsubaddhr,                     // llvm.hexagon.S4.vxsubaddhr (IntrinsicsHexagonDep.td:3559)
    hexagon_S4_vxsubaddw,                      // llvm.hexagon.S4.vxsubaddw (IntrinsicsHexagonDep.td:3562)
    hexagon_S5_asrhub_rnd_sat_goodsyntax,      // llvm.hexagon.S5.asrhub.rnd.sat.goodsyntax (IntrinsicsHexagonDep.td:3565)
    hexagon_S5_asrhub_sat,                     // llvm.hexagon.S5.asrhub.sat (IntrinsicsHexagonDep.td:3568)
    hexagon_S5_popcountp,                      // llvm.hexagon.S5.popcountp (IntrinsicsHexagonDep.td:3571)
    hexagon_S5_vasrhrnd_goodsyntax,            // llvm.hexagon.S5.vasrhrnd.goodsyntax (IntrinsicsHexagonDep.td:3574)
    hexagon_S6_rol_i_p,                        // llvm.hexagon.S6.rol.i.p (IntrinsicsHexagonDep.td:3600)
    hexagon_S6_rol_i_p_acc,                    // llvm.hexagon.S6.rol.i.p.acc (IntrinsicsHexagonDep.td:3603)
    hexagon_S6_rol_i_p_and,                    // llvm.hexagon.S6.rol.i.p.and (IntrinsicsHexagonDep.td:3606)
    hexagon_S6_rol_i_p_nac,                    // llvm.hexagon.S6.rol.i.p.nac (IntrinsicsHexagonDep.td:3609)
    hexagon_S6_rol_i_p_or,                     // llvm.hexagon.S6.rol.i.p.or (IntrinsicsHexagonDep.td:3612)
    hexagon_S6_rol_i_p_xacc,                   // llvm.hexagon.S6.rol.i.p.xacc (IntrinsicsHexagonDep.td:3615)
    hexagon_S6_rol_i_r,                        // llvm.hexagon.S6.rol.i.r (IntrinsicsHexagonDep.td:3618)
    hexagon_S6_rol_i_r_acc,                    // llvm.hexagon.S6.rol.i.r.acc (IntrinsicsHexagonDep.td:3621)
    hexagon_S6_rol_i_r_and,                    // llvm.hexagon.S6.rol.i.r.and (IntrinsicsHexagonDep.td:3624)
    hexagon_S6_rol_i_r_nac,                    // llvm.hexagon.S6.rol.i.r.nac (IntrinsicsHexagonDep.td:3627)
    hexagon_S6_rol_i_r_or,                     // llvm.hexagon.S6.rol.i.r.or (IntrinsicsHexagonDep.td:3630)
    hexagon_S6_rol_i_r_xacc,                   // llvm.hexagon.S6.rol.i.r.xacc (IntrinsicsHexagonDep.td:3633)
    hexagon_S6_vsplatrbp,                      // llvm.hexagon.S6.vsplatrbp (IntrinsicsHexagonDep.td:3644)
    hexagon_S6_vtrunehb_ppp,                   // llvm.hexagon.S6.vtrunehb.ppp (IntrinsicsHexagonDep.td:3647)
    hexagon_S6_vtrunohb_ppp,                   // llvm.hexagon.S6.vtrunohb.ppp (IntrinsicsHexagonDep.td:3650)
    hexagon_V6_extractw,                       // llvm.hexagon.V6.extractw (IntrinsicsHexagonDep.td:3780)
    hexagon_V6_extractw_128B,                  // llvm.hexagon.V6.extractw.128B (IntrinsicsHexagonDep.td:3783)
    hexagon_V6_get_qfext,                      // llvm.hexagon.V6.get.qfext (IntrinsicsHexagonDep.td:6710)
    hexagon_V6_get_qfext_128B,                 // llvm.hexagon.V6.get.qfext.128B (IntrinsicsHexagonDep.td:6713)
    hexagon_V6_get_qfext_oracc,                // llvm.hexagon.V6.get.qfext.oracc (IntrinsicsHexagonDep.td:6716)
    hexagon_V6_get_qfext_oracc_128B,           // llvm.hexagon.V6.get.qfext.oracc.128B (IntrinsicsHexagonDep.td:6719)
    hexagon_V6_hi,                             // llvm.hexagon.V6.hi (IntrinsicsHexagonDep.td:3786)
    hexagon_V6_hi_128B,                        // llvm.hexagon.V6.hi.128B (IntrinsicsHexagonDep.td:3789)
    hexagon_V6_lo,                             // llvm.hexagon.V6.lo (IntrinsicsHexagonDep.td:3792)
    hexagon_V6_lo_128B,                        // llvm.hexagon.V6.lo.128B (IntrinsicsHexagonDep.td:3795)
    hexagon_V6_lvsplatb,                       // llvm.hexagon.V6.lvsplatb (IntrinsicsHexagonDep.td:5594)
    hexagon_V6_lvsplatb_128B,                  // llvm.hexagon.V6.lvsplatb.128B (IntrinsicsHexagonDep.td:5597)
    hexagon_V6_lvsplath,                       // llvm.hexagon.V6.lvsplath (IntrinsicsHexagonDep.td:5600)
    hexagon_V6_lvsplath_128B,                  // llvm.hexagon.V6.lvsplath.128B (IntrinsicsHexagonDep.td:5603)
    hexagon_V6_lvsplatw,                       // llvm.hexagon.V6.lvsplatw (IntrinsicsHexagonDep.td:3798)
    hexagon_V6_lvsplatw_128B,                  // llvm.hexagon.V6.lvsplatw.128B (IntrinsicsHexagonDep.td:3801)
    hexagon_V6_pred_and,                       // llvm.hexagon.V6.pred.and (IntrinsicsHexagonDep.td:3804)
    hexagon_V6_pred_and_128B,                  // llvm.hexagon.V6.pred.and.128B (IntrinsicsHexagonDep.td:3807)
    hexagon_V6_pred_and_n,                     // llvm.hexagon.V6.pred.and.n (IntrinsicsHexagonDep.td:3810)
    hexagon_V6_pred_and_n_128B,                // llvm.hexagon.V6.pred.and.n.128B (IntrinsicsHexagonDep.td:3813)
    hexagon_V6_pred_not,                       // llvm.hexagon.V6.pred.not (IntrinsicsHexagonDep.td:3816)
    hexagon_V6_pred_not_128B,                  // llvm.hexagon.V6.pred.not.128B (IntrinsicsHexagonDep.td:3819)
    hexagon_V6_pred_or,                        // llvm.hexagon.V6.pred.or (IntrinsicsHexagonDep.td:3822)
    hexagon_V6_pred_or_128B,                   // llvm.hexagon.V6.pred.or.128B (IntrinsicsHexagonDep.td:3825)
    hexagon_V6_pred_or_n,                      // llvm.hexagon.V6.pred.or.n (IntrinsicsHexagonDep.td:3828)
    hexagon_V6_pred_or_n_128B,                 // llvm.hexagon.V6.pred.or.n.128B (IntrinsicsHexagonDep.td:3831)
    hexagon_V6_pred_scalar2,                   // llvm.hexagon.V6.pred.scalar2 (IntrinsicsHexagonDep.td:3834)
    hexagon_V6_pred_scalar2_128B,              // llvm.hexagon.V6.pred.scalar2.128B (IntrinsicsHexagonDep.td:3837)
    hexagon_V6_pred_scalar2v2,                 // llvm.hexagon.V6.pred.scalar2v2 (IntrinsicsHexagonDep.td:5606)
    hexagon_V6_pred_scalar2v2_128B,            // llvm.hexagon.V6.pred.scalar2v2.128B (IntrinsicsHexagonDep.td:5609)
    hexagon_V6_pred_typecast,                  // llvm.hexagon.V6.pred.typecast (IntrinsicsHexagon.td:383)
    hexagon_V6_pred_typecast_128B,             // llvm.hexagon.V6.pred.typecast.128B (IntrinsicsHexagon.td:386)
    hexagon_V6_pred_xor,                       // llvm.hexagon.V6.pred.xor (IntrinsicsHexagonDep.td:3840)
    hexagon_V6_pred_xor_128B,                  // llvm.hexagon.V6.pred.xor.128B (IntrinsicsHexagonDep.td:3843)
    hexagon_V6_set_qfext,                      // llvm.hexagon.V6.set.qfext (IntrinsicsHexagonDep.td:6722)
    hexagon_V6_set_qfext_128B,                 // llvm.hexagon.V6.set.qfext.128B (IntrinsicsHexagonDep.td:6725)
    hexagon_V6_shuffeqh,                       // llvm.hexagon.V6.shuffeqh (IntrinsicsHexagonDep.td:5612)
    hexagon_V6_shuffeqh_128B,                  // llvm.hexagon.V6.shuffeqh.128B (IntrinsicsHexagonDep.td:5615)
    hexagon_V6_shuffeqw,                       // llvm.hexagon.V6.shuffeqw (IntrinsicsHexagonDep.td:5618)
    hexagon_V6_shuffeqw_128B,                  // llvm.hexagon.V6.shuffeqw.128B (IntrinsicsHexagonDep.td:5621)
    hexagon_V6_v6mpyhubs10,                    // llvm.hexagon.V6.v6mpyhubs10 (IntrinsicsHexagonDep.td:6158)
    hexagon_V6_v6mpyhubs10_128B,               // llvm.hexagon.V6.v6mpyhubs10.128B (IntrinsicsHexagonDep.td:6161)
    hexagon_V6_v6mpyhubs10_vxx,                // llvm.hexagon.V6.v6mpyhubs10.vxx (IntrinsicsHexagonDep.td:6164)
    hexagon_V6_v6mpyhubs10_vxx_128B,           // llvm.hexagon.V6.v6mpyhubs10.vxx.128B (IntrinsicsHexagonDep.td:6167)
    hexagon_V6_v6mpyvubs10,                    // llvm.hexagon.V6.v6mpyvubs10 (IntrinsicsHexagonDep.td:6170)
    hexagon_V6_v6mpyvubs10_128B,               // llvm.hexagon.V6.v6mpyvubs10.128B (IntrinsicsHexagonDep.td:6173)
    hexagon_V6_v6mpyvubs10_vxx,                // llvm.hexagon.V6.v6mpyvubs10.vxx (IntrinsicsHexagonDep.td:6176)
    hexagon_V6_v6mpyvubs10_vxx_128B,           // llvm.hexagon.V6.v6mpyvubs10.vxx.128B (IntrinsicsHexagonDep.td:6179)
    hexagon_V6_vL32b_npred_ai,                 // llvm.hexagon.V6.vL32b.npred.ai (IntrinsicsHexagon.td:274)
    hexagon_V6_vL32b_npred_ai_128B,            // llvm.hexagon.V6.vL32b.npred.ai.128B (IntrinsicsHexagon.td:278)
    hexagon_V6_vL32b_npred_pi,                 // llvm.hexagon.V6.vL32b.npred.pi (IntrinsicsHexagon.td:297)
    hexagon_V6_vL32b_npred_pi_128B,            // llvm.hexagon.V6.vL32b.npred.pi.128B (IntrinsicsHexagon.td:301)
    hexagon_V6_vL32b_npred_ppu,                // llvm.hexagon.V6.vL32b.npred.ppu (IntrinsicsHexagon.td:306)
    hexagon_V6_vL32b_npred_ppu_128B,           // llvm.hexagon.V6.vL32b.npred.ppu.128B (IntrinsicsHexagon.td:310)
    hexagon_V6_vL32b_nt_npred_ai,              // llvm.hexagon.V6.vL32b.nt.npred.ai (IntrinsicsHexagon.td:276)
    hexagon_V6_vL32b_nt_npred_ai_128B,         // llvm.hexagon.V6.vL32b.nt.npred.ai.128B (IntrinsicsHexagon.td:280)
    hexagon_V6_vL32b_nt_npred_pi,              // llvm.hexagon.V6.vL32b.nt.npred.pi (IntrinsicsHexagon.td:299)
    hexagon_V6_vL32b_nt_npred_pi_128B,         // llvm.hexagon.V6.vL32b.nt.npred.pi.128B (IntrinsicsHexagon.td:303)
    hexagon_V6_vL32b_nt_npred_ppu,             // llvm.hexagon.V6.vL32b.nt.npred.ppu (IntrinsicsHexagon.td:308)
    hexagon_V6_vL32b_nt_npred_ppu_128B,        // llvm.hexagon.V6.vL32b.nt.npred.ppu.128B (IntrinsicsHexagon.td:312)
    hexagon_V6_vL32b_nt_pred_ai,               // llvm.hexagon.V6.vL32b.nt.pred.ai (IntrinsicsHexagon.td:275)
    hexagon_V6_vL32b_nt_pred_ai_128B,          // llvm.hexagon.V6.vL32b.nt.pred.ai.128B (IntrinsicsHexagon.td:279)
    hexagon_V6_vL32b_nt_pred_pi,               // llvm.hexagon.V6.vL32b.nt.pred.pi (IntrinsicsHexagon.td:298)
    hexagon_V6_vL32b_nt_pred_pi_128B,          // llvm.hexagon.V6.vL32b.nt.pred.pi.128B (IntrinsicsHexagon.td:302)
    hexagon_V6_vL32b_nt_pred_ppu,              // llvm.hexagon.V6.vL32b.nt.pred.ppu (IntrinsicsHexagon.td:307)
    hexagon_V6_vL32b_nt_pred_ppu_128B,         // llvm.hexagon.V6.vL32b.nt.pred.ppu.128B (IntrinsicsHexagon.td:311)
    hexagon_V6_vL32b_pred_ai,                  // llvm.hexagon.V6.vL32b.pred.ai (IntrinsicsHexagon.td:273)
    hexagon_V6_vL32b_pred_ai_128B,             // llvm.hexagon.V6.vL32b.pred.ai.128B (IntrinsicsHexagon.td:277)
    hexagon_V6_vL32b_pred_pi,                  // llvm.hexagon.V6.vL32b.pred.pi (IntrinsicsHexagon.td:296)
    hexagon_V6_vL32b_pred_pi_128B,             // llvm.hexagon.V6.vL32b.pred.pi.128B (IntrinsicsHexagon.td:300)
    hexagon_V6_vL32b_pred_ppu,                 // llvm.hexagon.V6.vL32b.pred.ppu (IntrinsicsHexagon.td:305)
    hexagon_V6_vL32b_pred_ppu_128B,            // llvm.hexagon.V6.vL32b.pred.ppu.128B (IntrinsicsHexagon.td:309)
    hexagon_V6_vS32Ub_npred_ai,                // llvm.hexagon.V6.vS32Ub.npred.ai (IntrinsicsHexagon.td:328)
    hexagon_V6_vS32Ub_npred_ai_128B,           // llvm.hexagon.V6.vS32Ub.npred.ai.128B (IntrinsicsHexagon.td:334)
    hexagon_V6_vS32Ub_npred_pi,                // llvm.hexagon.V6.vS32Ub.npred.pi (IntrinsicsHexagon.td:355)
    hexagon_V6_vS32Ub_npred_pi_128B,           // llvm.hexagon.V6.vS32Ub.npred.pi.128B (IntrinsicsHexagon.td:361)
    hexagon_V6_vS32Ub_npred_ppu,               // llvm.hexagon.V6.vS32Ub.npred.ppu (IntrinsicsHexagon.td:368)
    hexagon_V6_vS32Ub_npred_ppu_128B,          // llvm.hexagon.V6.vS32Ub.npred.ppu.128B (IntrinsicsHexagon.td:374)
    hexagon_V6_vS32Ub_pred_ai,                 // llvm.hexagon.V6.vS32Ub.pred.ai (IntrinsicsHexagon.td:327)
    hexagon_V6_vS32Ub_pred_ai_128B,            // llvm.hexagon.V6.vS32Ub.pred.ai.128B (IntrinsicsHexagon.td:333)
    hexagon_V6_vS32Ub_pred_pi,                 // llvm.hexagon.V6.vS32Ub.pred.pi (IntrinsicsHexagon.td:354)
    hexagon_V6_vS32Ub_pred_pi_128B,            // llvm.hexagon.V6.vS32Ub.pred.pi.128B (IntrinsicsHexagon.td:360)
    hexagon_V6_vS32Ub_pred_ppu,                // llvm.hexagon.V6.vS32Ub.pred.ppu (IntrinsicsHexagon.td:367)
    hexagon_V6_vS32Ub_pred_ppu_128B,           // llvm.hexagon.V6.vS32Ub.pred.ppu.128B (IntrinsicsHexagon.td:373)
    hexagon_V6_vS32b_npred_ai,                 // llvm.hexagon.V6.vS32b.npred.ai (IntrinsicsHexagon.td:326)
    hexagon_V6_vS32b_npred_ai_128B,            // llvm.hexagon.V6.vS32b.npred.ai.128B (IntrinsicsHexagon.td:332)
    hexagon_V6_vS32b_npred_pi,                 // llvm.hexagon.V6.vS32b.npred.pi (IntrinsicsHexagon.td:353)
    hexagon_V6_vS32b_npred_pi_128B,            // llvm.hexagon.V6.vS32b.npred.pi.128B (IntrinsicsHexagon.td:359)
    hexagon_V6_vS32b_npred_ppu,                // llvm.hexagon.V6.vS32b.npred.ppu (IntrinsicsHexagon.td:366)
    hexagon_V6_vS32b_npred_ppu_128B,           // llvm.hexagon.V6.vS32b.npred.ppu.128B (IntrinsicsHexagon.td:372)
    hexagon_V6_vS32b_nqpred_ai,                // llvm.hexagon.V6.vS32b.nqpred.ai (IntrinsicsHexagonDep.td:3846)
    hexagon_V6_vS32b_nqpred_ai_128B,           // llvm.hexagon.V6.vS32b.nqpred.ai.128B (IntrinsicsHexagonDep.td:3849)
    hexagon_V6_vS32b_nt_npred_ai,              // llvm.hexagon.V6.vS32b.nt.npred.ai (IntrinsicsHexagon.td:330)
    hexagon_V6_vS32b_nt_npred_ai_128B,         // llvm.hexagon.V6.vS32b.nt.npred.ai.128B (IntrinsicsHexagon.td:336)
    hexagon_V6_vS32b_nt_npred_pi,              // llvm.hexagon.V6.vS32b.nt.npred.pi (IntrinsicsHexagon.td:357)
    hexagon_V6_vS32b_nt_npred_pi_128B,         // llvm.hexagon.V6.vS32b.nt.npred.pi.128B (IntrinsicsHexagon.td:363)
    hexagon_V6_vS32b_nt_npred_ppu,             // llvm.hexagon.V6.vS32b.nt.npred.ppu (IntrinsicsHexagon.td:370)
    hexagon_V6_vS32b_nt_npred_ppu_128B,        // llvm.hexagon.V6.vS32b.nt.npred.ppu.128B (IntrinsicsHexagon.td:376)
    hexagon_V6_vS32b_nt_nqpred_ai,             // llvm.hexagon.V6.vS32b.nt.nqpred.ai (IntrinsicsHexagonDep.td:3852)
    hexagon_V6_vS32b_nt_nqpred_ai_128B,        // llvm.hexagon.V6.vS32b.nt.nqpred.ai.128B (IntrinsicsHexagonDep.td:3855)
    hexagon_V6_vS32b_nt_pred_ai,               // llvm.hexagon.V6.vS32b.nt.pred.ai (IntrinsicsHexagon.td:329)
    hexagon_V6_vS32b_nt_pred_ai_128B,          // llvm.hexagon.V6.vS32b.nt.pred.ai.128B (IntrinsicsHexagon.td:335)
    hexagon_V6_vS32b_nt_pred_pi,               // llvm.hexagon.V6.vS32b.nt.pred.pi (IntrinsicsHexagon.td:356)
    hexagon_V6_vS32b_nt_pred_pi_128B,          // llvm.hexagon.V6.vS32b.nt.pred.pi.128B (IntrinsicsHexagon.td:362)
    hexagon_V6_vS32b_nt_pred_ppu,              // llvm.hexagon.V6.vS32b.nt.pred.ppu (IntrinsicsHexagon.td:369)
    hexagon_V6_vS32b_nt_pred_ppu_128B,         // llvm.hexagon.V6.vS32b.nt.pred.ppu.128B (IntrinsicsHexagon.td:375)
    hexagon_V6_vS32b_nt_qpred_ai,              // llvm.hexagon.V6.vS32b.nt.qpred.ai (IntrinsicsHexagonDep.td:3858)
    hexagon_V6_vS32b_nt_qpred_ai_128B,         // llvm.hexagon.V6.vS32b.nt.qpred.ai.128B (IntrinsicsHexagonDep.td:3861)
    hexagon_V6_vS32b_pred_ai,                  // llvm.hexagon.V6.vS32b.pred.ai (IntrinsicsHexagon.td:325)
    hexagon_V6_vS32b_pred_ai_128B,             // llvm.hexagon.V6.vS32b.pred.ai.128B (IntrinsicsHexagon.td:331)
    hexagon_V6_vS32b_pred_pi,                  // llvm.hexagon.V6.vS32b.pred.pi (IntrinsicsHexagon.td:352)
    hexagon_V6_vS32b_pred_pi_128B,             // llvm.hexagon.V6.vS32b.pred.pi.128B (IntrinsicsHexagon.td:358)
    hexagon_V6_vS32b_pred_ppu,                 // llvm.hexagon.V6.vS32b.pred.ppu (IntrinsicsHexagon.td:365)
    hexagon_V6_vS32b_pred_ppu_128B,            // llvm.hexagon.V6.vS32b.pred.ppu.128B (IntrinsicsHexagon.td:371)
    hexagon_V6_vS32b_qpred_ai,                 // llvm.hexagon.V6.vS32b.qpred.ai (IntrinsicsHexagonDep.td:3864)
    hexagon_V6_vS32b_qpred_ai_128B,            // llvm.hexagon.V6.vS32b.qpred.ai.128B (IntrinsicsHexagonDep.td:3867)
    hexagon_V6_vabs_f8,                        // llvm.hexagon.V6.vabs.f8 (IntrinsicsHexagonDep.td:6728)
    hexagon_V6_vabs_f8_128B,                   // llvm.hexagon.V6.vabs.f8.128B (IntrinsicsHexagonDep.td:6731)
    hexagon_V6_vabs_hf,                        // llvm.hexagon.V6.vabs.hf (IntrinsicsHexagonDep.td:6182)
    hexagon_V6_vabs_hf_128B,                   // llvm.hexagon.V6.vabs.hf.128B (IntrinsicsHexagonDep.td:6185)
    hexagon_V6_vabs_qf16_hf,                   // llvm.hexagon.V6.vabs.qf16.hf (IntrinsicsHexagonDep.td:6838)
    hexagon_V6_vabs_qf16_hf_128B,              // llvm.hexagon.V6.vabs.qf16.hf.128B (IntrinsicsHexagonDep.td:6841)
    hexagon_V6_vabs_qf16_qf16,                 // llvm.hexagon.V6.vabs.qf16.qf16 (IntrinsicsHexagonDep.td:6844)
    hexagon_V6_vabs_qf16_qf16_128B,            // llvm.hexagon.V6.vabs.qf16.qf16.128B (IntrinsicsHexagonDep.td:6847)
    hexagon_V6_vabs_qf32_qf32,                 // llvm.hexagon.V6.vabs.qf32.qf32 (IntrinsicsHexagonDep.td:6850)
    hexagon_V6_vabs_qf32_qf32_128B,            // llvm.hexagon.V6.vabs.qf32.qf32.128B (IntrinsicsHexagonDep.td:6853)
    hexagon_V6_vabs_qf32_sf,                   // llvm.hexagon.V6.vabs.qf32.sf (IntrinsicsHexagonDep.td:6856)
    hexagon_V6_vabs_qf32_sf_128B,              // llvm.hexagon.V6.vabs.qf32.sf.128B (IntrinsicsHexagonDep.td:6859)
    hexagon_V6_vabs_sf,                        // llvm.hexagon.V6.vabs.sf (IntrinsicsHexagonDep.td:6188)
    hexagon_V6_vabs_sf_128B,                   // llvm.hexagon.V6.vabs.sf.128B (IntrinsicsHexagonDep.td:6191)
    hexagon_V6_vabsb,                          // llvm.hexagon.V6.vabsb (IntrinsicsHexagonDep.td:5878)
    hexagon_V6_vabsb_128B,                     // llvm.hexagon.V6.vabsb.128B (IntrinsicsHexagonDep.td:5881)
    hexagon_V6_vabsb_sat,                      // llvm.hexagon.V6.vabsb.sat (IntrinsicsHexagonDep.td:5884)
    hexagon_V6_vabsb_sat_128B,                 // llvm.hexagon.V6.vabsb.sat.128B (IntrinsicsHexagonDep.td:5887)
    hexagon_V6_vabsdiffh,                      // llvm.hexagon.V6.vabsdiffh (IntrinsicsHexagonDep.td:3870)
    hexagon_V6_vabsdiffh_128B,                 // llvm.hexagon.V6.vabsdiffh.128B (IntrinsicsHexagonDep.td:3873)
    hexagon_V6_vabsdiffub,                     // llvm.hexagon.V6.vabsdiffub (IntrinsicsHexagonDep.td:3876)
    hexagon_V6_vabsdiffub_128B,                // llvm.hexagon.V6.vabsdiffub.128B (IntrinsicsHexagonDep.td:3879)
    hexagon_V6_vabsdiffuh,                     // llvm.hexagon.V6.vabsdiffuh (IntrinsicsHexagonDep.td:3882)
    hexagon_V6_vabsdiffuh_128B,                // llvm.hexagon.V6.vabsdiffuh.128B (IntrinsicsHexagonDep.td:3885)
    hexagon_V6_vabsdiffw,                      // llvm.hexagon.V6.vabsdiffw (IntrinsicsHexagonDep.td:3888)
    hexagon_V6_vabsdiffw_128B,                 // llvm.hexagon.V6.vabsdiffw.128B (IntrinsicsHexagonDep.td:3891)
    hexagon_V6_vabsh,                          // llvm.hexagon.V6.vabsh (IntrinsicsHexagonDep.td:3894)
    hexagon_V6_vabsh_128B,                     // llvm.hexagon.V6.vabsh.128B (IntrinsicsHexagonDep.td:3897)
    hexagon_V6_vabsh_sat,                      // llvm.hexagon.V6.vabsh.sat (IntrinsicsHexagonDep.td:3900)
    hexagon_V6_vabsh_sat_128B,                 // llvm.hexagon.V6.vabsh.sat.128B (IntrinsicsHexagonDep.td:3903)
    hexagon_V6_vabsw,                          // llvm.hexagon.V6.vabsw (IntrinsicsHexagonDep.td:3906)
    hexagon_V6_vabsw_128B,                     // llvm.hexagon.V6.vabsw.128B (IntrinsicsHexagonDep.td:3909)
    hexagon_V6_vabsw_sat,                      // llvm.hexagon.V6.vabsw.sat (IntrinsicsHexagonDep.td:3912)
    hexagon_V6_vabsw_sat_128B,                 // llvm.hexagon.V6.vabsw.sat.128B (IntrinsicsHexagonDep.td:3915)
    hexagon_V6_vadd_hf,                        // llvm.hexagon.V6.vadd.hf (IntrinsicsHexagonDep.td:6194)
    hexagon_V6_vadd_hf_128B,                   // llvm.hexagon.V6.vadd.hf.128B (IntrinsicsHexagonDep.td:6197)
    hexagon_V6_vadd_hf_f8,                     // llvm.hexagon.V6.vadd.hf.f8 (IntrinsicsHexagonDep.td:6734)
    hexagon_V6_vadd_hf_f8_128B,                // llvm.hexagon.V6.vadd.hf.f8.128B (IntrinsicsHexagonDep.td:6737)
    hexagon_V6_vadd_hf_hf,                     // llvm.hexagon.V6.vadd.hf.hf (IntrinsicsHexagonDep.td:6200)
    hexagon_V6_vadd_hf_hf_128B,                // llvm.hexagon.V6.vadd.hf.hf.128B (IntrinsicsHexagonDep.td:6203)
    hexagon_V6_vadd_qf16,                      // llvm.hexagon.V6.vadd.qf16 (IntrinsicsHexagonDep.td:6206)
    hexagon_V6_vadd_qf16_128B,                 // llvm.hexagon.V6.vadd.qf16.128B (IntrinsicsHexagonDep.td:6209)
    hexagon_V6_vadd_qf16_mix,                  // llvm.hexagon.V6.vadd.qf16.mix (IntrinsicsHexagonDep.td:6212)
    hexagon_V6_vadd_qf16_mix_128B,             // llvm.hexagon.V6.vadd.qf16.mix.128B (IntrinsicsHexagonDep.td:6215)
    hexagon_V6_vadd_qf32,                      // llvm.hexagon.V6.vadd.qf32 (IntrinsicsHexagonDep.td:6218)
    hexagon_V6_vadd_qf32_128B,                 // llvm.hexagon.V6.vadd.qf32.128B (IntrinsicsHexagonDep.td:6221)
    hexagon_V6_vadd_qf32_mix,                  // llvm.hexagon.V6.vadd.qf32.mix (IntrinsicsHexagonDep.td:6224)
    hexagon_V6_vadd_qf32_mix_128B,             // llvm.hexagon.V6.vadd.qf32.mix.128B (IntrinsicsHexagonDep.td:6227)
    hexagon_V6_vadd_sf,                        // llvm.hexagon.V6.vadd.sf (IntrinsicsHexagonDep.td:6230)
    hexagon_V6_vadd_sf_128B,                   // llvm.hexagon.V6.vadd.sf.128B (IntrinsicsHexagonDep.td:6233)
    hexagon_V6_vadd_sf_bf,                     // llvm.hexagon.V6.vadd.sf.bf (IntrinsicsHexagonDep.td:6618)
    hexagon_V6_vadd_sf_bf_128B,                // llvm.hexagon.V6.vadd.sf.bf.128B (IntrinsicsHexagonDep.td:6621)
    hexagon_V6_vadd_sf_hf,                     // llvm.hexagon.V6.vadd.sf.hf (IntrinsicsHexagonDep.td:6236)
    hexagon_V6_vadd_sf_hf_128B,                // llvm.hexagon.V6.vadd.sf.hf.128B (IntrinsicsHexagonDep.td:6239)
    hexagon_V6_vadd_sf_sf,                     // llvm.hexagon.V6.vadd.sf.sf (IntrinsicsHexagonDep.td:6242)
    hexagon_V6_vadd_sf_sf_128B,                // llvm.hexagon.V6.vadd.sf.sf.128B (IntrinsicsHexagonDep.td:6245)
    hexagon_V6_vaddb,                          // llvm.hexagon.V6.vaddb (IntrinsicsHexagonDep.td:3918)
    hexagon_V6_vaddb_128B,                     // llvm.hexagon.V6.vaddb.128B (IntrinsicsHexagonDep.td:3921)
    hexagon_V6_vaddb_dv,                       // llvm.hexagon.V6.vaddb.dv (IntrinsicsHexagonDep.td:3924)
    hexagon_V6_vaddb_dv_128B,                  // llvm.hexagon.V6.vaddb.dv.128B (IntrinsicsHexagonDep.td:3927)
    hexagon_V6_vaddbnq,                        // llvm.hexagon.V6.vaddbnq (IntrinsicsHexagonDep.td:3930)
    hexagon_V6_vaddbnq_128B,                   // llvm.hexagon.V6.vaddbnq.128B (IntrinsicsHexagonDep.td:3933)
    hexagon_V6_vaddbq,                         // llvm.hexagon.V6.vaddbq (IntrinsicsHexagonDep.td:3936)
    hexagon_V6_vaddbq_128B,                    // llvm.hexagon.V6.vaddbq.128B (IntrinsicsHexagonDep.td:3939)
    hexagon_V6_vaddbsat,                       // llvm.hexagon.V6.vaddbsat (IntrinsicsHexagonDep.td:5624)
    hexagon_V6_vaddbsat_128B,                  // llvm.hexagon.V6.vaddbsat.128B (IntrinsicsHexagonDep.td:5627)
    hexagon_V6_vaddbsat_dv,                    // llvm.hexagon.V6.vaddbsat.dv (IntrinsicsHexagonDep.td:5630)
    hexagon_V6_vaddbsat_dv_128B,               // llvm.hexagon.V6.vaddbsat.dv.128B (IntrinsicsHexagonDep.td:5633)
    hexagon_V6_vaddcarry,                      // llvm.hexagon.V6.vaddcarry (IntrinsicsHexagonDep.td:5636)
    hexagon_V6_vaddcarry_128B,                 // llvm.hexagon.V6.vaddcarry.128B (IntrinsicsHexagonDep.td:5639)
    hexagon_V6_vaddcarryo,                     // llvm.hexagon.V6.vaddcarryo (IntrinsicsHexagonDep.td:6120)
    hexagon_V6_vaddcarryo_128B,                // llvm.hexagon.V6.vaddcarryo.128B (IntrinsicsHexagonDep.td:6123)
    hexagon_V6_vaddcarrysat,                   // llvm.hexagon.V6.vaddcarrysat (IntrinsicsHexagonDep.td:6126)
    hexagon_V6_vaddcarrysat_128B,              // llvm.hexagon.V6.vaddcarrysat.128B (IntrinsicsHexagonDep.td:6129)
    hexagon_V6_vaddclbh,                       // llvm.hexagon.V6.vaddclbh (IntrinsicsHexagonDep.td:5642)
    hexagon_V6_vaddclbh_128B,                  // llvm.hexagon.V6.vaddclbh.128B (IntrinsicsHexagonDep.td:5645)
    hexagon_V6_vaddclbw,                       // llvm.hexagon.V6.vaddclbw (IntrinsicsHexagonDep.td:5648)
    hexagon_V6_vaddclbw_128B,                  // llvm.hexagon.V6.vaddclbw.128B (IntrinsicsHexagonDep.td:5651)
    hexagon_V6_vaddh,                          // llvm.hexagon.V6.vaddh (IntrinsicsHexagonDep.td:3942)
    hexagon_V6_vaddh_128B,                     // llvm.hexagon.V6.vaddh.128B (IntrinsicsHexagonDep.td:3945)
    hexagon_V6_vaddh_dv,                       // llvm.hexagon.V6.vaddh.dv (IntrinsicsHexagonDep.td:3948)
    hexagon_V6_vaddh_dv_128B,                  // llvm.hexagon.V6.vaddh.dv.128B (IntrinsicsHexagonDep.td:3951)
    hexagon_V6_vaddhnq,                        // llvm.hexagon.V6.vaddhnq (IntrinsicsHexagonDep.td:3954)
    hexagon_V6_vaddhnq_128B,                   // llvm.hexagon.V6.vaddhnq.128B (IntrinsicsHexagonDep.td:3957)
    hexagon_V6_vaddhq,                         // llvm.hexagon.V6.vaddhq (IntrinsicsHexagonDep.td:3960)
    hexagon_V6_vaddhq_128B,                    // llvm.hexagon.V6.vaddhq.128B (IntrinsicsHexagonDep.td:3963)
    hexagon_V6_vaddhsat,                       // llvm.hexagon.V6.vaddhsat (IntrinsicsHexagonDep.td:3966)
    hexagon_V6_vaddhsat_128B,                  // llvm.hexagon.V6.vaddhsat.128B (IntrinsicsHexagonDep.td:3969)
    hexagon_V6_vaddhsat_dv,                    // llvm.hexagon.V6.vaddhsat.dv (IntrinsicsHexagonDep.td:3972)
    hexagon_V6_vaddhsat_dv_128B,               // llvm.hexagon.V6.vaddhsat.dv.128B (IntrinsicsHexagonDep.td:3975)
    hexagon_V6_vaddhw,                         // llvm.hexagon.V6.vaddhw (IntrinsicsHexagonDep.td:3978)
    hexagon_V6_vaddhw_128B,                    // llvm.hexagon.V6.vaddhw.128B (IntrinsicsHexagonDep.td:3981)
    hexagon_V6_vaddhw_acc,                     // llvm.hexagon.V6.vaddhw.acc (IntrinsicsHexagonDep.td:5654)
    hexagon_V6_vaddhw_acc_128B,                // llvm.hexagon.V6.vaddhw.acc.128B (IntrinsicsHexagonDep.td:5657)
    hexagon_V6_vaddubh,                        // llvm.hexagon.V6.vaddubh (IntrinsicsHexagonDep.td:3984)
    hexagon_V6_vaddubh_128B,                   // llvm.hexagon.V6.vaddubh.128B (IntrinsicsHexagonDep.td:3987)
    hexagon_V6_vaddubh_acc,                    // llvm.hexagon.V6.vaddubh.acc (IntrinsicsHexagonDep.td:5660)
    hexagon_V6_vaddubh_acc_128B,               // llvm.hexagon.V6.vaddubh.acc.128B (IntrinsicsHexagonDep.td:5663)
    hexagon_V6_vaddubsat,                      // llvm.hexagon.V6.vaddubsat (IntrinsicsHexagonDep.td:3990)
    hexagon_V6_vaddubsat_128B,                 // llvm.hexagon.V6.vaddubsat.128B (IntrinsicsHexagonDep.td:3993)
    hexagon_V6_vaddubsat_dv,                   // llvm.hexagon.V6.vaddubsat.dv (IntrinsicsHexagonDep.td:3996)
    hexagon_V6_vaddubsat_dv_128B,              // llvm.hexagon.V6.vaddubsat.dv.128B (IntrinsicsHexagonDep.td:3999)
    hexagon_V6_vaddububb_sat,                  // llvm.hexagon.V6.vaddububb.sat (IntrinsicsHexagonDep.td:5666)
    hexagon_V6_vaddububb_sat_128B,             // llvm.hexagon.V6.vaddububb.sat.128B (IntrinsicsHexagonDep.td:5669)
    hexagon_V6_vadduhsat,                      // llvm.hexagon.V6.vadduhsat (IntrinsicsHexagonDep.td:4002)
    hexagon_V6_vadduhsat_128B,                 // llvm.hexagon.V6.vadduhsat.128B (IntrinsicsHexagonDep.td:4005)
    hexagon_V6_vadduhsat_dv,                   // llvm.hexagon.V6.vadduhsat.dv (IntrinsicsHexagonDep.td:4008)
    hexagon_V6_vadduhsat_dv_128B,              // llvm.hexagon.V6.vadduhsat.dv.128B (IntrinsicsHexagonDep.td:4011)
    hexagon_V6_vadduhw,                        // llvm.hexagon.V6.vadduhw (IntrinsicsHexagonDep.td:4014)
    hexagon_V6_vadduhw_128B,                   // llvm.hexagon.V6.vadduhw.128B (IntrinsicsHexagonDep.td:4017)
    hexagon_V6_vadduhw_acc,                    // llvm.hexagon.V6.vadduhw.acc (IntrinsicsHexagonDep.td:5672)
    hexagon_V6_vadduhw_acc_128B,               // llvm.hexagon.V6.vadduhw.acc.128B (IntrinsicsHexagonDep.td:5675)
    hexagon_V6_vadduwsat,                      // llvm.hexagon.V6.vadduwsat (IntrinsicsHexagonDep.td:5678)
    hexagon_V6_vadduwsat_128B,                 // llvm.hexagon.V6.vadduwsat.128B (IntrinsicsHexagonDep.td:5681)
    hexagon_V6_vadduwsat_dv,                   // llvm.hexagon.V6.vadduwsat.dv (IntrinsicsHexagonDep.td:5684)
    hexagon_V6_vadduwsat_dv_128B,              // llvm.hexagon.V6.vadduwsat.dv.128B (IntrinsicsHexagonDep.td:5687)
    hexagon_V6_vaddw,                          // llvm.hexagon.V6.vaddw (IntrinsicsHexagonDep.td:4020)
    hexagon_V6_vaddw_128B,                     // llvm.hexagon.V6.vaddw.128B (IntrinsicsHexagonDep.td:4023)
    hexagon_V6_vaddw_dv,                       // llvm.hexagon.V6.vaddw.dv (IntrinsicsHexagonDep.td:4026)
    hexagon_V6_vaddw_dv_128B,                  // llvm.hexagon.V6.vaddw.dv.128B (IntrinsicsHexagonDep.td:4029)
    hexagon_V6_vaddwnq,                        // llvm.hexagon.V6.vaddwnq (IntrinsicsHexagonDep.td:4032)
    hexagon_V6_vaddwnq_128B,                   // llvm.hexagon.V6.vaddwnq.128B (IntrinsicsHexagonDep.td:4035)
    hexagon_V6_vaddwq,                         // llvm.hexagon.V6.vaddwq (IntrinsicsHexagonDep.td:4038)
    hexagon_V6_vaddwq_128B,                    // llvm.hexagon.V6.vaddwq.128B (IntrinsicsHexagonDep.td:4041)
    hexagon_V6_vaddwsat,                       // llvm.hexagon.V6.vaddwsat (IntrinsicsHexagonDep.td:4044)
    hexagon_V6_vaddwsat_128B,                  // llvm.hexagon.V6.vaddwsat.128B (IntrinsicsHexagonDep.td:4047)
    hexagon_V6_vaddwsat_dv,                    // llvm.hexagon.V6.vaddwsat.dv (IntrinsicsHexagonDep.td:4050)
    hexagon_V6_vaddwsat_dv_128B,               // llvm.hexagon.V6.vaddwsat.dv.128B (IntrinsicsHexagonDep.td:4053)
    hexagon_V6_valign4,                        // llvm.hexagon.V6.valign4 (IntrinsicsHexagonDep.td:6862)
    hexagon_V6_valign4_128B,                   // llvm.hexagon.V6.valign4.128B (IntrinsicsHexagonDep.td:6865)
    hexagon_V6_valignb,                        // llvm.hexagon.V6.valignb (IntrinsicsHexagonDep.td:4056)
    hexagon_V6_valignb_128B,                   // llvm.hexagon.V6.valignb.128B (IntrinsicsHexagonDep.td:4059)
    hexagon_V6_valignbi,                       // llvm.hexagon.V6.valignbi (IntrinsicsHexagonDep.td:4062)
    hexagon_V6_valignbi_128B,                  // llvm.hexagon.V6.valignbi.128B (IntrinsicsHexagonDep.td:4065)
    hexagon_V6_vand,                           // llvm.hexagon.V6.vand (IntrinsicsHexagonDep.td:4068)
    hexagon_V6_vand_128B,                      // llvm.hexagon.V6.vand.128B (IntrinsicsHexagonDep.td:4071)
    hexagon_V6_vandnqrt,                       // llvm.hexagon.V6.vandnqrt (IntrinsicsHexagonDep.td:5690)
    hexagon_V6_vandnqrt_128B,                  // llvm.hexagon.V6.vandnqrt.128B (IntrinsicsHexagonDep.td:5693)
    hexagon_V6_vandnqrt_acc,                   // llvm.hexagon.V6.vandnqrt.acc (IntrinsicsHexagonDep.td:5696)
    hexagon_V6_vandnqrt_acc_128B,              // llvm.hexagon.V6.vandnqrt.acc.128B (IntrinsicsHexagonDep.td:5699)
    hexagon_V6_vandqrt,                        // llvm.hexagon.V6.vandqrt (IntrinsicsHexagonDep.td:4074)
    hexagon_V6_vandqrt_128B,                   // llvm.hexagon.V6.vandqrt.128B (IntrinsicsHexagonDep.td:4077)
    hexagon_V6_vandqrt_acc,                    // llvm.hexagon.V6.vandqrt.acc (IntrinsicsHexagonDep.td:4080)
    hexagon_V6_vandqrt_acc_128B,               // llvm.hexagon.V6.vandqrt.acc.128B (IntrinsicsHexagonDep.td:4083)
    hexagon_V6_vandvnqv,                       // llvm.hexagon.V6.vandvnqv (IntrinsicsHexagonDep.td:5702)
    hexagon_V6_vandvnqv_128B,                  // llvm.hexagon.V6.vandvnqv.128B (IntrinsicsHexagonDep.td:5705)
    hexagon_V6_vandvqv,                        // llvm.hexagon.V6.vandvqv (IntrinsicsHexagonDep.td:5708)
    hexagon_V6_vandvqv_128B,                   // llvm.hexagon.V6.vandvqv.128B (IntrinsicsHexagonDep.td:5711)
    hexagon_V6_vandvrt,                        // llvm.hexagon.V6.vandvrt (IntrinsicsHexagonDep.td:4086)
    hexagon_V6_vandvrt_128B,                   // llvm.hexagon.V6.vandvrt.128B (IntrinsicsHexagonDep.td:4089)
    hexagon_V6_vandvrt_acc,                    // llvm.hexagon.V6.vandvrt.acc (IntrinsicsHexagonDep.td:4092)
    hexagon_V6_vandvrt_acc_128B,               // llvm.hexagon.V6.vandvrt.acc.128B (IntrinsicsHexagonDep.td:4095)
    hexagon_V6_vaslh,                          // llvm.hexagon.V6.vaslh (IntrinsicsHexagonDep.td:4098)
    hexagon_V6_vaslh_128B,                     // llvm.hexagon.V6.vaslh.128B (IntrinsicsHexagonDep.td:4101)
    hexagon_V6_vaslh_acc,                      // llvm.hexagon.V6.vaslh.acc (IntrinsicsHexagonDep.td:5890)
    hexagon_V6_vaslh_acc_128B,                 // llvm.hexagon.V6.vaslh.acc.128B (IntrinsicsHexagonDep.td:5893)
    hexagon_V6_vaslhv,                         // llvm.hexagon.V6.vaslhv (IntrinsicsHexagonDep.td:4104)
    hexagon_V6_vaslhv_128B,                    // llvm.hexagon.V6.vaslhv.128B (IntrinsicsHexagonDep.td:4107)
    hexagon_V6_vaslw,                          // llvm.hexagon.V6.vaslw (IntrinsicsHexagonDep.td:4110)
    hexagon_V6_vaslw_128B,                     // llvm.hexagon.V6.vaslw.128B (IntrinsicsHexagonDep.td:4113)
    hexagon_V6_vaslw_acc,                      // llvm.hexagon.V6.vaslw.acc (IntrinsicsHexagonDep.td:4116)
    hexagon_V6_vaslw_acc_128B,                 // llvm.hexagon.V6.vaslw.acc.128B (IntrinsicsHexagonDep.td:4119)
    hexagon_V6_vaslwv,                         // llvm.hexagon.V6.vaslwv (IntrinsicsHexagonDep.td:4122)
    hexagon_V6_vaslwv_128B,                    // llvm.hexagon.V6.vaslwv.128B (IntrinsicsHexagonDep.td:4125)
    hexagon_V6_vasr_into,                      // llvm.hexagon.V6.vasr.into (IntrinsicsHexagonDep.td:6132)
    hexagon_V6_vasr_into_128B,                 // llvm.hexagon.V6.vasr.into.128B (IntrinsicsHexagonDep.td:6135)
    hexagon_V6_vasrh,                          // llvm.hexagon.V6.vasrh (IntrinsicsHexagonDep.td:4128)
    hexagon_V6_vasrh_128B,                     // llvm.hexagon.V6.vasrh.128B (IntrinsicsHexagonDep.td:4131)
    hexagon_V6_vasrh_acc,                      // llvm.hexagon.V6.vasrh.acc (IntrinsicsHexagonDep.td:5896)
    hexagon_V6_vasrh_acc_128B,                 // llvm.hexagon.V6.vasrh.acc.128B (IntrinsicsHexagonDep.td:5899)
    hexagon_V6_vasrhbrndsat,                   // llvm.hexagon.V6.vasrhbrndsat (IntrinsicsHexagonDep.td:4134)
    hexagon_V6_vasrhbrndsat_128B,              // llvm.hexagon.V6.vasrhbrndsat.128B (IntrinsicsHexagonDep.td:4137)
    hexagon_V6_vasrhbsat,                      // llvm.hexagon.V6.vasrhbsat (IntrinsicsHexagonDep.td:5714)
    hexagon_V6_vasrhbsat_128B,                 // llvm.hexagon.V6.vasrhbsat.128B (IntrinsicsHexagonDep.td:5717)
    hexagon_V6_vasrhubrndsat,                  // llvm.hexagon.V6.vasrhubrndsat (IntrinsicsHexagonDep.td:4140)
    hexagon_V6_vasrhubrndsat_128B,             // llvm.hexagon.V6.vasrhubrndsat.128B (IntrinsicsHexagonDep.td:4143)
    hexagon_V6_vasrhubsat,                     // llvm.hexagon.V6.vasrhubsat (IntrinsicsHexagonDep.td:4146)
    hexagon_V6_vasrhubsat_128B,                // llvm.hexagon.V6.vasrhubsat.128B (IntrinsicsHexagonDep.td:4149)
    hexagon_V6_vasrhv,                         // llvm.hexagon.V6.vasrhv (IntrinsicsHexagonDep.td:4152)
    hexagon_V6_vasrhv_128B,                    // llvm.hexagon.V6.vasrhv.128B (IntrinsicsHexagonDep.td:4155)
    hexagon_V6_vasruhubrndsat,                 // llvm.hexagon.V6.vasruhubrndsat (IntrinsicsHexagonDep.td:5902)
    hexagon_V6_vasruhubrndsat_128B,            // llvm.hexagon.V6.vasruhubrndsat.128B (IntrinsicsHexagonDep.td:5905)
    hexagon_V6_vasruhubsat,                    // llvm.hexagon.V6.vasruhubsat (IntrinsicsHexagonDep.td:5908)
    hexagon_V6_vasruhubsat_128B,               // llvm.hexagon.V6.vasruhubsat.128B (IntrinsicsHexagonDep.td:5911)
    hexagon_V6_vasruwuhrndsat,                 // llvm.hexagon.V6.vasruwuhrndsat (IntrinsicsHexagonDep.td:5720)
    hexagon_V6_vasruwuhrndsat_128B,            // llvm.hexagon.V6.vasruwuhrndsat.128B (IntrinsicsHexagonDep.td:5723)
    hexagon_V6_vasruwuhsat,                    // llvm.hexagon.V6.vasruwuhsat (IntrinsicsHexagonDep.td:5914)
    hexagon_V6_vasruwuhsat_128B,               // llvm.hexagon.V6.vasruwuhsat.128B (IntrinsicsHexagonDep.td:5917)
    hexagon_V6_vasrvuhubrndsat,                // llvm.hexagon.V6.vasrvuhubrndsat (IntrinsicsHexagonDep.td:6586)
    hexagon_V6_vasrvuhubrndsat_128B,           // llvm.hexagon.V6.vasrvuhubrndsat.128B (IntrinsicsHexagonDep.td:6589)
    hexagon_V6_vasrvuhubsat,                   // llvm.hexagon.V6.vasrvuhubsat (IntrinsicsHexagonDep.td:6592)
    hexagon_V6_vasrvuhubsat_128B,              // llvm.hexagon.V6.vasrvuhubsat.128B (IntrinsicsHexagonDep.td:6595)
    hexagon_V6_vasrvwuhrndsat,                 // llvm.hexagon.V6.vasrvwuhrndsat (IntrinsicsHexagonDep.td:6598)
    hexagon_V6_vasrvwuhrndsat_128B,            // llvm.hexagon.V6.vasrvwuhrndsat.128B (IntrinsicsHexagonDep.td:6601)
    hexagon_V6_vasrvwuhsat,                    // llvm.hexagon.V6.vasrvwuhsat (IntrinsicsHexagonDep.td:6604)
    hexagon_V6_vasrvwuhsat_128B,               // llvm.hexagon.V6.vasrvwuhsat.128B (IntrinsicsHexagonDep.td:6607)
    hexagon_V6_vasrw,                          // llvm.hexagon.V6.vasrw (IntrinsicsHexagonDep.td:4158)
    hexagon_V6_vasrw_128B,                     // llvm.hexagon.V6.vasrw.128B (IntrinsicsHexagonDep.td:4161)
    hexagon_V6_vasrw_acc,                      // llvm.hexagon.V6.vasrw.acc (IntrinsicsHexagonDep.td:4164)
    hexagon_V6_vasrw_acc_128B,                 // llvm.hexagon.V6.vasrw.acc.128B (IntrinsicsHexagonDep.td:4167)
    hexagon_V6_vasrwh,                         // llvm.hexagon.V6.vasrwh (IntrinsicsHexagonDep.td:4170)
    hexagon_V6_vasrwh_128B,                    // llvm.hexagon.V6.vasrwh.128B (IntrinsicsHexagonDep.td:4173)
    hexagon_V6_vasrwhrndsat,                   // llvm.hexagon.V6.vasrwhrndsat (IntrinsicsHexagonDep.td:4176)
    hexagon_V6_vasrwhrndsat_128B,              // llvm.hexagon.V6.vasrwhrndsat.128B (IntrinsicsHexagonDep.td:4179)
    hexagon_V6_vasrwhsat,                      // llvm.hexagon.V6.vasrwhsat (IntrinsicsHexagonDep.td:4182)
    hexagon_V6_vasrwhsat_128B,                 // llvm.hexagon.V6.vasrwhsat.128B (IntrinsicsHexagonDep.td:4185)
    hexagon_V6_vasrwuhrndsat,                  // llvm.hexagon.V6.vasrwuhrndsat (IntrinsicsHexagonDep.td:5726)
    hexagon_V6_vasrwuhrndsat_128B,             // llvm.hexagon.V6.vasrwuhrndsat.128B (IntrinsicsHexagonDep.td:5729)
    hexagon_V6_vasrwuhsat,                     // llvm.hexagon.V6.vasrwuhsat (IntrinsicsHexagonDep.td:4188)
    hexagon_V6_vasrwuhsat_128B,                // llvm.hexagon.V6.vasrwuhsat.128B (IntrinsicsHexagonDep.td:4191)
    hexagon_V6_vasrwv,                         // llvm.hexagon.V6.vasrwv (IntrinsicsHexagonDep.td:4194)
    hexagon_V6_vasrwv_128B,                    // llvm.hexagon.V6.vasrwv.128B (IntrinsicsHexagonDep.td:4197)
    hexagon_V6_vassign,                        // llvm.hexagon.V6.vassign (IntrinsicsHexagonDep.td:4200)
    hexagon_V6_vassign_128B,                   // llvm.hexagon.V6.vassign.128B (IntrinsicsHexagonDep.td:4203)
    hexagon_V6_vassign_fp,                     // llvm.hexagon.V6.vassign.fp (IntrinsicsHexagonDep.td:6248)
    hexagon_V6_vassign_fp_128B,                // llvm.hexagon.V6.vassign.fp.128B (IntrinsicsHexagonDep.td:6251)
    hexagon_V6_vassignp,                       // llvm.hexagon.V6.vassignp (IntrinsicsHexagonDep.td:4206)
    hexagon_V6_vassignp_128B,                  // llvm.hexagon.V6.vassignp.128B (IntrinsicsHexagonDep.td:4209)
    hexagon_V6_vavgb,                          // llvm.hexagon.V6.vavgb (IntrinsicsHexagonDep.td:5920)
    hexagon_V6_vavgb_128B,                     // llvm.hexagon.V6.vavgb.128B (IntrinsicsHexagonDep.td:5923)
    hexagon_V6_vavgbrnd,                       // llvm.hexagon.V6.vavgbrnd (IntrinsicsHexagonDep.td:5926)
    hexagon_V6_vavgbrnd_128B,                  // llvm.hexagon.V6.vavgbrnd.128B (IntrinsicsHexagonDep.td:5929)
    hexagon_V6_vavgh,                          // llvm.hexagon.V6.vavgh (IntrinsicsHexagonDep.td:4212)
    hexagon_V6_vavgh_128B,                     // llvm.hexagon.V6.vavgh.128B (IntrinsicsHexagonDep.td:4215)
    hexagon_V6_vavghrnd,                       // llvm.hexagon.V6.vavghrnd (IntrinsicsHexagonDep.td:4218)
    hexagon_V6_vavghrnd_128B,                  // llvm.hexagon.V6.vavghrnd.128B (IntrinsicsHexagonDep.td:4221)
    hexagon_V6_vavgub,                         // llvm.hexagon.V6.vavgub (IntrinsicsHexagonDep.td:4224)
    hexagon_V6_vavgub_128B,                    // llvm.hexagon.V6.vavgub.128B (IntrinsicsHexagonDep.td:4227)
    hexagon_V6_vavgubrnd,                      // llvm.hexagon.V6.vavgubrnd (IntrinsicsHexagonDep.td:4230)
    hexagon_V6_vavgubrnd_128B,                 // llvm.hexagon.V6.vavgubrnd.128B (IntrinsicsHexagonDep.td:4233)
    hexagon_V6_vavguh,                         // llvm.hexagon.V6.vavguh (IntrinsicsHexagonDep.td:4236)
    hexagon_V6_vavguh_128B,                    // llvm.hexagon.V6.vavguh.128B (IntrinsicsHexagonDep.td:4239)
    hexagon_V6_vavguhrnd,                      // llvm.hexagon.V6.vavguhrnd (IntrinsicsHexagonDep.td:4242)
    hexagon_V6_vavguhrnd_128B,                 // llvm.hexagon.V6.vavguhrnd.128B (IntrinsicsHexagonDep.td:4245)
    hexagon_V6_vavguw,                         // llvm.hexagon.V6.vavguw (IntrinsicsHexagonDep.td:5932)
    hexagon_V6_vavguw_128B,                    // llvm.hexagon.V6.vavguw.128B (IntrinsicsHexagonDep.td:5935)
    hexagon_V6_vavguwrnd,                      // llvm.hexagon.V6.vavguwrnd (IntrinsicsHexagonDep.td:5938)
    hexagon_V6_vavguwrnd_128B,                 // llvm.hexagon.V6.vavguwrnd.128B (IntrinsicsHexagonDep.td:5941)
    hexagon_V6_vavgw,                          // llvm.hexagon.V6.vavgw (IntrinsicsHexagonDep.td:4248)
    hexagon_V6_vavgw_128B,                     // llvm.hexagon.V6.vavgw.128B (IntrinsicsHexagonDep.td:4251)
    hexagon_V6_vavgwrnd,                       // llvm.hexagon.V6.vavgwrnd (IntrinsicsHexagonDep.td:4254)
    hexagon_V6_vavgwrnd_128B,                  // llvm.hexagon.V6.vavgwrnd.128B (IntrinsicsHexagonDep.td:4257)
    hexagon_V6_vcl0h,                          // llvm.hexagon.V6.vcl0h (IntrinsicsHexagonDep.td:4260)
    hexagon_V6_vcl0h_128B,                     // llvm.hexagon.V6.vcl0h.128B (IntrinsicsHexagonDep.td:4263)
    hexagon_V6_vcl0w,                          // llvm.hexagon.V6.vcl0w (IntrinsicsHexagonDep.td:4266)
    hexagon_V6_vcl0w_128B,                     // llvm.hexagon.V6.vcl0w.128B (IntrinsicsHexagonDep.td:4269)
    hexagon_V6_vcombine,                       // llvm.hexagon.V6.vcombine (IntrinsicsHexagonDep.td:4272)
    hexagon_V6_vcombine_128B,                  // llvm.hexagon.V6.vcombine.128B (IntrinsicsHexagonDep.td:4275)
    hexagon_V6_vconv_bf_qf32,                  // llvm.hexagon.V6.vconv.bf.qf32 (IntrinsicsHexagonDep.td:6868)
    hexagon_V6_vconv_bf_qf32_128B,             // llvm.hexagon.V6.vconv.bf.qf32.128B (IntrinsicsHexagonDep.td:6871)
    hexagon_V6_vconv_f8_qf16,                  // llvm.hexagon.V6.vconv.f8.qf16 (IntrinsicsHexagonDep.td:6874)
    hexagon_V6_vconv_f8_qf16_128B,             // llvm.hexagon.V6.vconv.f8.qf16.128B (IntrinsicsHexagonDep.td:6877)
    hexagon_V6_vconv_h_hf,                     // llvm.hexagon.V6.vconv.h.hf (IntrinsicsHexagonDep.td:6624)
    hexagon_V6_vconv_h_hf_128B,                // llvm.hexagon.V6.vconv.h.hf.128B (IntrinsicsHexagonDep.td:6627)
    hexagon_V6_vconv_h_hf_rnd,                 // llvm.hexagon.V6.vconv.h.hf.rnd (IntrinsicsHexagonDep.td:6880)
    hexagon_V6_vconv_h_hf_rnd_128B,            // llvm.hexagon.V6.vconv.h.hf.rnd.128B (IntrinsicsHexagonDep.td:6883)
    hexagon_V6_vconv_hf_h,                     // llvm.hexagon.V6.vconv.hf.h (IntrinsicsHexagonDep.td:6630)
    hexagon_V6_vconv_hf_h_128B,                // llvm.hexagon.V6.vconv.hf.h.128B (IntrinsicsHexagonDep.td:6633)
    hexagon_V6_vconv_hf_qf16,                  // llvm.hexagon.V6.vconv.hf.qf16 (IntrinsicsHexagonDep.td:6254)
    hexagon_V6_vconv_hf_qf16_128B,             // llvm.hexagon.V6.vconv.hf.qf16.128B (IntrinsicsHexagonDep.td:6257)
    hexagon_V6_vconv_hf_qf32,                  // llvm.hexagon.V6.vconv.hf.qf32 (IntrinsicsHexagonDep.td:6260)
    hexagon_V6_vconv_hf_qf32_128B,             // llvm.hexagon.V6.vconv.hf.qf32.128B (IntrinsicsHexagonDep.td:6263)
    hexagon_V6_vconv_qf16_f8,                  // llvm.hexagon.V6.vconv.qf16.f8 (IntrinsicsHexagonDep.td:6886)
    hexagon_V6_vconv_qf16_f8_128B,             // llvm.hexagon.V6.vconv.qf16.f8.128B (IntrinsicsHexagonDep.td:6889)
    hexagon_V6_vconv_qf16_hf,                  // llvm.hexagon.V6.vconv.qf16.hf (IntrinsicsHexagonDep.td:6892)
    hexagon_V6_vconv_qf16_hf_128B,             // llvm.hexagon.V6.vconv.qf16.hf.128B (IntrinsicsHexagonDep.td:6895)
    hexagon_V6_vconv_qf16_qf16,                // llvm.hexagon.V6.vconv.qf16.qf16 (IntrinsicsHexagonDep.td:6898)
    hexagon_V6_vconv_qf16_qf16_128B,           // llvm.hexagon.V6.vconv.qf16.qf16.128B (IntrinsicsHexagonDep.td:6901)
    hexagon_V6_vconv_qf32_qf32,                // llvm.hexagon.V6.vconv.qf32.qf32 (IntrinsicsHexagonDep.td:6904)
    hexagon_V6_vconv_qf32_qf32_128B,           // llvm.hexagon.V6.vconv.qf32.qf32.128B (IntrinsicsHexagonDep.td:6907)
    hexagon_V6_vconv_qf32_sf,                  // llvm.hexagon.V6.vconv.qf32.sf (IntrinsicsHexagonDep.td:6910)
    hexagon_V6_vconv_qf32_sf_128B,             // llvm.hexagon.V6.vconv.qf32.sf.128B (IntrinsicsHexagonDep.td:6913)
    hexagon_V6_vconv_sf_qf32,                  // llvm.hexagon.V6.vconv.sf.qf32 (IntrinsicsHexagonDep.td:6266)
    hexagon_V6_vconv_sf_qf32_128B,             // llvm.hexagon.V6.vconv.sf.qf32.128B (IntrinsicsHexagonDep.td:6269)
    hexagon_V6_vconv_sf_w,                     // llvm.hexagon.V6.vconv.sf.w (IntrinsicsHexagonDep.td:6636)
    hexagon_V6_vconv_sf_w_128B,                // llvm.hexagon.V6.vconv.sf.w.128B (IntrinsicsHexagonDep.td:6639)
    hexagon_V6_vconv_w_sf,                     // llvm.hexagon.V6.vconv.w.sf (IntrinsicsHexagonDep.td:6642)
    hexagon_V6_vconv_w_sf_128B,                // llvm.hexagon.V6.vconv.w.sf.128B (IntrinsicsHexagonDep.td:6645)
    hexagon_V6_vcvt_b_hf,                      // llvm.hexagon.V6.vcvt.b.hf (IntrinsicsHexagonDep.td:6272)
    hexagon_V6_vcvt_b_hf_128B,                 // llvm.hexagon.V6.vcvt.b.hf.128B (IntrinsicsHexagonDep.td:6275)
    hexagon_V6_vcvt_bf_sf,                     // llvm.hexagon.V6.vcvt.bf.sf (IntrinsicsHexagonDep.td:6648)
    hexagon_V6_vcvt_bf_sf_128B,                // llvm.hexagon.V6.vcvt.bf.sf.128B (IntrinsicsHexagonDep.td:6651)
    hexagon_V6_vcvt_f8_hf,                     // llvm.hexagon.V6.vcvt.f8.hf (IntrinsicsHexagonDep.td:6764)
    hexagon_V6_vcvt_f8_hf_128B,                // llvm.hexagon.V6.vcvt.f8.hf.128B (IntrinsicsHexagonDep.td:6767)
    hexagon_V6_vcvt_h_hf,                      // llvm.hexagon.V6.vcvt.h.hf (IntrinsicsHexagonDep.td:6278)
    hexagon_V6_vcvt_h_hf_128B,                 // llvm.hexagon.V6.vcvt.h.hf.128B (IntrinsicsHexagonDep.td:6281)
    hexagon_V6_vcvt_hf_b,                      // llvm.hexagon.V6.vcvt.hf.b (IntrinsicsHexagonDep.td:6284)
    hexagon_V6_vcvt_hf_b_128B,                 // llvm.hexagon.V6.vcvt.hf.b.128B (IntrinsicsHexagonDep.td:6287)
    hexagon_V6_vcvt_hf_f8,                     // llvm.hexagon.V6.vcvt.hf.f8 (IntrinsicsHexagonDep.td:6770)
    hexagon_V6_vcvt_hf_f8_128B,                // llvm.hexagon.V6.vcvt.hf.f8.128B (IntrinsicsHexagonDep.td:6773)
    hexagon_V6_vcvt_hf_h,                      // llvm.hexagon.V6.vcvt.hf.h (IntrinsicsHexagonDep.td:6290)
    hexagon_V6_vcvt_hf_h_128B,                 // llvm.hexagon.V6.vcvt.hf.h.128B (IntrinsicsHexagonDep.td:6293)
    hexagon_V6_vcvt_hf_sf,                     // llvm.hexagon.V6.vcvt.hf.sf (IntrinsicsHexagonDep.td:6296)
    hexagon_V6_vcvt_hf_sf_128B,                // llvm.hexagon.V6.vcvt.hf.sf.128B (IntrinsicsHexagonDep.td:6299)
    hexagon_V6_vcvt_hf_ub,                     // llvm.hexagon.V6.vcvt.hf.ub (IntrinsicsHexagonDep.td:6302)
    hexagon_V6_vcvt_hf_ub_128B,                // llvm.hexagon.V6.vcvt.hf.ub.128B (IntrinsicsHexagonDep.td:6305)
    hexagon_V6_vcvt_hf_uh,                     // llvm.hexagon.V6.vcvt.hf.uh (IntrinsicsHexagonDep.td:6308)
    hexagon_V6_vcvt_hf_uh_128B,                // llvm.hexagon.V6.vcvt.hf.uh.128B (IntrinsicsHexagonDep.td:6311)
    hexagon_V6_vcvt_sf_hf,                     // llvm.hexagon.V6.vcvt.sf.hf (IntrinsicsHexagonDep.td:6314)
    hexagon_V6_vcvt_sf_hf_128B,                // llvm.hexagon.V6.vcvt.sf.hf.128B (IntrinsicsHexagonDep.td:6317)
    hexagon_V6_vcvt_ub_hf,                     // llvm.hexagon.V6.vcvt.ub.hf (IntrinsicsHexagonDep.td:6320)
    hexagon_V6_vcvt_ub_hf_128B,                // llvm.hexagon.V6.vcvt.ub.hf.128B (IntrinsicsHexagonDep.td:6323)
    hexagon_V6_vcvt_uh_hf,                     // llvm.hexagon.V6.vcvt.uh.hf (IntrinsicsHexagonDep.td:6326)
    hexagon_V6_vcvt_uh_hf_128B,                // llvm.hexagon.V6.vcvt.uh.hf.128B (IntrinsicsHexagonDep.td:6329)
    hexagon_V6_vcvt2_b_hf,                     // llvm.hexagon.V6.vcvt2.b.hf (IntrinsicsHexagonDep.td:6740)
    hexagon_V6_vcvt2_b_hf_128B,                // llvm.hexagon.V6.vcvt2.b.hf.128B (IntrinsicsHexagonDep.td:6743)
    hexagon_V6_vcvt2_hf_b,                     // llvm.hexagon.V6.vcvt2.hf.b (IntrinsicsHexagonDep.td:6746)
    hexagon_V6_vcvt2_hf_b_128B,                // llvm.hexagon.V6.vcvt2.hf.b.128B (IntrinsicsHexagonDep.td:6749)
    hexagon_V6_vcvt2_hf_ub,                    // llvm.hexagon.V6.vcvt2.hf.ub (IntrinsicsHexagonDep.td:6752)
    hexagon_V6_vcvt2_hf_ub_128B,               // llvm.hexagon.V6.vcvt2.hf.ub.128B (IntrinsicsHexagonDep.td:6755)
    hexagon_V6_vcvt2_ub_hf,                    // llvm.hexagon.V6.vcvt2.ub.hf (IntrinsicsHexagonDep.td:6758)
    hexagon_V6_vcvt2_ub_hf_128B,               // llvm.hexagon.V6.vcvt2.ub.hf.128B (IntrinsicsHexagonDep.td:6761)
    hexagon_V6_vd0,                            // llvm.hexagon.V6.vd0 (IntrinsicsHexagonDep.td:4278)
    hexagon_V6_vd0_128B,                       // llvm.hexagon.V6.vd0.128B (IntrinsicsHexagonDep.td:4281)
    hexagon_V6_vdd0,                           // llvm.hexagon.V6.vdd0 (IntrinsicsHexagonDep.td:5944)
    hexagon_V6_vdd0_128B,                      // llvm.hexagon.V6.vdd0.128B (IntrinsicsHexagonDep.td:5947)
    hexagon_V6_vdealb,                         // llvm.hexagon.V6.vdealb (IntrinsicsHexagonDep.td:4284)
    hexagon_V6_vdealb_128B,                    // llvm.hexagon.V6.vdealb.128B (IntrinsicsHexagonDep.td:4287)
    hexagon_V6_vdealb4w,                       // llvm.hexagon.V6.vdealb4w (IntrinsicsHexagonDep.td:4290)
    hexagon_V6_vdealb4w_128B,                  // llvm.hexagon.V6.vdealb4w.128B (IntrinsicsHexagonDep.td:4293)
    hexagon_V6_vdealh,                         // llvm.hexagon.V6.vdealh (IntrinsicsHexagonDep.td:4296)
    hexagon_V6_vdealh_128B,                    // llvm.hexagon.V6.vdealh.128B (IntrinsicsHexagonDep.td:4299)
    hexagon_V6_vdealvdd,                       // llvm.hexagon.V6.vdealvdd (IntrinsicsHexagonDep.td:4302)
    hexagon_V6_vdealvdd_128B,                  // llvm.hexagon.V6.vdealvdd.128B (IntrinsicsHexagonDep.td:4305)
    hexagon_V6_vdelta,                         // llvm.hexagon.V6.vdelta (IntrinsicsHexagonDep.td:4308)
    hexagon_V6_vdelta_128B,                    // llvm.hexagon.V6.vdelta.128B (IntrinsicsHexagonDep.td:4311)
    hexagon_V6_vdmpy_sf_hf,                    // llvm.hexagon.V6.vdmpy.sf.hf (IntrinsicsHexagonDep.td:6332)
    hexagon_V6_vdmpy_sf_hf_128B,               // llvm.hexagon.V6.vdmpy.sf.hf.128B (IntrinsicsHexagonDep.td:6335)
    hexagon_V6_vdmpy_sf_hf_acc,                // llvm.hexagon.V6.vdmpy.sf.hf.acc (IntrinsicsHexagonDep.td:6338)
    hexagon_V6_vdmpy_sf_hf_acc_128B,           // llvm.hexagon.V6.vdmpy.sf.hf.acc.128B (IntrinsicsHexagonDep.td:6341)
    hexagon_V6_vdmpybus,                       // llvm.hexagon.V6.vdmpybus (IntrinsicsHexagonDep.td:4314)
    hexagon_V6_vdmpybus_128B,                  // llvm.hexagon.V6.vdmpybus.128B (IntrinsicsHexagonDep.td:4317)
    hexagon_V6_vdmpybus_acc,                   // llvm.hexagon.V6.vdmpybus.acc (IntrinsicsHexagonDep.td:4320)
    hexagon_V6_vdmpybus_acc_128B,              // llvm.hexagon.V6.vdmpybus.acc.128B (IntrinsicsHexagonDep.td:4323)
    hexagon_V6_vdmpybus_dv,                    // llvm.hexagon.V6.vdmpybus.dv (IntrinsicsHexagonDep.td:4326)
    hexagon_V6_vdmpybus_dv_128B,               // llvm.hexagon.V6.vdmpybus.dv.128B (IntrinsicsHexagonDep.td:4329)
    hexagon_V6_vdmpybus_dv_acc,                // llvm.hexagon.V6.vdmpybus.dv.acc (IntrinsicsHexagonDep.td:4332)
    hexagon_V6_vdmpybus_dv_acc_128B,           // llvm.hexagon.V6.vdmpybus.dv.acc.128B (IntrinsicsHexagonDep.td:4335)
    hexagon_V6_vdmpyhb,                        // llvm.hexagon.V6.vdmpyhb (IntrinsicsHexagonDep.td:4338)
    hexagon_V6_vdmpyhb_128B,                   // llvm.hexagon.V6.vdmpyhb.128B (IntrinsicsHexagonDep.td:4341)
    hexagon_V6_vdmpyhb_acc,                    // llvm.hexagon.V6.vdmpyhb.acc (IntrinsicsHexagonDep.td:4344)
    hexagon_V6_vdmpyhb_acc_128B,               // llvm.hexagon.V6.vdmpyhb.acc.128B (IntrinsicsHexagonDep.td:4347)
    hexagon_V6_vdmpyhb_dv,                     // llvm.hexagon.V6.vdmpyhb.dv (IntrinsicsHexagonDep.td:4350)
    hexagon_V6_vdmpyhb_dv_128B,                // llvm.hexagon.V6.vdmpyhb.dv.128B (IntrinsicsHexagonDep.td:4353)
    hexagon_V6_vdmpyhb_dv_acc,                 // llvm.hexagon.V6.vdmpyhb.dv.acc (IntrinsicsHexagonDep.td:4356)
    hexagon_V6_vdmpyhb_dv_acc_128B,            // llvm.hexagon.V6.vdmpyhb.dv.acc.128B (IntrinsicsHexagonDep.td:4359)
    hexagon_V6_vdmpyhisat,                     // llvm.hexagon.V6.vdmpyhisat (IntrinsicsHexagonDep.td:4362)
    hexagon_V6_vdmpyhisat_128B,                // llvm.hexagon.V6.vdmpyhisat.128B (IntrinsicsHexagonDep.td:4365)
    hexagon_V6_vdmpyhisat_acc,                 // llvm.hexagon.V6.vdmpyhisat.acc (IntrinsicsHexagonDep.td:4368)
    hexagon_V6_vdmpyhisat_acc_128B,            // llvm.hexagon.V6.vdmpyhisat.acc.128B (IntrinsicsHexagonDep.td:4371)
    hexagon_V6_vdmpyhsat,                      // llvm.hexagon.V6.vdmpyhsat (IntrinsicsHexagonDep.td:4374)
    hexagon_V6_vdmpyhsat_128B,                 // llvm.hexagon.V6.vdmpyhsat.128B (IntrinsicsHexagonDep.td:4377)
    hexagon_V6_vdmpyhsat_acc,                  // llvm.hexagon.V6.vdmpyhsat.acc (IntrinsicsHexagonDep.td:4380)
    hexagon_V6_vdmpyhsat_acc_128B,             // llvm.hexagon.V6.vdmpyhsat.acc.128B (IntrinsicsHexagonDep.td:4383)
    hexagon_V6_vdmpyhsuisat,                   // llvm.hexagon.V6.vdmpyhsuisat (IntrinsicsHexagonDep.td:4386)
    hexagon_V6_vdmpyhsuisat_128B,              // llvm.hexagon.V6.vdmpyhsuisat.128B (IntrinsicsHexagonDep.td:4389)
    hexagon_V6_vdmpyhsuisat_acc,               // llvm.hexagon.V6.vdmpyhsuisat.acc (IntrinsicsHexagonDep.td:4392)
    hexagon_V6_vdmpyhsuisat_acc_128B,          // llvm.hexagon.V6.vdmpyhsuisat.acc.128B (IntrinsicsHexagonDep.td:4395)
    hexagon_V6_vdmpyhsusat,                    // llvm.hexagon.V6.vdmpyhsusat (IntrinsicsHexagonDep.td:4398)
    hexagon_V6_vdmpyhsusat_128B,               // llvm.hexagon.V6.vdmpyhsusat.128B (IntrinsicsHexagonDep.td:4401)
    hexagon_V6_vdmpyhsusat_acc,                // llvm.hexagon.V6.vdmpyhsusat.acc (IntrinsicsHexagonDep.td:4404)
    hexagon_V6_vdmpyhsusat_acc_128B,           // llvm.hexagon.V6.vdmpyhsusat.acc.128B (IntrinsicsHexagonDep.td:4407)
    hexagon_V6_vdmpyhvsat,                     // llvm.hexagon.V6.vdmpyhvsat (IntrinsicsHexagonDep.td:4410)
    hexagon_V6_vdmpyhvsat_128B,                // llvm.hexagon.V6.vdmpyhvsat.128B (IntrinsicsHexagonDep.td:4413)
    hexagon_V6_vdmpyhvsat_acc,                 // llvm.hexagon.V6.vdmpyhvsat.acc (IntrinsicsHexagonDep.td:4416)
    hexagon_V6_vdmpyhvsat_acc_128B,            // llvm.hexagon.V6.vdmpyhvsat.acc.128B (IntrinsicsHexagonDep.td:4419)
    hexagon_V6_vdsaduh,                        // llvm.hexagon.V6.vdsaduh (IntrinsicsHexagonDep.td:4422)
    hexagon_V6_vdsaduh_128B,                   // llvm.hexagon.V6.vdsaduh.128B (IntrinsicsHexagonDep.td:4425)
    hexagon_V6_vdsaduh_acc,                    // llvm.hexagon.V6.vdsaduh.acc (IntrinsicsHexagonDep.td:4428)
    hexagon_V6_vdsaduh_acc_128B,               // llvm.hexagon.V6.vdsaduh.acc.128B (IntrinsicsHexagonDep.td:4431)
    hexagon_V6_veqb,                           // llvm.hexagon.V6.veqb (IntrinsicsHexagonDep.td:4434)
    hexagon_V6_veqb_128B,                      // llvm.hexagon.V6.veqb.128B (IntrinsicsHexagonDep.td:4437)
    hexagon_V6_veqb_and,                       // llvm.hexagon.V6.veqb.and (IntrinsicsHexagonDep.td:4440)
    hexagon_V6_veqb_and_128B,                  // llvm.hexagon.V6.veqb.and.128B (IntrinsicsHexagonDep.td:4443)
    hexagon_V6_veqb_or,                        // llvm.hexagon.V6.veqb.or (IntrinsicsHexagonDep.td:4446)
    hexagon_V6_veqb_or_128B,                   // llvm.hexagon.V6.veqb.or.128B (IntrinsicsHexagonDep.td:4449)
    hexagon_V6_veqb_xor,                       // llvm.hexagon.V6.veqb.xor (IntrinsicsHexagonDep.td:4452)
    hexagon_V6_veqb_xor_128B,                  // llvm.hexagon.V6.veqb.xor.128B (IntrinsicsHexagonDep.td:4455)
    hexagon_V6_veqh,                           // llvm.hexagon.V6.veqh (IntrinsicsHexagonDep.td:4458)
    hexagon_V6_veqh_128B,                      // llvm.hexagon.V6.veqh.128B (IntrinsicsHexagonDep.td:4461)
    hexagon_V6_veqh_and,                       // llvm.hexagon.V6.veqh.and (IntrinsicsHexagonDep.td:4464)
    hexagon_V6_veqh_and_128B,                  // llvm.hexagon.V6.veqh.and.128B (IntrinsicsHexagonDep.td:4467)
    hexagon_V6_veqh_or,                        // llvm.hexagon.V6.veqh.or (IntrinsicsHexagonDep.td:4470)
    hexagon_V6_veqh_or_128B,                   // llvm.hexagon.V6.veqh.or.128B (IntrinsicsHexagonDep.td:4473)
    hexagon_V6_veqh_xor,                       // llvm.hexagon.V6.veqh.xor (IntrinsicsHexagonDep.td:4476)
    hexagon_V6_veqh_xor_128B,                  // llvm.hexagon.V6.veqh.xor.128B (IntrinsicsHexagonDep.td:4479)
    hexagon_V6_veqhf,                          // llvm.hexagon.V6.veqhf (IntrinsicsHexagonDep.td:6916)
    hexagon_V6_veqhf_128B,                     // llvm.hexagon.V6.veqhf.128B (IntrinsicsHexagonDep.td:6919)
    hexagon_V6_veqhf_and,                      // llvm.hexagon.V6.veqhf.and (IntrinsicsHexagonDep.td:6922)
    hexagon_V6_veqhf_and_128B,                 // llvm.hexagon.V6.veqhf.and.128B (IntrinsicsHexagonDep.td:6925)
    hexagon_V6_veqhf_or,                       // llvm.hexagon.V6.veqhf.or (IntrinsicsHexagonDep.td:6928)
    hexagon_V6_veqhf_or_128B,                  // llvm.hexagon.V6.veqhf.or.128B (IntrinsicsHexagonDep.td:6931)
    hexagon_V6_veqhf_xor,                      // llvm.hexagon.V6.veqhf.xor (IntrinsicsHexagonDep.td:6934)
    hexagon_V6_veqhf_xor_128B,                 // llvm.hexagon.V6.veqhf.xor.128B (IntrinsicsHexagonDep.td:6937)
    hexagon_V6_veqsf,                          // llvm.hexagon.V6.veqsf (IntrinsicsHexagonDep.td:6940)
    hexagon_V6_veqsf_128B,                     // llvm.hexagon.V6.veqsf.128B (IntrinsicsHexagonDep.td:6943)
    hexagon_V6_veqsf_and,                      // llvm.hexagon.V6.veqsf.and (IntrinsicsHexagonDep.td:6946)
    hexagon_V6_veqsf_and_128B,                 // llvm.hexagon.V6.veqsf.and.128B (IntrinsicsHexagonDep.td:6949)
    hexagon_V6_veqsf_or,                       // llvm.hexagon.V6.veqsf.or (IntrinsicsHexagonDep.td:6952)
    hexagon_V6_veqsf_or_128B,                  // llvm.hexagon.V6.veqsf.or.128B (IntrinsicsHexagonDep.td:6955)
    hexagon_V6_veqsf_xor,                      // llvm.hexagon.V6.veqsf.xor (IntrinsicsHexagonDep.td:6958)
    hexagon_V6_veqsf_xor_128B,                 // llvm.hexagon.V6.veqsf.xor.128B (IntrinsicsHexagonDep.td:6961)
    hexagon_V6_veqw,                           // llvm.hexagon.V6.veqw (IntrinsicsHexagonDep.td:4482)
    hexagon_V6_veqw_128B,                      // llvm.hexagon.V6.veqw.128B (IntrinsicsHexagonDep.td:4485)
    hexagon_V6_veqw_and,                       // llvm.hexagon.V6.veqw.and (IntrinsicsHexagonDep.td:4488)
    hexagon_V6_veqw_and_128B,                  // llvm.hexagon.V6.veqw.and.128B (IntrinsicsHexagonDep.td:4491)
    hexagon_V6_veqw_or,                        // llvm.hexagon.V6.veqw.or (IntrinsicsHexagonDep.td:4494)
    hexagon_V6_veqw_or_128B,                   // llvm.hexagon.V6.veqw.or.128B (IntrinsicsHexagonDep.td:4497)
    hexagon_V6_veqw_xor,                       // llvm.hexagon.V6.veqw.xor (IntrinsicsHexagonDep.td:4500)
    hexagon_V6_veqw_xor_128B,                  // llvm.hexagon.V6.veqw.xor.128B (IntrinsicsHexagonDep.td:4503)
    hexagon_V6_vfmax_f8,                       // llvm.hexagon.V6.vfmax.f8 (IntrinsicsHexagonDep.td:6776)
    hexagon_V6_vfmax_f8_128B,                  // llvm.hexagon.V6.vfmax.f8.128B (IntrinsicsHexagonDep.td:6779)
    hexagon_V6_vfmax_hf,                       // llvm.hexagon.V6.vfmax.hf (IntrinsicsHexagonDep.td:6344)
    hexagon_V6_vfmax_hf_128B,                  // llvm.hexagon.V6.vfmax.hf.128B (IntrinsicsHexagonDep.td:6347)
    hexagon_V6_vfmax_sf,                       // llvm.hexagon.V6.vfmax.sf (IntrinsicsHexagonDep.td:6350)
    hexagon_V6_vfmax_sf_128B,                  // llvm.hexagon.V6.vfmax.sf.128B (IntrinsicsHexagonDep.td:6353)
    hexagon_V6_vfmin_f8,                       // llvm.hexagon.V6.vfmin.f8 (IntrinsicsHexagonDep.td:6782)
    hexagon_V6_vfmin_f8_128B,                  // llvm.hexagon.V6.vfmin.f8.128B (IntrinsicsHexagonDep.td:6785)
    hexagon_V6_vfmin_hf,                       // llvm.hexagon.V6.vfmin.hf (IntrinsicsHexagonDep.td:6356)
    hexagon_V6_vfmin_hf_128B,                  // llvm.hexagon.V6.vfmin.hf.128B (IntrinsicsHexagonDep.td:6359)
    hexagon_V6_vfmin_sf,                       // llvm.hexagon.V6.vfmin.sf (IntrinsicsHexagonDep.td:6362)
    hexagon_V6_vfmin_sf_128B,                  // llvm.hexagon.V6.vfmin.sf.128B (IntrinsicsHexagonDep.td:6365)
    hexagon_V6_vfneg_f8,                       // llvm.hexagon.V6.vfneg.f8 (IntrinsicsHexagonDep.td:6788)
    hexagon_V6_vfneg_f8_128B,                  // llvm.hexagon.V6.vfneg.f8.128B (IntrinsicsHexagonDep.td:6791)
    hexagon_V6_vfneg_hf,                       // llvm.hexagon.V6.vfneg.hf (IntrinsicsHexagonDep.td:6368)
    hexagon_V6_vfneg_hf_128B,                  // llvm.hexagon.V6.vfneg.hf.128B (IntrinsicsHexagonDep.td:6371)
    hexagon_V6_vfneg_sf,                       // llvm.hexagon.V6.vfneg.sf (IntrinsicsHexagonDep.td:6374)
    hexagon_V6_vfneg_sf_128B,                  // llvm.hexagon.V6.vfneg.sf.128B (IntrinsicsHexagonDep.td:6377)
    hexagon_V6_vgather_vscattermh,             // llvm.hexagon.V6.vgather.vscattermh (IntrinsicsHexagon.td:457)
    hexagon_V6_vgather_vscattermh_128B,        // llvm.hexagon.V6.vgather.vscattermh.128B (IntrinsicsHexagon.td:460)
    hexagon_V6_vgathermh,                      // llvm.hexagon.V6.vgathermh (IntrinsicsHexagonDep.td:5950)
    hexagon_V6_vgathermh_128B,                 // llvm.hexagon.V6.vgathermh.128B (IntrinsicsHexagonDep.td:5953)
    hexagon_V6_vgathermhq,                     // llvm.hexagon.V6.vgathermhq (IntrinsicsHexagonDep.td:5956)
    hexagon_V6_vgathermhq_128B,                // llvm.hexagon.V6.vgathermhq.128B (IntrinsicsHexagonDep.td:5959)
    hexagon_V6_vgathermhw,                     // llvm.hexagon.V6.vgathermhw (IntrinsicsHexagonDep.td:5962)
    hexagon_V6_vgathermhw_128B,                // llvm.hexagon.V6.vgathermhw.128B (IntrinsicsHexagonDep.td:5965)
    hexagon_V6_vgathermhwq,                    // llvm.hexagon.V6.vgathermhwq (IntrinsicsHexagonDep.td:5968)
    hexagon_V6_vgathermhwq_128B,               // llvm.hexagon.V6.vgathermhwq.128B (IntrinsicsHexagonDep.td:5971)
    hexagon_V6_vgathermw,                      // llvm.hexagon.V6.vgathermw (IntrinsicsHexagonDep.td:5974)
    hexagon_V6_vgathermw_128B,                 // llvm.hexagon.V6.vgathermw.128B (IntrinsicsHexagonDep.td:5977)
    hexagon_V6_vgathermwq,                     // llvm.hexagon.V6.vgathermwq (IntrinsicsHexagonDep.td:5980)
    hexagon_V6_vgathermwq_128B,                // llvm.hexagon.V6.vgathermwq.128B (IntrinsicsHexagonDep.td:5983)
    hexagon_V6_vgtb,                           // llvm.hexagon.V6.vgtb (IntrinsicsHexagonDep.td:4506)
    hexagon_V6_vgtb_128B,                      // llvm.hexagon.V6.vgtb.128B (IntrinsicsHexagonDep.td:4509)
    hexagon_V6_vgtb_and,                       // llvm.hexagon.V6.vgtb.and (IntrinsicsHexagonDep.td:4512)
    hexagon_V6_vgtb_and_128B,                  // llvm.hexagon.V6.vgtb.and.128B (IntrinsicsHexagonDep.td:4515)
    hexagon_V6_vgtb_or,                        // llvm.hexagon.V6.vgtb.or (IntrinsicsHexagonDep.td:4518)
    hexagon_V6_vgtb_or_128B,                   // llvm.hexagon.V6.vgtb.or.128B (IntrinsicsHexagonDep.td:4521)
    hexagon_V6_vgtb_xor,                       // llvm.hexagon.V6.vgtb.xor (IntrinsicsHexagonDep.td:4524)
    hexagon_V6_vgtb_xor_128B,                  // llvm.hexagon.V6.vgtb.xor.128B (IntrinsicsHexagonDep.td:4527)
    hexagon_V6_vgtbf,                          // llvm.hexagon.V6.vgtbf (IntrinsicsHexagonDep.td:6654)
    hexagon_V6_vgtbf_128B,                     // llvm.hexagon.V6.vgtbf.128B (IntrinsicsHexagonDep.td:6657)
    hexagon_V6_vgtbf_and,                      // llvm.hexagon.V6.vgtbf.and (IntrinsicsHexagonDep.td:6660)
    hexagon_V6_vgtbf_and_128B,                 // llvm.hexagon.V6.vgtbf.and.128B (IntrinsicsHexagonDep.td:6663)
    hexagon_V6_vgtbf_or,                       // llvm.hexagon.V6.vgtbf.or (IntrinsicsHexagonDep.td:6666)
    hexagon_V6_vgtbf_or_128B,                  // llvm.hexagon.V6.vgtbf.or.128B (IntrinsicsHexagonDep.td:6669)
    hexagon_V6_vgtbf_xor,                      // llvm.hexagon.V6.vgtbf.xor (IntrinsicsHexagonDep.td:6672)
    hexagon_V6_vgtbf_xor_128B,                 // llvm.hexagon.V6.vgtbf.xor.128B (IntrinsicsHexagonDep.td:6675)
    hexagon_V6_vgth,                           // llvm.hexagon.V6.vgth (IntrinsicsHexagonDep.td:4530)
    hexagon_V6_vgth_128B,                      // llvm.hexagon.V6.vgth.128B (IntrinsicsHexagonDep.td:4533)
    hexagon_V6_vgth_and,                       // llvm.hexagon.V6.vgth.and (IntrinsicsHexagonDep.td:4536)
    hexagon_V6_vgth_and_128B,                  // llvm.hexagon.V6.vgth.and.128B (IntrinsicsHexagonDep.td:4539)
    hexagon_V6_vgth_or,                        // llvm.hexagon.V6.vgth.or (IntrinsicsHexagonDep.td:4542)
    hexagon_V6_vgth_or_128B,                   // llvm.hexagon.V6.vgth.or.128B (IntrinsicsHexagonDep.td:4545)
    hexagon_V6_vgth_xor,                       // llvm.hexagon.V6.vgth.xor (IntrinsicsHexagonDep.td:4548)
    hexagon_V6_vgth_xor_128B,                  // llvm.hexagon.V6.vgth.xor.128B (IntrinsicsHexagonDep.td:4551)
    hexagon_V6_vgthf,                          // llvm.hexagon.V6.vgthf (IntrinsicsHexagonDep.td:6380)
    hexagon_V6_vgthf_128B,                     // llvm.hexagon.V6.vgthf.128B (IntrinsicsHexagonDep.td:6383)
    hexagon_V6_vgthf_and,                      // llvm.hexagon.V6.vgthf.and (IntrinsicsHexagonDep.td:6386)
    hexagon_V6_vgthf_and_128B,                 // llvm.hexagon.V6.vgthf.and.128B (IntrinsicsHexagonDep.td:6389)
    hexagon_V6_vgthf_or,                       // llvm.hexagon.V6.vgthf.or (IntrinsicsHexagonDep.td:6392)
    hexagon_V6_vgthf_or_128B,                  // llvm.hexagon.V6.vgthf.or.128B (IntrinsicsHexagonDep.td:6395)
    hexagon_V6_vgthf_xor,                      // llvm.hexagon.V6.vgthf.xor (IntrinsicsHexagonDep.td:6398)
    hexagon_V6_vgthf_xor_128B,                 // llvm.hexagon.V6.vgthf.xor.128B (IntrinsicsHexagonDep.td:6401)
    hexagon_V6_vgtsf,                          // llvm.hexagon.V6.vgtsf (IntrinsicsHexagonDep.td:6404)
    hexagon_V6_vgtsf_128B,                     // llvm.hexagon.V6.vgtsf.128B (IntrinsicsHexagonDep.td:6407)
    hexagon_V6_vgtsf_and,                      // llvm.hexagon.V6.vgtsf.and (IntrinsicsHexagonDep.td:6410)
    hexagon_V6_vgtsf_and_128B,                 // llvm.hexagon.V6.vgtsf.and.128B (IntrinsicsHexagonDep.td:6413)
    hexagon_V6_vgtsf_or,                       // llvm.hexagon.V6.vgtsf.or (IntrinsicsHexagonDep.td:6416)
    hexagon_V6_vgtsf_or_128B,                  // llvm.hexagon.V6.vgtsf.or.128B (IntrinsicsHexagonDep.td:6419)
    hexagon_V6_vgtsf_xor,                      // llvm.hexagon.V6.vgtsf.xor (IntrinsicsHexagonDep.td:6422)
    hexagon_V6_vgtsf_xor_128B,                 // llvm.hexagon.V6.vgtsf.xor.128B (IntrinsicsHexagonDep.td:6425)
    hexagon_V6_vgtub,                          // llvm.hexagon.V6.vgtub (IntrinsicsHexagonDep.td:4554)
    hexagon_V6_vgtub_128B,                     // llvm.hexagon.V6.vgtub.128B (IntrinsicsHexagonDep.td:4557)
    hexagon_V6_vgtub_and,                      // llvm.hexagon.V6.vgtub.and (IntrinsicsHexagonDep.td:4560)
    hexagon_V6_vgtub_and_128B,                 // llvm.hexagon.V6.vgtub.and.128B (IntrinsicsHexagonDep.td:4563)
    hexagon_V6_vgtub_or,                       // llvm.hexagon.V6.vgtub.or (IntrinsicsHexagonDep.td:4566)
    hexagon_V6_vgtub_or_128B,                  // llvm.hexagon.V6.vgtub.or.128B (IntrinsicsHexagonDep.td:4569)
    hexagon_V6_vgtub_xor,                      // llvm.hexagon.V6.vgtub.xor (IntrinsicsHexagonDep.td:4572)
    hexagon_V6_vgtub_xor_128B,                 // llvm.hexagon.V6.vgtub.xor.128B (IntrinsicsHexagonDep.td:4575)
    hexagon_V6_vgtuh,                          // llvm.hexagon.V6.vgtuh (IntrinsicsHexagonDep.td:4578)
    hexagon_V6_vgtuh_128B,                     // llvm.hexagon.V6.vgtuh.128B (IntrinsicsHexagonDep.td:4581)
    hexagon_V6_vgtuh_and,                      // llvm.hexagon.V6.vgtuh.and (IntrinsicsHexagonDep.td:4584)
    hexagon_V6_vgtuh_and_128B,                 // llvm.hexagon.V6.vgtuh.and.128B (IntrinsicsHexagonDep.td:4587)
    hexagon_V6_vgtuh_or,                       // llvm.hexagon.V6.vgtuh.or (IntrinsicsHexagonDep.td:4590)
    hexagon_V6_vgtuh_or_128B,                  // llvm.hexagon.V6.vgtuh.or.128B (IntrinsicsHexagonDep.td:4593)
    hexagon_V6_vgtuh_xor,                      // llvm.hexagon.V6.vgtuh.xor (IntrinsicsHexagonDep.td:4596)
    hexagon_V6_vgtuh_xor_128B,                 // llvm.hexagon.V6.vgtuh.xor.128B (IntrinsicsHexagonDep.td:4599)
    hexagon_V6_vgtuw,                          // llvm.hexagon.V6.vgtuw (IntrinsicsHexagonDep.td:4602)
    hexagon_V6_vgtuw_128B,                     // llvm.hexagon.V6.vgtuw.128B (IntrinsicsHexagonDep.td:4605)
    hexagon_V6_vgtuw_and,                      // llvm.hexagon.V6.vgtuw.and (IntrinsicsHexagonDep.td:4608)
    hexagon_V6_vgtuw_and_128B,                 // llvm.hexagon.V6.vgtuw.and.128B (IntrinsicsHexagonDep.td:4611)
    hexagon_V6_vgtuw_or,                       // llvm.hexagon.V6.vgtuw.or (IntrinsicsHexagonDep.td:4614)
    hexagon_V6_vgtuw_or_128B,                  // llvm.hexagon.V6.vgtuw.or.128B (IntrinsicsHexagonDep.td:4617)
    hexagon_V6_vgtuw_xor,                      // llvm.hexagon.V6.vgtuw.xor (IntrinsicsHexagonDep.td:4620)
    hexagon_V6_vgtuw_xor_128B,                 // llvm.hexagon.V6.vgtuw.xor.128B (IntrinsicsHexagonDep.td:4623)
    hexagon_V6_vgtw,                           // llvm.hexagon.V6.vgtw (IntrinsicsHexagonDep.td:4626)
    hexagon_V6_vgtw_128B,                      // llvm.hexagon.V6.vgtw.128B (IntrinsicsHexagonDep.td:4629)
    hexagon_V6_vgtw_and,                       // llvm.hexagon.V6.vgtw.and (IntrinsicsHexagonDep.td:4632)
    hexagon_V6_vgtw_and_128B,                  // llvm.hexagon.V6.vgtw.and.128B (IntrinsicsHexagonDep.td:4635)
    hexagon_V6_vgtw_or,                        // llvm.hexagon.V6.vgtw.or (IntrinsicsHexagonDep.td:4638)
    hexagon_V6_vgtw_or_128B,                   // llvm.hexagon.V6.vgtw.or.128B (IntrinsicsHexagonDep.td:4641)
    hexagon_V6_vgtw_xor,                       // llvm.hexagon.V6.vgtw.xor (IntrinsicsHexagonDep.td:4644)
    hexagon_V6_vgtw_xor_128B,                  // llvm.hexagon.V6.vgtw.xor.128B (IntrinsicsHexagonDep.td:4647)
    hexagon_V6_vilog2_hf,                      // llvm.hexagon.V6.vilog2.hf (IntrinsicsHexagonDep.td:6964)
    hexagon_V6_vilog2_hf_128B,                 // llvm.hexagon.V6.vilog2.hf.128B (IntrinsicsHexagonDep.td:6967)
    hexagon_V6_vilog2_qf16,                    // llvm.hexagon.V6.vilog2.qf16 (IntrinsicsHexagonDep.td:6970)
    hexagon_V6_vilog2_qf16_128B,               // llvm.hexagon.V6.vilog2.qf16.128B (IntrinsicsHexagonDep.td:6973)
    hexagon_V6_vilog2_qf32,                    // llvm.hexagon.V6.vilog2.qf32 (IntrinsicsHexagonDep.td:6976)
    hexagon_V6_vilog2_qf32_128B,               // llvm.hexagon.V6.vilog2.qf32.128B (IntrinsicsHexagonDep.td:6979)
    hexagon_V6_vilog2_sf,                      // llvm.hexagon.V6.vilog2.sf (IntrinsicsHexagonDep.td:6982)
    hexagon_V6_vilog2_sf_128B,                 // llvm.hexagon.V6.vilog2.sf.128B (IntrinsicsHexagonDep.td:6985)
    hexagon_V6_vinsertwr,                      // llvm.hexagon.V6.vinsertwr (IntrinsicsHexagonDep.td:4650)
    hexagon_V6_vinsertwr_128B,                 // llvm.hexagon.V6.vinsertwr.128B (IntrinsicsHexagonDep.td:4653)
    hexagon_V6_vlalignb,                       // llvm.hexagon.V6.vlalignb (IntrinsicsHexagonDep.td:4656)
    hexagon_V6_vlalignb_128B,                  // llvm.hexagon.V6.vlalignb.128B (IntrinsicsHexagonDep.td:4659)
    hexagon_V6_vlalignbi,                      // llvm.hexagon.V6.vlalignbi (IntrinsicsHexagonDep.td:4662)
    hexagon_V6_vlalignbi_128B,                 // llvm.hexagon.V6.vlalignbi.128B (IntrinsicsHexagonDep.td:4665)
    hexagon_V6_vlsrb,                          // llvm.hexagon.V6.vlsrb (IntrinsicsHexagonDep.td:5732)
    hexagon_V6_vlsrb_128B,                     // llvm.hexagon.V6.vlsrb.128B (IntrinsicsHexagonDep.td:5735)
    hexagon_V6_vlsrh,                          // llvm.hexagon.V6.vlsrh (IntrinsicsHexagonDep.td:4668)
    hexagon_V6_vlsrh_128B,                     // llvm.hexagon.V6.vlsrh.128B (IntrinsicsHexagonDep.td:4671)
    hexagon_V6_vlsrhv,                         // llvm.hexagon.V6.vlsrhv (IntrinsicsHexagonDep.td:4674)
    hexagon_V6_vlsrhv_128B,                    // llvm.hexagon.V6.vlsrhv.128B (IntrinsicsHexagonDep.td:4677)
    hexagon_V6_vlsrw,                          // llvm.hexagon.V6.vlsrw (IntrinsicsHexagonDep.td:4680)
    hexagon_V6_vlsrw_128B,                     // llvm.hexagon.V6.vlsrw.128B (IntrinsicsHexagonDep.td:4683)
    hexagon_V6_vlsrwv,                         // llvm.hexagon.V6.vlsrwv (IntrinsicsHexagonDep.td:4686)
    hexagon_V6_vlsrwv_128B,                    // llvm.hexagon.V6.vlsrwv.128B (IntrinsicsHexagonDep.td:4689)
    hexagon_V6_vlut4,                          // llvm.hexagon.V6.vlut4 (IntrinsicsHexagonDep.td:5986)
    hexagon_V6_vlut4_128B,                     // llvm.hexagon.V6.vlut4.128B (IntrinsicsHexagonDep.td:5989)
    hexagon_V6_vlutvvb,                        // llvm.hexagon.V6.vlutvvb (IntrinsicsHexagonDep.td:4692)
    hexagon_V6_vlutvvb_128B,                   // llvm.hexagon.V6.vlutvvb.128B (IntrinsicsHexagonDep.td:4695)
    hexagon_V6_vlutvvb_nm,                     // llvm.hexagon.V6.vlutvvb.nm (IntrinsicsHexagonDep.td:5738)
    hexagon_V6_vlutvvb_nm_128B,                // llvm.hexagon.V6.vlutvvb.nm.128B (IntrinsicsHexagonDep.td:5741)
    hexagon_V6_vlutvvb_oracc,                  // llvm.hexagon.V6.vlutvvb.oracc (IntrinsicsHexagonDep.td:4698)
    hexagon_V6_vlutvvb_oracc_128B,             // llvm.hexagon.V6.vlutvvb.oracc.128B (IntrinsicsHexagonDep.td:4701)
    hexagon_V6_vlutvvb_oracci,                 // llvm.hexagon.V6.vlutvvb.oracci (IntrinsicsHexagonDep.td:5744)
    hexagon_V6_vlutvvb_oracci_128B,            // llvm.hexagon.V6.vlutvvb.oracci.128B (IntrinsicsHexagonDep.td:5747)
    hexagon_V6_vlutvvbi,                       // llvm.hexagon.V6.vlutvvbi (IntrinsicsHexagonDep.td:5750)
    hexagon_V6_vlutvvbi_128B,                  // llvm.hexagon.V6.vlutvvbi.128B (IntrinsicsHexagonDep.td:5753)
    hexagon_V6_vlutvwh,                        // llvm.hexagon.V6.vlutvwh (IntrinsicsHexagonDep.td:4704)
    hexagon_V6_vlutvwh_128B,                   // llvm.hexagon.V6.vlutvwh.128B (IntrinsicsHexagonDep.td:4707)
    hexagon_V6_vlutvwh_nm,                     // llvm.hexagon.V6.vlutvwh.nm (IntrinsicsHexagonDep.td:5756)
    hexagon_V6_vlutvwh_nm_128B,                // llvm.hexagon.V6.vlutvwh.nm.128B (IntrinsicsHexagonDep.td:5759)
    hexagon_V6_vlutvwh_oracc,                  // llvm.hexagon.V6.vlutvwh.oracc (IntrinsicsHexagonDep.td:4710)
    hexagon_V6_vlutvwh_oracc_128B,             // llvm.hexagon.V6.vlutvwh.oracc.128B (IntrinsicsHexagonDep.td:4713)
    hexagon_V6_vlutvwh_oracci,                 // llvm.hexagon.V6.vlutvwh.oracci (IntrinsicsHexagonDep.td:5762)
    hexagon_V6_vlutvwh_oracci_128B,            // llvm.hexagon.V6.vlutvwh.oracci.128B (IntrinsicsHexagonDep.td:5765)
    hexagon_V6_vlutvwhi,                       // llvm.hexagon.V6.vlutvwhi (IntrinsicsHexagonDep.td:5768)
    hexagon_V6_vlutvwhi_128B,                  // llvm.hexagon.V6.vlutvwhi.128B (IntrinsicsHexagonDep.td:5771)
    hexagon_V6_vmaskedstorenq,                 // llvm.hexagon.V6.vmaskedstorenq (IntrinsicsHexagon.td:429)
    hexagon_V6_vmaskedstorenq_128B,            // llvm.hexagon.V6.vmaskedstorenq.128B (IntrinsicsHexagon.td:434)
    hexagon_V6_vmaskedstorentnq,               // llvm.hexagon.V6.vmaskedstorentnq (IntrinsicsHexagon.td:431)
    hexagon_V6_vmaskedstorentnq_128B,          // llvm.hexagon.V6.vmaskedstorentnq.128B (IntrinsicsHexagon.td:436)
    hexagon_V6_vmaskedstorentq,                // llvm.hexagon.V6.vmaskedstorentq (IntrinsicsHexagon.td:430)
    hexagon_V6_vmaskedstorentq_128B,           // llvm.hexagon.V6.vmaskedstorentq.128B (IntrinsicsHexagon.td:435)
    hexagon_V6_vmaskedstoreq,                  // llvm.hexagon.V6.vmaskedstoreq (IntrinsicsHexagon.td:428)
    hexagon_V6_vmaskedstoreq_128B,             // llvm.hexagon.V6.vmaskedstoreq.128B (IntrinsicsHexagon.td:433)
    hexagon_V6_vmax_bf,                        // llvm.hexagon.V6.vmax.bf (IntrinsicsHexagonDep.td:6678)
    hexagon_V6_vmax_bf_128B,                   // llvm.hexagon.V6.vmax.bf.128B (IntrinsicsHexagonDep.td:6681)
    hexagon_V6_vmax_hf,                        // llvm.hexagon.V6.vmax.hf (IntrinsicsHexagonDep.td:6428)
    hexagon_V6_vmax_hf_128B,                   // llvm.hexagon.V6.vmax.hf.128B (IntrinsicsHexagonDep.td:6431)
    hexagon_V6_vmax_sf,                        // llvm.hexagon.V6.vmax.sf (IntrinsicsHexagonDep.td:6434)
    hexagon_V6_vmax_sf_128B,                   // llvm.hexagon.V6.vmax.sf.128B (IntrinsicsHexagonDep.td:6437)
    hexagon_V6_vmaxb,                          // llvm.hexagon.V6.vmaxb (IntrinsicsHexagonDep.td:5774)
    hexagon_V6_vmaxb_128B,                     // llvm.hexagon.V6.vmaxb.128B (IntrinsicsHexagonDep.td:5777)
    hexagon_V6_vmaxh,                          // llvm.hexagon.V6.vmaxh (IntrinsicsHexagonDep.td:4716)
    hexagon_V6_vmaxh_128B,                     // llvm.hexagon.V6.vmaxh.128B (IntrinsicsHexagonDep.td:4719)
    hexagon_V6_vmaxub,                         // llvm.hexagon.V6.vmaxub (IntrinsicsHexagonDep.td:4722)
    hexagon_V6_vmaxub_128B,                    // llvm.hexagon.V6.vmaxub.128B (IntrinsicsHexagonDep.td:4725)
    hexagon_V6_vmaxuh,                         // llvm.hexagon.V6.vmaxuh (IntrinsicsHexagonDep.td:4728)
    hexagon_V6_vmaxuh_128B,                    // llvm.hexagon.V6.vmaxuh.128B (IntrinsicsHexagonDep.td:4731)
    hexagon_V6_vmaxw,                          // llvm.hexagon.V6.vmaxw (IntrinsicsHexagonDep.td:4734)
    hexagon_V6_vmaxw_128B,                     // llvm.hexagon.V6.vmaxw.128B (IntrinsicsHexagonDep.td:4737)
    hexagon_V6_vmerge_qf,                      // llvm.hexagon.V6.vmerge.qf (IntrinsicsHexagonDep.td:6794)
    hexagon_V6_vmerge_qf_128B,                 // llvm.hexagon.V6.vmerge.qf.128B (IntrinsicsHexagonDep.td:6797)
    hexagon_V6_vmin_bf,                        // llvm.hexagon.V6.vmin.bf (IntrinsicsHexagonDep.td:6684)
    hexagon_V6_vmin_bf_128B,                   // llvm.hexagon.V6.vmin.bf.128B (IntrinsicsHexagonDep.td:6687)
    hexagon_V6_vmin_hf,                        // llvm.hexagon.V6.vmin.hf (IntrinsicsHexagonDep.td:6440)
    hexagon_V6_vmin_hf_128B,                   // llvm.hexagon.V6.vmin.hf.128B (IntrinsicsHexagonDep.td:6443)
    hexagon_V6_vmin_sf,                        // llvm.hexagon.V6.vmin.sf (IntrinsicsHexagonDep.td:6446)
    hexagon_V6_vmin_sf_128B,                   // llvm.hexagon.V6.vmin.sf.128B (IntrinsicsHexagonDep.td:6449)
    hexagon_V6_vminb,                          // llvm.hexagon.V6.vminb (IntrinsicsHexagonDep.td:5780)
    hexagon_V6_vminb_128B,                     // llvm.hexagon.V6.vminb.128B (IntrinsicsHexagonDep.td:5783)
    hexagon_V6_vminh,                          // llvm.hexagon.V6.vminh (IntrinsicsHexagonDep.td:4740)
    hexagon_V6_vminh_128B,                     // llvm.hexagon.V6.vminh.128B (IntrinsicsHexagonDep.td:4743)
    hexagon_V6_vminub,                         // llvm.hexagon.V6.vminub (IntrinsicsHexagonDep.td:4746)
    hexagon_V6_vminub_128B,                    // llvm.hexagon.V6.vminub.128B (IntrinsicsHexagonDep.td:4749)
    hexagon_V6_vminuh,                         // llvm.hexagon.V6.vminuh (IntrinsicsHexagonDep.td:4752)
    hexagon_V6_vminuh_128B,                    // llvm.hexagon.V6.vminuh.128B (IntrinsicsHexagonDep.td:4755)
    hexagon_V6_vminw,                          // llvm.hexagon.V6.vminw (IntrinsicsHexagonDep.td:4758)
    hexagon_V6_vminw_128B,                     // llvm.hexagon.V6.vminw.128B (IntrinsicsHexagonDep.td:4761)
    hexagon_V6_vmpabus,                        // llvm.hexagon.V6.vmpabus (IntrinsicsHexagonDep.td:4764)
    hexagon_V6_vmpabus_128B,                   // llvm.hexagon.V6.vmpabus.128B (IntrinsicsHexagonDep.td:4767)
    hexagon_V6_vmpabus_acc,                    // llvm.hexagon.V6.vmpabus.acc (IntrinsicsHexagonDep.td:4770)
    hexagon_V6_vmpabus_acc_128B,               // llvm.hexagon.V6.vmpabus.acc.128B (IntrinsicsHexagonDep.td:4773)
    hexagon_V6_vmpabusv,                       // llvm.hexagon.V6.vmpabusv (IntrinsicsHexagonDep.td:4776)
    hexagon_V6_vmpabusv_128B,                  // llvm.hexagon.V6.vmpabusv.128B (IntrinsicsHexagonDep.td:4779)
    hexagon_V6_vmpabuu,                        // llvm.hexagon.V6.vmpabuu (IntrinsicsHexagonDep.td:5992)
    hexagon_V6_vmpabuu_128B,                   // llvm.hexagon.V6.vmpabuu.128B (IntrinsicsHexagonDep.td:5995)
    hexagon_V6_vmpabuu_acc,                    // llvm.hexagon.V6.vmpabuu.acc (IntrinsicsHexagonDep.td:5998)
    hexagon_V6_vmpabuu_acc_128B,               // llvm.hexagon.V6.vmpabuu.acc.128B (IntrinsicsHexagonDep.td:6001)
    hexagon_V6_vmpabuuv,                       // llvm.hexagon.V6.vmpabuuv (IntrinsicsHexagonDep.td:4782)
    hexagon_V6_vmpabuuv_128B,                  // llvm.hexagon.V6.vmpabuuv.128B (IntrinsicsHexagonDep.td:4785)
    hexagon_V6_vmpahb,                         // llvm.hexagon.V6.vmpahb (IntrinsicsHexagonDep.td:4788)
    hexagon_V6_vmpahb_128B,                    // llvm.hexagon.V6.vmpahb.128B (IntrinsicsHexagonDep.td:4791)
    hexagon_V6_vmpahb_acc,                     // llvm.hexagon.V6.vmpahb.acc (IntrinsicsHexagonDep.td:4794)
    hexagon_V6_vmpahb_acc_128B,                // llvm.hexagon.V6.vmpahb.acc.128B (IntrinsicsHexagonDep.td:4797)
    hexagon_V6_vmpahhsat,                      // llvm.hexagon.V6.vmpahhsat (IntrinsicsHexagonDep.td:6004)
    hexagon_V6_vmpahhsat_128B,                 // llvm.hexagon.V6.vmpahhsat.128B (IntrinsicsHexagonDep.td:6007)
    hexagon_V6_vmpauhb,                        // llvm.hexagon.V6.vmpauhb (IntrinsicsHexagonDep.td:5786)
    hexagon_V6_vmpauhb_128B,                   // llvm.hexagon.V6.vmpauhb.128B (IntrinsicsHexagonDep.td:5789)
    hexagon_V6_vmpauhb_acc,                    // llvm.hexagon.V6.vmpauhb.acc (IntrinsicsHexagonDep.td:5792)
    hexagon_V6_vmpauhb_acc_128B,               // llvm.hexagon.V6.vmpauhb.acc.128B (IntrinsicsHexagonDep.td:5795)
    hexagon_V6_vmpauhuhsat,                    // llvm.hexagon.V6.vmpauhuhsat (IntrinsicsHexagonDep.td:6010)
    hexagon_V6_vmpauhuhsat_128B,               // llvm.hexagon.V6.vmpauhuhsat.128B (IntrinsicsHexagonDep.td:6013)
    hexagon_V6_vmpsuhuhsat,                    // llvm.hexagon.V6.vmpsuhuhsat (IntrinsicsHexagonDep.td:6016)
    hexagon_V6_vmpsuhuhsat_128B,               // llvm.hexagon.V6.vmpsuhuhsat.128B (IntrinsicsHexagonDep.td:6019)
    hexagon_V6_vmpy_hf_f8,                     // llvm.hexagon.V6.vmpy.hf.f8 (IntrinsicsHexagonDep.td:6800)
    hexagon_V6_vmpy_hf_f8_128B,                // llvm.hexagon.V6.vmpy.hf.f8.128B (IntrinsicsHexagonDep.td:6803)
    hexagon_V6_vmpy_hf_f8_acc,                 // llvm.hexagon.V6.vmpy.hf.f8.acc (IntrinsicsHexagonDep.td:6806)
    hexagon_V6_vmpy_hf_f8_acc_128B,            // llvm.hexagon.V6.vmpy.hf.f8.acc.128B (IntrinsicsHexagonDep.td:6809)
    hexagon_V6_vmpy_hf_hf,                     // llvm.hexagon.V6.vmpy.hf.hf (IntrinsicsHexagonDep.td:6452)
    hexagon_V6_vmpy_hf_hf_128B,                // llvm.hexagon.V6.vmpy.hf.hf.128B (IntrinsicsHexagonDep.td:6455)
    hexagon_V6_vmpy_hf_hf_acc,                 // llvm.hexagon.V6.vmpy.hf.hf.acc (IntrinsicsHexagonDep.td:6458)
    hexagon_V6_vmpy_hf_hf_acc_128B,            // llvm.hexagon.V6.vmpy.hf.hf.acc.128B (IntrinsicsHexagonDep.td:6461)
    hexagon_V6_vmpy_qf16,                      // llvm.hexagon.V6.vmpy.qf16 (IntrinsicsHexagonDep.td:6464)
    hexagon_V6_vmpy_qf16_128B,                 // llvm.hexagon.V6.vmpy.qf16.128B (IntrinsicsHexagonDep.td:6467)
    hexagon_V6_vmpy_qf16_hf,                   // llvm.hexagon.V6.vmpy.qf16.hf (IntrinsicsHexagonDep.td:6470)
    hexagon_V6_vmpy_qf16_hf_128B,              // llvm.hexagon.V6.vmpy.qf16.hf.128B (IntrinsicsHexagonDep.td:6473)
    hexagon_V6_vmpy_qf16_mix_hf,               // llvm.hexagon.V6.vmpy.qf16.mix.hf (IntrinsicsHexagonDep.td:6476)
    hexagon_V6_vmpy_qf16_mix_hf_128B,          // llvm.hexagon.V6.vmpy.qf16.mix.hf.128B (IntrinsicsHexagonDep.td:6479)
    hexagon_V6_vmpy_qf32,                      // llvm.hexagon.V6.vmpy.qf32 (IntrinsicsHexagonDep.td:6482)
    hexagon_V6_vmpy_qf32_128B,                 // llvm.hexagon.V6.vmpy.qf32.128B (IntrinsicsHexagonDep.td:6485)
    hexagon_V6_vmpy_qf32_hf,                   // llvm.hexagon.V6.vmpy.qf32.hf (IntrinsicsHexagonDep.td:6488)
    hexagon_V6_vmpy_qf32_hf_128B,              // llvm.hexagon.V6.vmpy.qf32.hf.128B (IntrinsicsHexagonDep.td:6491)
    hexagon_V6_vmpy_qf32_mix_hf,               // llvm.hexagon.V6.vmpy.qf32.mix.hf (IntrinsicsHexagonDep.td:6494)
    hexagon_V6_vmpy_qf32_mix_hf_128B,          // llvm.hexagon.V6.vmpy.qf32.mix.hf.128B (IntrinsicsHexagonDep.td:6497)
    hexagon_V6_vmpy_qf32_qf16,                 // llvm.hexagon.V6.vmpy.qf32.qf16 (IntrinsicsHexagonDep.td:6500)
    hexagon_V6_vmpy_qf32_qf16_128B,            // llvm.hexagon.V6.vmpy.qf32.qf16.128B (IntrinsicsHexagonDep.td:6503)
    hexagon_V6_vmpy_qf32_sf,                   // llvm.hexagon.V6.vmpy.qf32.sf (IntrinsicsHexagonDep.td:6506)
    hexagon_V6_vmpy_qf32_sf_128B,              // llvm.hexagon.V6.vmpy.qf32.sf.128B (IntrinsicsHexagonDep.td:6509)
    hexagon_V6_vmpy_rt_hf,                     // llvm.hexagon.V6.vmpy.rt.hf (IntrinsicsHexagonDep.td:6812)
    hexagon_V6_vmpy_rt_hf_128B,                // llvm.hexagon.V6.vmpy.rt.hf.128B (IntrinsicsHexagonDep.td:6815)
    hexagon_V6_vmpy_rt_qf16,                   // llvm.hexagon.V6.vmpy.rt.qf16 (IntrinsicsHexagonDep.td:6818)
    hexagon_V6_vmpy_rt_qf16_128B,              // llvm.hexagon.V6.vmpy.rt.qf16.128B (IntrinsicsHexagonDep.td:6821)
    hexagon_V6_vmpy_rt_sf,                     // llvm.hexagon.V6.vmpy.rt.sf (IntrinsicsHexagonDep.td:6824)
    hexagon_V6_vmpy_rt_sf_128B,                // llvm.hexagon.V6.vmpy.rt.sf.128B (IntrinsicsHexagonDep.td:6827)
    hexagon_V6_vmpy_sf_bf,                     // llvm.hexagon.V6.vmpy.sf.bf (IntrinsicsHexagonDep.td:6690)
    hexagon_V6_vmpy_sf_bf_128B,                // llvm.hexagon.V6.vmpy.sf.bf.128B (IntrinsicsHexagonDep.td:6693)
    hexagon_V6_vmpy_sf_bf_acc,                 // llvm.hexagon.V6.vmpy.sf.bf.acc (IntrinsicsHexagonDep.td:6696)
    hexagon_V6_vmpy_sf_bf_acc_128B,            // llvm.hexagon.V6.vmpy.sf.bf.acc.128B (IntrinsicsHexagonDep.td:6699)
    hexagon_V6_vmpy_sf_hf,                     // llvm.hexagon.V6.vmpy.sf.hf (IntrinsicsHexagonDep.td:6512)
    hexagon_V6_vmpy_sf_hf_128B,                // llvm.hexagon.V6.vmpy.sf.hf.128B (IntrinsicsHexagonDep.td:6515)
    hexagon_V6_vmpy_sf_hf_acc,                 // llvm.hexagon.V6.vmpy.sf.hf.acc (IntrinsicsHexagonDep.td:6518)
    hexagon_V6_vmpy_sf_hf_acc_128B,            // llvm.hexagon.V6.vmpy.sf.hf.acc.128B (IntrinsicsHexagonDep.td:6521)
    hexagon_V6_vmpy_sf_sf,                     // llvm.hexagon.V6.vmpy.sf.sf (IntrinsicsHexagonDep.td:6524)
    hexagon_V6_vmpy_sf_sf_128B,                // llvm.hexagon.V6.vmpy.sf.sf.128B (IntrinsicsHexagonDep.td:6527)
    hexagon_V6_vmpybus,                        // llvm.hexagon.V6.vmpybus (IntrinsicsHexagonDep.td:4800)
    hexagon_V6_vmpybus_128B,                   // llvm.hexagon.V6.vmpybus.128B (IntrinsicsHexagonDep.td:4803)
    hexagon_V6_vmpybus_acc,                    // llvm.hexagon.V6.vmpybus.acc (IntrinsicsHexagonDep.td:4806)
    hexagon_V6_vmpybus_acc_128B,               // llvm.hexagon.V6.vmpybus.acc.128B (IntrinsicsHexagonDep.td:4809)
    hexagon_V6_vmpybusv,                       // llvm.hexagon.V6.vmpybusv (IntrinsicsHexagonDep.td:4812)
    hexagon_V6_vmpybusv_128B,                  // llvm.hexagon.V6.vmpybusv.128B (IntrinsicsHexagonDep.td:4815)
    hexagon_V6_vmpybusv_acc,                   // llvm.hexagon.V6.vmpybusv.acc (IntrinsicsHexagonDep.td:4818)
    hexagon_V6_vmpybusv_acc_128B,              // llvm.hexagon.V6.vmpybusv.acc.128B (IntrinsicsHexagonDep.td:4821)
    hexagon_V6_vmpybv,                         // llvm.hexagon.V6.vmpybv (IntrinsicsHexagonDep.td:4824)
    hexagon_V6_vmpybv_128B,                    // llvm.hexagon.V6.vmpybv.128B (IntrinsicsHexagonDep.td:4827)
    hexagon_V6_vmpybv_acc,                     // llvm.hexagon.V6.vmpybv.acc (IntrinsicsHexagonDep.td:4830)
    hexagon_V6_vmpybv_acc_128B,                // llvm.hexagon.V6.vmpybv.acc.128B (IntrinsicsHexagonDep.td:4833)
    hexagon_V6_vmpyewuh,                       // llvm.hexagon.V6.vmpyewuh (IntrinsicsHexagonDep.td:4836)
    hexagon_V6_vmpyewuh_128B,                  // llvm.hexagon.V6.vmpyewuh.128B (IntrinsicsHexagonDep.td:4839)
    hexagon_V6_vmpyewuh_64,                    // llvm.hexagon.V6.vmpyewuh.64 (IntrinsicsHexagonDep.td:5798)
    hexagon_V6_vmpyewuh_64_128B,               // llvm.hexagon.V6.vmpyewuh.64.128B (IntrinsicsHexagonDep.td:5801)
    hexagon_V6_vmpyh,                          // llvm.hexagon.V6.vmpyh (IntrinsicsHexagonDep.td:4842)
    hexagon_V6_vmpyh_128B,                     // llvm.hexagon.V6.vmpyh.128B (IntrinsicsHexagonDep.td:4845)
    hexagon_V6_vmpyh_acc,                      // llvm.hexagon.V6.vmpyh.acc (IntrinsicsHexagonDep.td:6022)
    hexagon_V6_vmpyh_acc_128B,                 // llvm.hexagon.V6.vmpyh.acc.128B (IntrinsicsHexagonDep.td:6025)
    hexagon_V6_vmpyhsat_acc,                   // llvm.hexagon.V6.vmpyhsat.acc (IntrinsicsHexagonDep.td:4848)
    hexagon_V6_vmpyhsat_acc_128B,              // llvm.hexagon.V6.vmpyhsat.acc.128B (IntrinsicsHexagonDep.td:4851)
    hexagon_V6_vmpyhsrs,                       // llvm.hexagon.V6.vmpyhsrs (IntrinsicsHexagonDep.td:4854)
    hexagon_V6_vmpyhsrs_128B,                  // llvm.hexagon.V6.vmpyhsrs.128B (IntrinsicsHexagonDep.td:4857)
    hexagon_V6_vmpyhss,                        // llvm.hexagon.V6.vmpyhss (IntrinsicsHexagonDep.td:4860)
    hexagon_V6_vmpyhss_128B,                   // llvm.hexagon.V6.vmpyhss.128B (IntrinsicsHexagonDep.td:4863)
    hexagon_V6_vmpyhus,                        // llvm.hexagon.V6.vmpyhus (IntrinsicsHexagonDep.td:4866)
    hexagon_V6_vmpyhus_128B,                   // llvm.hexagon.V6.vmpyhus.128B (IntrinsicsHexagonDep.td:4869)
    hexagon_V6_vmpyhus_acc,                    // llvm.hexagon.V6.vmpyhus.acc (IntrinsicsHexagonDep.td:4872)
    hexagon_V6_vmpyhus_acc_128B,               // llvm.hexagon.V6.vmpyhus.acc.128B (IntrinsicsHexagonDep.td:4875)
    hexagon_V6_vmpyhv,                         // llvm.hexagon.V6.vmpyhv (IntrinsicsHexagonDep.td:4878)
    hexagon_V6_vmpyhv_128B,                    // llvm.hexagon.V6.vmpyhv.128B (IntrinsicsHexagonDep.td:4881)
    hexagon_V6_vmpyhv_acc,                     // llvm.hexagon.V6.vmpyhv.acc (IntrinsicsHexagonDep.td:4884)
    hexagon_V6_vmpyhv_acc_128B,                // llvm.hexagon.V6.vmpyhv.acc.128B (IntrinsicsHexagonDep.td:4887)
    hexagon_V6_vmpyhvsrs,                      // llvm.hexagon.V6.vmpyhvsrs (IntrinsicsHexagonDep.td:4890)
    hexagon_V6_vmpyhvsrs_128B,                 // llvm.hexagon.V6.vmpyhvsrs.128B (IntrinsicsHexagonDep.td:4893)
    hexagon_V6_vmpyieoh,                       // llvm.hexagon.V6.vmpyieoh (IntrinsicsHexagonDep.td:4896)
    hexagon_V6_vmpyieoh_128B,                  // llvm.hexagon.V6.vmpyieoh.128B (IntrinsicsHexagonDep.td:4899)
    hexagon_V6_vmpyiewh_acc,                   // llvm.hexagon.V6.vmpyiewh.acc (IntrinsicsHexagonDep.td:4902)
    hexagon_V6_vmpyiewh_acc_128B,              // llvm.hexagon.V6.vmpyiewh.acc.128B (IntrinsicsHexagonDep.td:4905)
    hexagon_V6_vmpyiewuh,                      // llvm.hexagon.V6.vmpyiewuh (IntrinsicsHexagonDep.td:4908)
    hexagon_V6_vmpyiewuh_128B,                 // llvm.hexagon.V6.vmpyiewuh.128B (IntrinsicsHexagonDep.td:4911)
    hexagon_V6_vmpyiewuh_acc,                  // llvm.hexagon.V6.vmpyiewuh.acc (IntrinsicsHexagonDep.td:4914)
    hexagon_V6_vmpyiewuh_acc_128B,             // llvm.hexagon.V6.vmpyiewuh.acc.128B (IntrinsicsHexagonDep.td:4917)
    hexagon_V6_vmpyih,                         // llvm.hexagon.V6.vmpyih (IntrinsicsHexagonDep.td:4920)
    hexagon_V6_vmpyih_128B,                    // llvm.hexagon.V6.vmpyih.128B (IntrinsicsHexagonDep.td:4923)
    hexagon_V6_vmpyih_acc,                     // llvm.hexagon.V6.vmpyih.acc (IntrinsicsHexagonDep.td:4926)
    hexagon_V6_vmpyih_acc_128B,                // llvm.hexagon.V6.vmpyih.acc.128B (IntrinsicsHexagonDep.td:4929)
    hexagon_V6_vmpyihb,                        // llvm.hexagon.V6.vmpyihb (IntrinsicsHexagonDep.td:4932)
    hexagon_V6_vmpyihb_128B,                   // llvm.hexagon.V6.vmpyihb.128B (IntrinsicsHexagonDep.td:4935)
    hexagon_V6_vmpyihb_acc,                    // llvm.hexagon.V6.vmpyihb.acc (IntrinsicsHexagonDep.td:4938)
    hexagon_V6_vmpyihb_acc_128B,               // llvm.hexagon.V6.vmpyihb.acc.128B (IntrinsicsHexagonDep.td:4941)
    hexagon_V6_vmpyiowh,                       // llvm.hexagon.V6.vmpyiowh (IntrinsicsHexagonDep.td:4944)
    hexagon_V6_vmpyiowh_128B,                  // llvm.hexagon.V6.vmpyiowh.128B (IntrinsicsHexagonDep.td:4947)
    hexagon_V6_vmpyiwb,                        // llvm.hexagon.V6.vmpyiwb (IntrinsicsHexagonDep.td:4950)
    hexagon_V6_vmpyiwb_128B,                   // llvm.hexagon.V6.vmpyiwb.128B (IntrinsicsHexagonDep.td:4953)
    hexagon_V6_vmpyiwb_acc,                    // llvm.hexagon.V6.vmpyiwb.acc (IntrinsicsHexagonDep.td:4956)
    hexagon_V6_vmpyiwb_acc_128B,               // llvm.hexagon.V6.vmpyiwb.acc.128B (IntrinsicsHexagonDep.td:4959)
    hexagon_V6_vmpyiwh,                        // llvm.hexagon.V6.vmpyiwh (IntrinsicsHexagonDep.td:4962)
    hexagon_V6_vmpyiwh_128B,                   // llvm.hexagon.V6.vmpyiwh.128B (IntrinsicsHexagonDep.td:4965)
    hexagon_V6_vmpyiwh_acc,                    // llvm.hexagon.V6.vmpyiwh.acc (IntrinsicsHexagonDep.td:4968)
    hexagon_V6_vmpyiwh_acc_128B,               // llvm.hexagon.V6.vmpyiwh.acc.128B (IntrinsicsHexagonDep.td:4971)
    hexagon_V6_vmpyiwub,                       // llvm.hexagon.V6.vmpyiwub (IntrinsicsHexagonDep.td:5804)
    hexagon_V6_vmpyiwub_128B,                  // llvm.hexagon.V6.vmpyiwub.128B (IntrinsicsHexagonDep.td:5807)
    hexagon_V6_vmpyiwub_acc,                   // llvm.hexagon.V6.vmpyiwub.acc (IntrinsicsHexagonDep.td:5810)
    hexagon_V6_vmpyiwub_acc_128B,              // llvm.hexagon.V6.vmpyiwub.acc.128B (IntrinsicsHexagonDep.td:5813)
    hexagon_V6_vmpyowh,                        // llvm.hexagon.V6.vmpyowh (IntrinsicsHexagonDep.td:4974)
    hexagon_V6_vmpyowh_128B,                   // llvm.hexagon.V6.vmpyowh.128B (IntrinsicsHexagonDep.td:4977)
    hexagon_V6_vmpyowh_64_acc,                 // llvm.hexagon.V6.vmpyowh.64.acc (IntrinsicsHexagonDep.td:5816)
    hexagon_V6_vmpyowh_64_acc_128B,            // llvm.hexagon.V6.vmpyowh.64.acc.128B (IntrinsicsHexagonDep.td:5819)
    hexagon_V6_vmpyowh_rnd,                    // llvm.hexagon.V6.vmpyowh.rnd (IntrinsicsHexagonDep.td:4980)
    hexagon_V6_vmpyowh_rnd_128B,               // llvm.hexagon.V6.vmpyowh.rnd.128B (IntrinsicsHexagonDep.td:4983)
    hexagon_V6_vmpyowh_rnd_sacc,               // llvm.hexagon.V6.vmpyowh.rnd.sacc (IntrinsicsHexagonDep.td:4986)
    hexagon_V6_vmpyowh_rnd_sacc_128B,          // llvm.hexagon.V6.vmpyowh.rnd.sacc.128B (IntrinsicsHexagonDep.td:4989)
    hexagon_V6_vmpyowh_sacc,                   // llvm.hexagon.V6.vmpyowh.sacc (IntrinsicsHexagonDep.td:4992)
    hexagon_V6_vmpyowh_sacc_128B,              // llvm.hexagon.V6.vmpyowh.sacc.128B (IntrinsicsHexagonDep.td:4995)
    hexagon_V6_vmpyss_parts,                   // llvm.hexagon.V6.vmpyss.parts (IntrinsicsHexagon.td:407)
    hexagon_V6_vmpyss_parts_128B,              // llvm.hexagon.V6.vmpyss.parts.128B (IntrinsicsHexagon.td:408)
    hexagon_V6_vmpyub,                         // llvm.hexagon.V6.vmpyub (IntrinsicsHexagonDep.td:4998)
    hexagon_V6_vmpyub_128B,                    // llvm.hexagon.V6.vmpyub.128B (IntrinsicsHexagonDep.td:5001)
    hexagon_V6_vmpyub_acc,                     // llvm.hexagon.V6.vmpyub.acc (IntrinsicsHexagonDep.td:5004)
    hexagon_V6_vmpyub_acc_128B,                // llvm.hexagon.V6.vmpyub.acc.128B (IntrinsicsHexagonDep.td:5007)
    hexagon_V6_vmpyubv,                        // llvm.hexagon.V6.vmpyubv (IntrinsicsHexagonDep.td:5010)
    hexagon_V6_vmpyubv_128B,                   // llvm.hexagon.V6.vmpyubv.128B (IntrinsicsHexagonDep.td:5013)
    hexagon_V6_vmpyubv_acc,                    // llvm.hexagon.V6.vmpyubv.acc (IntrinsicsHexagonDep.td:5016)
    hexagon_V6_vmpyubv_acc_128B,               // llvm.hexagon.V6.vmpyubv.acc.128B (IntrinsicsHexagonDep.td:5019)
    hexagon_V6_vmpyuh,                         // llvm.hexagon.V6.vmpyuh (IntrinsicsHexagonDep.td:5022)
    hexagon_V6_vmpyuh_128B,                    // llvm.hexagon.V6.vmpyuh.128B (IntrinsicsHexagonDep.td:5025)
    hexagon_V6_vmpyuh_acc,                     // llvm.hexagon.V6.vmpyuh.acc (IntrinsicsHexagonDep.td:5028)
    hexagon_V6_vmpyuh_acc_128B,                // llvm.hexagon.V6.vmpyuh.acc.128B (IntrinsicsHexagonDep.td:5031)
    hexagon_V6_vmpyuhe,                        // llvm.hexagon.V6.vmpyuhe (IntrinsicsHexagonDep.td:6028)
    hexagon_V6_vmpyuhe_128B,                   // llvm.hexagon.V6.vmpyuhe.128B (IntrinsicsHexagonDep.td:6031)
    hexagon_V6_vmpyuhe_acc,                    // llvm.hexagon.V6.vmpyuhe.acc (IntrinsicsHexagonDep.td:6034)
    hexagon_V6_vmpyuhe_acc_128B,               // llvm.hexagon.V6.vmpyuhe.acc.128B (IntrinsicsHexagonDep.td:6037)
    hexagon_V6_vmpyuhv,                        // llvm.hexagon.V6.vmpyuhv (IntrinsicsHexagonDep.td:5034)
    hexagon_V6_vmpyuhv_128B,                   // llvm.hexagon.V6.vmpyuhv.128B (IntrinsicsHexagonDep.td:5037)
    hexagon_V6_vmpyuhv_acc,                    // llvm.hexagon.V6.vmpyuhv.acc (IntrinsicsHexagonDep.td:5040)
    hexagon_V6_vmpyuhv_acc_128B,               // llvm.hexagon.V6.vmpyuhv.acc.128B (IntrinsicsHexagonDep.td:5043)
    hexagon_V6_vmpyuhvs,                       // llvm.hexagon.V6.vmpyuhvs (IntrinsicsHexagonDep.td:6610)
    hexagon_V6_vmpyuhvs_128B,                  // llvm.hexagon.V6.vmpyuhvs.128B (IntrinsicsHexagonDep.td:6613)
    hexagon_V6_vmpyus_parts,                   // llvm.hexagon.V6.vmpyus.parts (IntrinsicsHexagon.td:411)
    hexagon_V6_vmpyus_parts_128B,              // llvm.hexagon.V6.vmpyus.parts.128B (IntrinsicsHexagon.td:412)
    hexagon_V6_vmpyuu_parts,                   // llvm.hexagon.V6.vmpyuu.parts (IntrinsicsHexagon.td:409)
    hexagon_V6_vmpyuu_parts_128B,              // llvm.hexagon.V6.vmpyuu.parts.128B (IntrinsicsHexagon.td:410)
    hexagon_V6_vmux,                           // llvm.hexagon.V6.vmux (IntrinsicsHexagonDep.td:5046)
    hexagon_V6_vmux_128B,                      // llvm.hexagon.V6.vmux.128B (IntrinsicsHexagonDep.td:5049)
    hexagon_V6_vnavgb,                         // llvm.hexagon.V6.vnavgb (IntrinsicsHexagonDep.td:6040)
    hexagon_V6_vnavgb_128B,                    // llvm.hexagon.V6.vnavgb.128B (IntrinsicsHexagonDep.td:6043)
    hexagon_V6_vnavgh,                         // llvm.hexagon.V6.vnavgh (IntrinsicsHexagonDep.td:5052)
    hexagon_V6_vnavgh_128B,                    // llvm.hexagon.V6.vnavgh.128B (IntrinsicsHexagonDep.td:5055)
    hexagon_V6_vnavgub,                        // llvm.hexagon.V6.vnavgub (IntrinsicsHexagonDep.td:5058)
    hexagon_V6_vnavgub_128B,                   // llvm.hexagon.V6.vnavgub.128B (IntrinsicsHexagonDep.td:5061)
    hexagon_V6_vnavgw,                         // llvm.hexagon.V6.vnavgw (IntrinsicsHexagonDep.td:5064)
    hexagon_V6_vnavgw_128B,                    // llvm.hexagon.V6.vnavgw.128B (IntrinsicsHexagonDep.td:5067)
    hexagon_V6_vneg_qf16_hf,                   // llvm.hexagon.V6.vneg.qf16.hf (IntrinsicsHexagonDep.td:6988)
    hexagon_V6_vneg_qf16_hf_128B,              // llvm.hexagon.V6.vneg.qf16.hf.128B (IntrinsicsHexagonDep.td:6991)
    hexagon_V6_vneg_qf16_qf16,                 // llvm.hexagon.V6.vneg.qf16.qf16 (IntrinsicsHexagonDep.td:6994)
    hexagon_V6_vneg_qf16_qf16_128B,            // llvm.hexagon.V6.vneg.qf16.qf16.128B (IntrinsicsHexagonDep.td:6997)
    hexagon_V6_vneg_qf32_qf32,                 // llvm.hexagon.V6.vneg.qf32.qf32 (IntrinsicsHexagonDep.td:7000)
    hexagon_V6_vneg_qf32_qf32_128B,            // llvm.hexagon.V6.vneg.qf32.qf32.128B (IntrinsicsHexagonDep.td:7003)
    hexagon_V6_vneg_qf32_sf,                   // llvm.hexagon.V6.vneg.qf32.sf (IntrinsicsHexagonDep.td:7006)
    hexagon_V6_vneg_qf32_sf_128B,              // llvm.hexagon.V6.vneg.qf32.sf.128B (IntrinsicsHexagonDep.td:7009)
    hexagon_V6_vnormamth,                      // llvm.hexagon.V6.vnormamth (IntrinsicsHexagonDep.td:5070)
    hexagon_V6_vnormamth_128B,                 // llvm.hexagon.V6.vnormamth.128B (IntrinsicsHexagonDep.td:5073)
    hexagon_V6_vnormamtw,                      // llvm.hexagon.V6.vnormamtw (IntrinsicsHexagonDep.td:5076)
    hexagon_V6_vnormamtw_128B,                 // llvm.hexagon.V6.vnormamtw.128B (IntrinsicsHexagonDep.td:5079)
    hexagon_V6_vnot,                           // llvm.hexagon.V6.vnot (IntrinsicsHexagonDep.td:5082)
    hexagon_V6_vnot_128B,                      // llvm.hexagon.V6.vnot.128B (IntrinsicsHexagonDep.td:5085)
    hexagon_V6_vor,                            // llvm.hexagon.V6.vor (IntrinsicsHexagonDep.td:5088)
    hexagon_V6_vor_128B,                       // llvm.hexagon.V6.vor.128B (IntrinsicsHexagonDep.td:5091)
    hexagon_V6_vpackeb,                        // llvm.hexagon.V6.vpackeb (IntrinsicsHexagonDep.td:5094)
    hexagon_V6_vpackeb_128B,                   // llvm.hexagon.V6.vpackeb.128B (IntrinsicsHexagonDep.td:5097)
    hexagon_V6_vpackeh,                        // llvm.hexagon.V6.vpackeh (IntrinsicsHexagonDep.td:5100)
    hexagon_V6_vpackeh_128B,                   // llvm.hexagon.V6.vpackeh.128B (IntrinsicsHexagonDep.td:5103)
    hexagon_V6_vpackhb_sat,                    // llvm.hexagon.V6.vpackhb.sat (IntrinsicsHexagonDep.td:5106)
    hexagon_V6_vpackhb_sat_128B,               // llvm.hexagon.V6.vpackhb.sat.128B (IntrinsicsHexagonDep.td:5109)
    hexagon_V6_vpackhub_sat,                   // llvm.hexagon.V6.vpackhub.sat (IntrinsicsHexagonDep.td:5112)
    hexagon_V6_vpackhub_sat_128B,              // llvm.hexagon.V6.vpackhub.sat.128B (IntrinsicsHexagonDep.td:5115)
    hexagon_V6_vpackob,                        // llvm.hexagon.V6.vpackob (IntrinsicsHexagonDep.td:5118)
    hexagon_V6_vpackob_128B,                   // llvm.hexagon.V6.vpackob.128B (IntrinsicsHexagonDep.td:5121)
    hexagon_V6_vpackoh,                        // llvm.hexagon.V6.vpackoh (IntrinsicsHexagonDep.td:5124)
    hexagon_V6_vpackoh_128B,                   // llvm.hexagon.V6.vpackoh.128B (IntrinsicsHexagonDep.td:5127)
    hexagon_V6_vpackwh_sat,                    // llvm.hexagon.V6.vpackwh.sat (IntrinsicsHexagonDep.td:5130)
    hexagon_V6_vpackwh_sat_128B,               // llvm.hexagon.V6.vpackwh.sat.128B (IntrinsicsHexagonDep.td:5133)
    hexagon_V6_vpackwuh_sat,                   // llvm.hexagon.V6.vpackwuh.sat (IntrinsicsHexagonDep.td:5136)
    hexagon_V6_vpackwuh_sat_128B,              // llvm.hexagon.V6.vpackwuh.sat.128B (IntrinsicsHexagonDep.td:5139)
    hexagon_V6_vpopcounth,                     // llvm.hexagon.V6.vpopcounth (IntrinsicsHexagonDep.td:5142)
    hexagon_V6_vpopcounth_128B,                // llvm.hexagon.V6.vpopcounth.128B (IntrinsicsHexagonDep.td:5145)
    hexagon_V6_vprefixqb,                      // llvm.hexagon.V6.vprefixqb (IntrinsicsHexagonDep.td:6046)
    hexagon_V6_vprefixqb_128B,                 // llvm.hexagon.V6.vprefixqb.128B (IntrinsicsHexagonDep.td:6049)
    hexagon_V6_vprefixqh,                      // llvm.hexagon.V6.vprefixqh (IntrinsicsHexagonDep.td:6052)
    hexagon_V6_vprefixqh_128B,                 // llvm.hexagon.V6.vprefixqh.128B (IntrinsicsHexagonDep.td:6055)
    hexagon_V6_vprefixqw,                      // llvm.hexagon.V6.vprefixqw (IntrinsicsHexagonDep.td:6058)
    hexagon_V6_vprefixqw_128B,                 // llvm.hexagon.V6.vprefixqw.128B (IntrinsicsHexagonDep.td:6061)
    hexagon_V6_vrdelta,                        // llvm.hexagon.V6.vrdelta (IntrinsicsHexagonDep.td:5148)
    hexagon_V6_vrdelta_128B,                   // llvm.hexagon.V6.vrdelta.128B (IntrinsicsHexagonDep.td:5151)
    hexagon_V6_vrmpybub_rtt,                   // llvm.hexagon.V6.vrmpybub.rtt (IntrinsicsHexagon.td:237)
    hexagon_V6_vrmpybub_rtt_128B,              // llvm.hexagon.V6.vrmpybub.rtt.128B (IntrinsicsHexagon.td:240)
    hexagon_V6_vrmpybub_rtt_acc,               // llvm.hexagon.V6.vrmpybub.rtt.acc (IntrinsicsHexagon.td:243)
    hexagon_V6_vrmpybub_rtt_acc_128B,          // llvm.hexagon.V6.vrmpybub.rtt.acc.128B (IntrinsicsHexagon.td:246)
    hexagon_V6_vrmpybus,                       // llvm.hexagon.V6.vrmpybus (IntrinsicsHexagonDep.td:5154)
    hexagon_V6_vrmpybus_128B,                  // llvm.hexagon.V6.vrmpybus.128B (IntrinsicsHexagonDep.td:5157)
    hexagon_V6_vrmpybus_acc,                   // llvm.hexagon.V6.vrmpybus.acc (IntrinsicsHexagonDep.td:5160)
    hexagon_V6_vrmpybus_acc_128B,              // llvm.hexagon.V6.vrmpybus.acc.128B (IntrinsicsHexagonDep.td:5163)
    hexagon_V6_vrmpybusi,                      // llvm.hexagon.V6.vrmpybusi (IntrinsicsHexagonDep.td:5166)
    hexagon_V6_vrmpybusi_128B,                 // llvm.hexagon.V6.vrmpybusi.128B (IntrinsicsHexagonDep.td:5169)
    hexagon_V6_vrmpybusi_acc,                  // llvm.hexagon.V6.vrmpybusi.acc (IntrinsicsHexagonDep.td:5172)
    hexagon_V6_vrmpybusi_acc_128B,             // llvm.hexagon.V6.vrmpybusi.acc.128B (IntrinsicsHexagonDep.td:5175)
    hexagon_V6_vrmpybusv,                      // llvm.hexagon.V6.vrmpybusv (IntrinsicsHexagonDep.td:5178)
    hexagon_V6_vrmpybusv_128B,                 // llvm.hexagon.V6.vrmpybusv.128B (IntrinsicsHexagonDep.td:5181)
    hexagon_V6_vrmpybusv_acc,                  // llvm.hexagon.V6.vrmpybusv.acc (IntrinsicsHexagonDep.td:5184)
    hexagon_V6_vrmpybusv_acc_128B,             // llvm.hexagon.V6.vrmpybusv.acc.128B (IntrinsicsHexagonDep.td:5187)
    hexagon_V6_vrmpybv,                        // llvm.hexagon.V6.vrmpybv (IntrinsicsHexagonDep.td:5190)
    hexagon_V6_vrmpybv_128B,                   // llvm.hexagon.V6.vrmpybv.128B (IntrinsicsHexagonDep.td:5193)
    hexagon_V6_vrmpybv_acc,                    // llvm.hexagon.V6.vrmpybv.acc (IntrinsicsHexagonDep.td:5196)
    hexagon_V6_vrmpybv_acc_128B,               // llvm.hexagon.V6.vrmpybv.acc.128B (IntrinsicsHexagonDep.td:5199)
    hexagon_V6_vrmpyub,                        // llvm.hexagon.V6.vrmpyub (IntrinsicsHexagonDep.td:5202)
    hexagon_V6_vrmpyub_128B,                   // llvm.hexagon.V6.vrmpyub.128B (IntrinsicsHexagonDep.td:5205)
    hexagon_V6_vrmpyub_acc,                    // llvm.hexagon.V6.vrmpyub.acc (IntrinsicsHexagonDep.td:5208)
    hexagon_V6_vrmpyub_acc_128B,               // llvm.hexagon.V6.vrmpyub.acc.128B (IntrinsicsHexagonDep.td:5211)
    hexagon_V6_vrmpyub_rtt,                    // llvm.hexagon.V6.vrmpyub.rtt (IntrinsicsHexagon.td:249)
    hexagon_V6_vrmpyub_rtt_128B,               // llvm.hexagon.V6.vrmpyub.rtt.128B (IntrinsicsHexagon.td:252)
    hexagon_V6_vrmpyub_rtt_acc,                // llvm.hexagon.V6.vrmpyub.rtt.acc (IntrinsicsHexagon.td:255)
    hexagon_V6_vrmpyub_rtt_acc_128B,           // llvm.hexagon.V6.vrmpyub.rtt.acc.128B (IntrinsicsHexagon.td:258)
    hexagon_V6_vrmpyubi,                       // llvm.hexagon.V6.vrmpyubi (IntrinsicsHexagonDep.td:5214)
    hexagon_V6_vrmpyubi_128B,                  // llvm.hexagon.V6.vrmpyubi.128B (IntrinsicsHexagonDep.td:5217)
    hexagon_V6_vrmpyubi_acc,                   // llvm.hexagon.V6.vrmpyubi.acc (IntrinsicsHexagonDep.td:5220)
    hexagon_V6_vrmpyubi_acc_128B,              // llvm.hexagon.V6.vrmpyubi.acc.128B (IntrinsicsHexagonDep.td:5223)
    hexagon_V6_vrmpyubv,                       // llvm.hexagon.V6.vrmpyubv (IntrinsicsHexagonDep.td:5226)
    hexagon_V6_vrmpyubv_128B,                  // llvm.hexagon.V6.vrmpyubv.128B (IntrinsicsHexagonDep.td:5229)
    hexagon_V6_vrmpyubv_acc,                   // llvm.hexagon.V6.vrmpyubv.acc (IntrinsicsHexagonDep.td:5232)
    hexagon_V6_vrmpyubv_acc_128B,              // llvm.hexagon.V6.vrmpyubv.acc.128B (IntrinsicsHexagonDep.td:5235)
    hexagon_V6_vror,                           // llvm.hexagon.V6.vror (IntrinsicsHexagonDep.td:5238)
    hexagon_V6_vror_128B,                      // llvm.hexagon.V6.vror.128B (IntrinsicsHexagonDep.td:5241)
    hexagon_V6_vrotr,                          // llvm.hexagon.V6.vrotr (IntrinsicsHexagonDep.td:6138)
    hexagon_V6_vrotr_128B,                     // llvm.hexagon.V6.vrotr.128B (IntrinsicsHexagonDep.td:6141)
    hexagon_V6_vroundhb,                       // llvm.hexagon.V6.vroundhb (IntrinsicsHexagonDep.td:5244)
    hexagon_V6_vroundhb_128B,                  // llvm.hexagon.V6.vroundhb.128B (IntrinsicsHexagonDep.td:5247)
    hexagon_V6_vroundhub,                      // llvm.hexagon.V6.vroundhub (IntrinsicsHexagonDep.td:5250)
    hexagon_V6_vroundhub_128B,                 // llvm.hexagon.V6.vroundhub.128B (IntrinsicsHexagonDep.td:5253)
    hexagon_V6_vrounduhub,                     // llvm.hexagon.V6.vrounduhub (IntrinsicsHexagonDep.td:5822)
    hexagon_V6_vrounduhub_128B,                // llvm.hexagon.V6.vrounduhub.128B (IntrinsicsHexagonDep.td:5825)
    hexagon_V6_vrounduwuh,                     // llvm.hexagon.V6.vrounduwuh (IntrinsicsHexagonDep.td:5828)
    hexagon_V6_vrounduwuh_128B,                // llvm.hexagon.V6.vrounduwuh.128B (IntrinsicsHexagonDep.td:5831)
    hexagon_V6_vroundwh,                       // llvm.hexagon.V6.vroundwh (IntrinsicsHexagonDep.td:5256)
    hexagon_V6_vroundwh_128B,                  // llvm.hexagon.V6.vroundwh.128B (IntrinsicsHexagonDep.td:5259)
    hexagon_V6_vroundwuh,                      // llvm.hexagon.V6.vroundwuh (IntrinsicsHexagonDep.td:5262)
    hexagon_V6_vroundwuh_128B,                 // llvm.hexagon.V6.vroundwuh.128B (IntrinsicsHexagonDep.td:5265)
    hexagon_V6_vrsadubi,                       // llvm.hexagon.V6.vrsadubi (IntrinsicsHexagonDep.td:5268)
    hexagon_V6_vrsadubi_128B,                  // llvm.hexagon.V6.vrsadubi.128B (IntrinsicsHexagonDep.td:5271)
    hexagon_V6_vrsadubi_acc,                   // llvm.hexagon.V6.vrsadubi.acc (IntrinsicsHexagonDep.td:5274)
    hexagon_V6_vrsadubi_acc_128B,              // llvm.hexagon.V6.vrsadubi.acc.128B (IntrinsicsHexagonDep.td:5277)
    hexagon_V6_vsatdw,                         // llvm.hexagon.V6.vsatdw (IntrinsicsHexagonDep.td:6144)
    hexagon_V6_vsatdw_128B,                    // llvm.hexagon.V6.vsatdw.128B (IntrinsicsHexagonDep.td:6147)
    hexagon_V6_vsathub,                        // llvm.hexagon.V6.vsathub (IntrinsicsHexagonDep.td:5280)
    hexagon_V6_vsathub_128B,                   // llvm.hexagon.V6.vsathub.128B (IntrinsicsHexagonDep.td:5283)
    hexagon_V6_vsatuwuh,                       // llvm.hexagon.V6.vsatuwuh (IntrinsicsHexagonDep.td:5834)
    hexagon_V6_vsatuwuh_128B,                  // llvm.hexagon.V6.vsatuwuh.128B (IntrinsicsHexagonDep.td:5837)
    hexagon_V6_vsatwh,                         // llvm.hexagon.V6.vsatwh (IntrinsicsHexagonDep.td:5286)
    hexagon_V6_vsatwh_128B,                    // llvm.hexagon.V6.vsatwh.128B (IntrinsicsHexagonDep.td:5289)
    hexagon_V6_vsb,                            // llvm.hexagon.V6.vsb (IntrinsicsHexagonDep.td:5292)
    hexagon_V6_vsb_128B,                       // llvm.hexagon.V6.vsb.128B (IntrinsicsHexagonDep.td:5295)
    hexagon_V6_vscattermh,                     // llvm.hexagon.V6.vscattermh (IntrinsicsHexagonDep.td:6064)
    hexagon_V6_vscattermh_128B,                // llvm.hexagon.V6.vscattermh.128B (IntrinsicsHexagonDep.td:6067)
    hexagon_V6_vscattermh_add,                 // llvm.hexagon.V6.vscattermh.add (IntrinsicsHexagonDep.td:6070)
    hexagon_V6_vscattermh_add_128B,            // llvm.hexagon.V6.vscattermh.add.128B (IntrinsicsHexagonDep.td:6073)
    hexagon_V6_vscattermhq,                    // llvm.hexagon.V6.vscattermhq (IntrinsicsHexagonDep.td:6076)
    hexagon_V6_vscattermhq_128B,               // llvm.hexagon.V6.vscattermhq.128B (IntrinsicsHexagonDep.td:6079)
    hexagon_V6_vscattermhw,                    // llvm.hexagon.V6.vscattermhw (IntrinsicsHexagonDep.td:6082)
    hexagon_V6_vscattermhw_128B,               // llvm.hexagon.V6.vscattermhw.128B (IntrinsicsHexagonDep.td:6085)
    hexagon_V6_vscattermhw_add,                // llvm.hexagon.V6.vscattermhw.add (IntrinsicsHexagonDep.td:6088)
    hexagon_V6_vscattermhw_add_128B,           // llvm.hexagon.V6.vscattermhw.add.128B (IntrinsicsHexagonDep.td:6091)
    hexagon_V6_vscattermhwq,                   // llvm.hexagon.V6.vscattermhwq (IntrinsicsHexagonDep.td:6094)
    hexagon_V6_vscattermhwq_128B,              // llvm.hexagon.V6.vscattermhwq.128B (IntrinsicsHexagonDep.td:6097)
    hexagon_V6_vscattermw,                     // llvm.hexagon.V6.vscattermw (IntrinsicsHexagonDep.td:6100)
    hexagon_V6_vscattermw_128B,                // llvm.hexagon.V6.vscattermw.128B (IntrinsicsHexagonDep.td:6103)
    hexagon_V6_vscattermw_add,                 // llvm.hexagon.V6.vscattermw.add (IntrinsicsHexagonDep.td:6106)
    hexagon_V6_vscattermw_add_128B,            // llvm.hexagon.V6.vscattermw.add.128B (IntrinsicsHexagonDep.td:6109)
    hexagon_V6_vscattermwq,                    // llvm.hexagon.V6.vscattermwq (IntrinsicsHexagonDep.td:6112)
    hexagon_V6_vscattermwq_128B,               // llvm.hexagon.V6.vscattermwq.128B (IntrinsicsHexagonDep.td:6115)
    hexagon_V6_vsh,                            // llvm.hexagon.V6.vsh (IntrinsicsHexagonDep.td:5298)
    hexagon_V6_vsh_128B,                       // llvm.hexagon.V6.vsh.128B (IntrinsicsHexagonDep.td:5301)
    hexagon_V6_vshufeh,                        // llvm.hexagon.V6.vshufeh (IntrinsicsHexagonDep.td:5304)
    hexagon_V6_vshufeh_128B,                   // llvm.hexagon.V6.vshufeh.128B (IntrinsicsHexagonDep.td:5307)
    hexagon_V6_vshuffb,                        // llvm.hexagon.V6.vshuffb (IntrinsicsHexagonDep.td:5310)
    hexagon_V6_vshuffb_128B,                   // llvm.hexagon.V6.vshuffb.128B (IntrinsicsHexagonDep.td:5313)
    hexagon_V6_vshuffeb,                       // llvm.hexagon.V6.vshuffeb (IntrinsicsHexagonDep.td:5316)
    hexagon_V6_vshuffeb_128B,                  // llvm.hexagon.V6.vshuffeb.128B (IntrinsicsHexagonDep.td:5319)
    hexagon_V6_vshuffh,                        // llvm.hexagon.V6.vshuffh (IntrinsicsHexagonDep.td:5322)
    hexagon_V6_vshuffh_128B,                   // llvm.hexagon.V6.vshuffh.128B (IntrinsicsHexagonDep.td:5325)
    hexagon_V6_vshuffob,                       // llvm.hexagon.V6.vshuffob (IntrinsicsHexagonDep.td:5328)
    hexagon_V6_vshuffob_128B,                  // llvm.hexagon.V6.vshuffob.128B (IntrinsicsHexagonDep.td:5331)
    hexagon_V6_vshuffvdd,                      // llvm.hexagon.V6.vshuffvdd (IntrinsicsHexagonDep.td:5334)
    hexagon_V6_vshuffvdd_128B,                 // llvm.hexagon.V6.vshuffvdd.128B (IntrinsicsHexagonDep.td:5337)
    hexagon_V6_vshufoeb,                       // llvm.hexagon.V6.vshufoeb (IntrinsicsHexagonDep.td:5340)
    hexagon_V6_vshufoeb_128B,                  // llvm.hexagon.V6.vshufoeb.128B (IntrinsicsHexagonDep.td:5343)
    hexagon_V6_vshufoeh,                       // llvm.hexagon.V6.vshufoeh (IntrinsicsHexagonDep.td:5346)
    hexagon_V6_vshufoeh_128B,                  // llvm.hexagon.V6.vshufoeh.128B (IntrinsicsHexagonDep.td:5349)
    hexagon_V6_vshufoh,                        // llvm.hexagon.V6.vshufoh (IntrinsicsHexagonDep.td:5352)
    hexagon_V6_vshufoh_128B,                   // llvm.hexagon.V6.vshufoh.128B (IntrinsicsHexagonDep.td:5355)
    hexagon_V6_vsub_hf,                        // llvm.hexagon.V6.vsub.hf (IntrinsicsHexagonDep.td:6530)
    hexagon_V6_vsub_hf_128B,                   // llvm.hexagon.V6.vsub.hf.128B (IntrinsicsHexagonDep.td:6533)
    hexagon_V6_vsub_hf_f8,                     // llvm.hexagon.V6.vsub.hf.f8 (IntrinsicsHexagonDep.td:6830)
    hexagon_V6_vsub_hf_f8_128B,                // llvm.hexagon.V6.vsub.hf.f8.128B (IntrinsicsHexagonDep.td:6833)
    hexagon_V6_vsub_hf_hf,                     // llvm.hexagon.V6.vsub.hf.hf (IntrinsicsHexagonDep.td:6536)
    hexagon_V6_vsub_hf_hf_128B,                // llvm.hexagon.V6.vsub.hf.hf.128B (IntrinsicsHexagonDep.td:6539)
    hexagon_V6_vsub_hf_mix,                    // llvm.hexagon.V6.vsub.hf.mix (IntrinsicsHexagonDep.td:7012)
    hexagon_V6_vsub_hf_mix_128B,               // llvm.hexagon.V6.vsub.hf.mix.128B (IntrinsicsHexagonDep.td:7015)
    hexagon_V6_vsub_qf16,                      // llvm.hexagon.V6.vsub.qf16 (IntrinsicsHexagonDep.td:6542)
    hexagon_V6_vsub_qf16_128B,                 // llvm.hexagon.V6.vsub.qf16.128B (IntrinsicsHexagonDep.td:6545)
    hexagon_V6_vsub_qf16_mix,                  // llvm.hexagon.V6.vsub.qf16.mix (IntrinsicsHexagonDep.td:6548)
    hexagon_V6_vsub_qf16_mix_128B,             // llvm.hexagon.V6.vsub.qf16.mix.128B (IntrinsicsHexagonDep.td:6551)
    hexagon_V6_vsub_qf32,                      // llvm.hexagon.V6.vsub.qf32 (IntrinsicsHexagonDep.td:6554)
    hexagon_V6_vsub_qf32_128B,                 // llvm.hexagon.V6.vsub.qf32.128B (IntrinsicsHexagonDep.td:6557)
    hexagon_V6_vsub_qf32_mix,                  // llvm.hexagon.V6.vsub.qf32.mix (IntrinsicsHexagonDep.td:6560)
    hexagon_V6_vsub_qf32_mix_128B,             // llvm.hexagon.V6.vsub.qf32.mix.128B (IntrinsicsHexagonDep.td:6563)
    hexagon_V6_vsub_sf,                        // llvm.hexagon.V6.vsub.sf (IntrinsicsHexagonDep.td:6566)
    hexagon_V6_vsub_sf_128B,                   // llvm.hexagon.V6.vsub.sf.128B (IntrinsicsHexagonDep.td:6569)
    hexagon_V6_vsub_sf_bf,                     // llvm.hexagon.V6.vsub.sf.bf (IntrinsicsHexagonDep.td:6702)
    hexagon_V6_vsub_sf_bf_128B,                // llvm.hexagon.V6.vsub.sf.bf.128B (IntrinsicsHexagonDep.td:6705)
    hexagon_V6_vsub_sf_hf,                     // llvm.hexagon.V6.vsub.sf.hf (IntrinsicsHexagonDep.td:6572)
    hexagon_V6_vsub_sf_hf_128B,                // llvm.hexagon.V6.vsub.sf.hf.128B (IntrinsicsHexagonDep.td:6575)
    hexagon_V6_vsub_sf_mix,                    // llvm.hexagon.V6.vsub.sf.mix (IntrinsicsHexagonDep.td:7018)
    hexagon_V6_vsub_sf_mix_128B,               // llvm.hexagon.V6.vsub.sf.mix.128B (IntrinsicsHexagonDep.td:7021)
    hexagon_V6_vsub_sf_sf,                     // llvm.hexagon.V6.vsub.sf.sf (IntrinsicsHexagonDep.td:6578)
    hexagon_V6_vsub_sf_sf_128B,                // llvm.hexagon.V6.vsub.sf.sf.128B (IntrinsicsHexagonDep.td:6581)
    hexagon_V6_vsubb,                          // llvm.hexagon.V6.vsubb (IntrinsicsHexagonDep.td:5358)
    hexagon_V6_vsubb_128B,                     // llvm.hexagon.V6.vsubb.128B (IntrinsicsHexagonDep.td:5361)
    hexagon_V6_vsubb_dv,                       // llvm.hexagon.V6.vsubb.dv (IntrinsicsHexagonDep.td:5364)
    hexagon_V6_vsubb_dv_128B,                  // llvm.hexagon.V6.vsubb.dv.128B (IntrinsicsHexagonDep.td:5367)
    hexagon_V6_vsubbnq,                        // llvm.hexagon.V6.vsubbnq (IntrinsicsHexagonDep.td:5370)
    hexagon_V6_vsubbnq_128B,                   // llvm.hexagon.V6.vsubbnq.128B (IntrinsicsHexagonDep.td:5373)
    hexagon_V6_vsubbq,                         // llvm.hexagon.V6.vsubbq (IntrinsicsHexagonDep.td:5376)
    hexagon_V6_vsubbq_128B,                    // llvm.hexagon.V6.vsubbq.128B (IntrinsicsHexagonDep.td:5379)
    hexagon_V6_vsubbsat,                       // llvm.hexagon.V6.vsubbsat (IntrinsicsHexagonDep.td:5840)
    hexagon_V6_vsubbsat_128B,                  // llvm.hexagon.V6.vsubbsat.128B (IntrinsicsHexagonDep.td:5843)
    hexagon_V6_vsubbsat_dv,                    // llvm.hexagon.V6.vsubbsat.dv (IntrinsicsHexagonDep.td:5846)
    hexagon_V6_vsubbsat_dv_128B,               // llvm.hexagon.V6.vsubbsat.dv.128B (IntrinsicsHexagonDep.td:5849)
    hexagon_V6_vsubcarry,                      // llvm.hexagon.V6.vsubcarry (IntrinsicsHexagonDep.td:5852)
    hexagon_V6_vsubcarry_128B,                 // llvm.hexagon.V6.vsubcarry.128B (IntrinsicsHexagonDep.td:5855)
    hexagon_V6_vsubcarryo,                     // llvm.hexagon.V6.vsubcarryo (IntrinsicsHexagonDep.td:6150)
    hexagon_V6_vsubcarryo_128B,                // llvm.hexagon.V6.vsubcarryo.128B (IntrinsicsHexagonDep.td:6153)
    hexagon_V6_vsubh,                          // llvm.hexagon.V6.vsubh (IntrinsicsHexagonDep.td:5382)
    hexagon_V6_vsubh_128B,                     // llvm.hexagon.V6.vsubh.128B (IntrinsicsHexagonDep.td:5385)
    hexagon_V6_vsubh_dv,                       // llvm.hexagon.V6.vsubh.dv (IntrinsicsHexagonDep.td:5388)
    hexagon_V6_vsubh_dv_128B,                  // llvm.hexagon.V6.vsubh.dv.128B (IntrinsicsHexagonDep.td:5391)
    hexagon_V6_vsubhnq,                        // llvm.hexagon.V6.vsubhnq (IntrinsicsHexagonDep.td:5394)
    hexagon_V6_vsubhnq_128B,                   // llvm.hexagon.V6.vsubhnq.128B (IntrinsicsHexagonDep.td:5397)
    hexagon_V6_vsubhq,                         // llvm.hexagon.V6.vsubhq (IntrinsicsHexagonDep.td:5400)
    hexagon_V6_vsubhq_128B,                    // llvm.hexagon.V6.vsubhq.128B (IntrinsicsHexagonDep.td:5403)
    hexagon_V6_vsubhsat,                       // llvm.hexagon.V6.vsubhsat (IntrinsicsHexagonDep.td:5406)
    hexagon_V6_vsubhsat_128B,                  // llvm.hexagon.V6.vsubhsat.128B (IntrinsicsHexagonDep.td:5409)
    hexagon_V6_vsubhsat_dv,                    // llvm.hexagon.V6.vsubhsat.dv (IntrinsicsHexagonDep.td:5412)
    hexagon_V6_vsubhsat_dv_128B,               // llvm.hexagon.V6.vsubhsat.dv.128B (IntrinsicsHexagonDep.td:5415)
    hexagon_V6_vsubhw,                         // llvm.hexagon.V6.vsubhw (IntrinsicsHexagonDep.td:5418)
    hexagon_V6_vsubhw_128B,                    // llvm.hexagon.V6.vsubhw.128B (IntrinsicsHexagonDep.td:5421)
    hexagon_V6_vsububh,                        // llvm.hexagon.V6.vsububh (IntrinsicsHexagonDep.td:5424)
    hexagon_V6_vsububh_128B,                   // llvm.hexagon.V6.vsububh.128B (IntrinsicsHexagonDep.td:5427)
    hexagon_V6_vsububsat,                      // llvm.hexagon.V6.vsububsat (IntrinsicsHexagonDep.td:5430)
    hexagon_V6_vsububsat_128B,                 // llvm.hexagon.V6.vsububsat.128B (IntrinsicsHexagonDep.td:5433)
    hexagon_V6_vsububsat_dv,                   // llvm.hexagon.V6.vsububsat.dv (IntrinsicsHexagonDep.td:5436)
    hexagon_V6_vsububsat_dv_128B,              // llvm.hexagon.V6.vsububsat.dv.128B (IntrinsicsHexagonDep.td:5439)
    hexagon_V6_vsubububb_sat,                  // llvm.hexagon.V6.vsubububb.sat (IntrinsicsHexagonDep.td:5858)
    hexagon_V6_vsubububb_sat_128B,             // llvm.hexagon.V6.vsubububb.sat.128B (IntrinsicsHexagonDep.td:5861)
    hexagon_V6_vsubuhsat,                      // llvm.hexagon.V6.vsubuhsat (IntrinsicsHexagonDep.td:5442)
    hexagon_V6_vsubuhsat_128B,                 // llvm.hexagon.V6.vsubuhsat.128B (IntrinsicsHexagonDep.td:5445)
    hexagon_V6_vsubuhsat_dv,                   // llvm.hexagon.V6.vsubuhsat.dv (IntrinsicsHexagonDep.td:5448)
    hexagon_V6_vsubuhsat_dv_128B,              // llvm.hexagon.V6.vsubuhsat.dv.128B (IntrinsicsHexagonDep.td:5451)
    hexagon_V6_vsubuhw,                        // llvm.hexagon.V6.vsubuhw (IntrinsicsHexagonDep.td:5454)
    hexagon_V6_vsubuhw_128B,                   // llvm.hexagon.V6.vsubuhw.128B (IntrinsicsHexagonDep.td:5457)
    hexagon_V6_vsubuwsat,                      // llvm.hexagon.V6.vsubuwsat (IntrinsicsHexagonDep.td:5864)
    hexagon_V6_vsubuwsat_128B,                 // llvm.hexagon.V6.vsubuwsat.128B (IntrinsicsHexagonDep.td:5867)
    hexagon_V6_vsubuwsat_dv,                   // llvm.hexagon.V6.vsubuwsat.dv (IntrinsicsHexagonDep.td:5870)
    hexagon_V6_vsubuwsat_dv_128B,              // llvm.hexagon.V6.vsubuwsat.dv.128B (IntrinsicsHexagonDep.td:5873)
    hexagon_V6_vsubw,                          // llvm.hexagon.V6.vsubw (IntrinsicsHexagonDep.td:5460)
    hexagon_V6_vsubw_128B,                     // llvm.hexagon.V6.vsubw.128B (IntrinsicsHexagonDep.td:5463)
    hexagon_V6_vsubw_dv,                       // llvm.hexagon.V6.vsubw.dv (IntrinsicsHexagonDep.td:5466)
    hexagon_V6_vsubw_dv_128B,                  // llvm.hexagon.V6.vsubw.dv.128B (IntrinsicsHexagonDep.td:5469)
    hexagon_V6_vsubwnq,                        // llvm.hexagon.V6.vsubwnq (IntrinsicsHexagonDep.td:5472)
    hexagon_V6_vsubwnq_128B,                   // llvm.hexagon.V6.vsubwnq.128B (IntrinsicsHexagonDep.td:5475)
    hexagon_V6_vsubwq,                         // llvm.hexagon.V6.vsubwq (IntrinsicsHexagonDep.td:5478)
    hexagon_V6_vsubwq_128B,                    // llvm.hexagon.V6.vsubwq.128B (IntrinsicsHexagonDep.td:5481)
    hexagon_V6_vsubwsat,                       // llvm.hexagon.V6.vsubwsat (IntrinsicsHexagonDep.td:5484)
    hexagon_V6_vsubwsat_128B,                  // llvm.hexagon.V6.vsubwsat.128B (IntrinsicsHexagonDep.td:5487)
    hexagon_V6_vsubwsat_dv,                    // llvm.hexagon.V6.vsubwsat.dv (IntrinsicsHexagonDep.td:5490)
    hexagon_V6_vsubwsat_dv_128B,               // llvm.hexagon.V6.vsubwsat.dv.128B (IntrinsicsHexagonDep.td:5493)
    hexagon_V6_vswap,                          // llvm.hexagon.V6.vswap (IntrinsicsHexagonDep.td:5496)
    hexagon_V6_vswap_128B,                     // llvm.hexagon.V6.vswap.128B (IntrinsicsHexagonDep.td:5499)
    hexagon_V6_vtmpyb,                         // llvm.hexagon.V6.vtmpyb (IntrinsicsHexagonDep.td:5502)
    hexagon_V6_vtmpyb_128B,                    // llvm.hexagon.V6.vtmpyb.128B (IntrinsicsHexagonDep.td:5505)
    hexagon_V6_vtmpyb_acc,                     // llvm.hexagon.V6.vtmpyb.acc (IntrinsicsHexagonDep.td:5508)
    hexagon_V6_vtmpyb_acc_128B,                // llvm.hexagon.V6.vtmpyb.acc.128B (IntrinsicsHexagonDep.td:5511)
    hexagon_V6_vtmpybus,                       // llvm.hexagon.V6.vtmpybus (IntrinsicsHexagonDep.td:5514)
    hexagon_V6_vtmpybus_128B,                  // llvm.hexagon.V6.vtmpybus.128B (IntrinsicsHexagonDep.td:5517)
    hexagon_V6_vtmpybus_acc,                   // llvm.hexagon.V6.vtmpybus.acc (IntrinsicsHexagonDep.td:5520)
    hexagon_V6_vtmpybus_acc_128B,              // llvm.hexagon.V6.vtmpybus.acc.128B (IntrinsicsHexagonDep.td:5523)
    hexagon_V6_vtmpyhb,                        // llvm.hexagon.V6.vtmpyhb (IntrinsicsHexagonDep.td:5526)
    hexagon_V6_vtmpyhb_128B,                   // llvm.hexagon.V6.vtmpyhb.128B (IntrinsicsHexagonDep.td:5529)
    hexagon_V6_vtmpyhb_acc,                    // llvm.hexagon.V6.vtmpyhb.acc (IntrinsicsHexagonDep.td:5532)
    hexagon_V6_vtmpyhb_acc_128B,               // llvm.hexagon.V6.vtmpyhb.acc.128B (IntrinsicsHexagonDep.td:5535)
    hexagon_V6_vunpackb,                       // llvm.hexagon.V6.vunpackb (IntrinsicsHexagonDep.td:5538)
    hexagon_V6_vunpackb_128B,                  // llvm.hexagon.V6.vunpackb.128B (IntrinsicsHexagonDep.td:5541)
    hexagon_V6_vunpackh,                       // llvm.hexagon.V6.vunpackh (IntrinsicsHexagonDep.td:5544)
    hexagon_V6_vunpackh_128B,                  // llvm.hexagon.V6.vunpackh.128B (IntrinsicsHexagonDep.td:5547)
    hexagon_V6_vunpackob,                      // llvm.hexagon.V6.vunpackob (IntrinsicsHexagonDep.td:5550)
    hexagon_V6_vunpackob_128B,                 // llvm.hexagon.V6.vunpackob.128B (IntrinsicsHexagonDep.td:5553)
    hexagon_V6_vunpackoh,                      // llvm.hexagon.V6.vunpackoh (IntrinsicsHexagonDep.td:5556)
    hexagon_V6_vunpackoh_128B,                 // llvm.hexagon.V6.vunpackoh.128B (IntrinsicsHexagonDep.td:5559)
    hexagon_V6_vunpackub,                      // llvm.hexagon.V6.vunpackub (IntrinsicsHexagonDep.td:5562)
    hexagon_V6_vunpackub_128B,                 // llvm.hexagon.V6.vunpackub.128B (IntrinsicsHexagonDep.td:5565)
    hexagon_V6_vunpackuh,                      // llvm.hexagon.V6.vunpackuh (IntrinsicsHexagonDep.td:5568)
    hexagon_V6_vunpackuh_128B,                 // llvm.hexagon.V6.vunpackuh.128B (IntrinsicsHexagonDep.td:5571)
    hexagon_V6_vxor,                           // llvm.hexagon.V6.vxor (IntrinsicsHexagonDep.td:5574)
    hexagon_V6_vxor_128B,                      // llvm.hexagon.V6.vxor.128B (IntrinsicsHexagonDep.td:5577)
    hexagon_V6_vzb,                            // llvm.hexagon.V6.vzb (IntrinsicsHexagonDep.td:5580)
    hexagon_V6_vzb_128B,                       // llvm.hexagon.V6.vzb.128B (IntrinsicsHexagonDep.td:5583)
    hexagon_V6_vzh,                            // llvm.hexagon.V6.vzh (IntrinsicsHexagonDep.td:5586)
    hexagon_V6_vzh_128B,                       // llvm.hexagon.V6.vzh.128B (IntrinsicsHexagonDep.td:5589)
    hexagon_Y2_dccleana,                       // llvm.hexagon.Y2.dccleana (IntrinsicsHexagonDep.td:3577)
    hexagon_Y2_dccleaninva,                    // llvm.hexagon.Y2.dccleaninva (IntrinsicsHexagonDep.td:3580)
    hexagon_Y2_dcfetch,                        // llvm.hexagon.Y2.dcfetch (IntrinsicsHexagonDep.td:3583)
    hexagon_Y2_dcinva,                         // llvm.hexagon.Y2.dcinva (IntrinsicsHexagonDep.td:3586)
    hexagon_Y2_dczeroa,                        // llvm.hexagon.Y2.dczeroa (IntrinsicsHexagonDep.td:3589)
    hexagon_Y4_l2fetch,                        // llvm.hexagon.Y4.l2fetch (IntrinsicsHexagonDep.td:3592)
    hexagon_Y5_l2fetch,                        // llvm.hexagon.Y5.l2fetch (IntrinsicsHexagonDep.td:3595)
    hexagon_Y6_dmlink,                         // llvm.hexagon.Y6.dmlink (IntrinsicsHexagonDep.td:3760)
    hexagon_Y6_dmpause,                        // llvm.hexagon.Y6.dmpause (IntrinsicsHexagonDep.td:3763)
    hexagon_Y6_dmpoll,                         // llvm.hexagon.Y6.dmpoll (IntrinsicsHexagonDep.td:3766)
    hexagon_Y6_dmresume,                       // llvm.hexagon.Y6.dmresume (IntrinsicsHexagonDep.td:3769)
    hexagon_Y6_dmstart,                        // llvm.hexagon.Y6.dmstart (IntrinsicsHexagonDep.td:3772)
    hexagon_Y6_dmwait,                         // llvm.hexagon.Y6.dmwait (IntrinsicsHexagonDep.td:3775)
    hexagon_circ_ldb,                          // llvm.hexagon.circ.ldb (IntrinsicsHexagon.td:91)
    hexagon_circ_ldd,                          // llvm.hexagon.circ.ldd (IntrinsicsHexagon.td:71)
    hexagon_circ_ldh,                          // llvm.hexagon.circ.ldh (IntrinsicsHexagon.td:81)
    hexagon_circ_ldub,                         // llvm.hexagon.circ.ldub (IntrinsicsHexagon.td:96)
    hexagon_circ_lduh,                         // llvm.hexagon.circ.lduh (IntrinsicsHexagon.td:86)
    hexagon_circ_ldw,                          // llvm.hexagon.circ.ldw (IntrinsicsHexagon.td:76)
    hexagon_circ_stb,                          // llvm.hexagon.circ.stb (IntrinsicsHexagon.td:122)
    hexagon_circ_std,                          // llvm.hexagon.circ.std (IntrinsicsHexagon.td:102)
    hexagon_circ_sth,                          // llvm.hexagon.circ.sth (IntrinsicsHexagon.td:112)
    hexagon_circ_sthhi,                        // llvm.hexagon.circ.sthhi (IntrinsicsHexagon.td:117)
    hexagon_circ_stw,                          // llvm.hexagon.circ.stw (IntrinsicsHexagon.td:107)
    hexagon_instrprof_custom,                  // llvm.hexagon.instrprof.custom (IntrinsicsHexagon.td:443)
    hexagon_prefetch,                          // llvm.hexagon.prefetch (IntrinsicsHexagon.td:125)
    hexagon_vmemcpy,                           // llvm.hexagon.vmemcpy (IntrinsicsHexagon.td:151)
    hexagon_vmemset,                           // llvm.hexagon.vmemset (IntrinsicsHexagon.td:155)
}; // enum
} // namespace llvm::Intrinsic
#endif

